;+
; estimate of memory used by a wavelet transform. The estimated memory
; use is 36.26*n_elements(transformed_data.y). The factor of 36 comes
; from testing different transforms for different types of data, for
; fgm (FGH and FGS) data, 2009-01-14, for ESA L2 density data
; 2007-07-07, and for GMAG data for both of those days. Note that this
; is currently only useful for default inputs.
;-
Function wv_memory_test, t      ;t is the input time array
  n = n_elements(t)
  dt = mean(t[1:*]-t)           ;default is to use mean value
;Hacked from wavelet2.pro -- these are defaults different from wavelet.pro
  w0 = 2.*!pi
  dj = 1/8.*(2.*!pi/w0)
  prange = [2.*dt, 0.05*n*dt] ; default range = nyquist period - 5% of time period
  srange = (2.*dt > prange < n*dt) * (w0+sqrt(2+w0^2))/4/!pi
  srange = (prange) * (w0+sqrt(2+w0^2))/4/!pi ;srange are the scales of the wavelets
  jv = FIX((ALOG(srange[1]/srange[0])/ALOG(2))/dj);jv+1 is the number of wavelets used
;The memory used in bytes is approximately 36 times the number of
;elements in the final product
  Return, 36.26*float(n)*float(jv+1)
End
;+
;NAME:
; thm_ui_loaded_data::dproc
;PURPOSE:
; extracts tplot variables from active data, performs data processing
; tasks, creates new variables, optionally sets those variables to
; active variables
;CALLING SEQUENCE:
; success = loaded_data_obj -> dproc(dp_task, dp_pars, out_string = $
;           out_string, names_out=names_out, no_setactive=no_setactive)
;INPUT:
; dp_task = a string variable specifying the task to be carried
;           out. The options are ['subavg', 'submed', 'smooth',
;           'blkavg','clip','deflag','degap','spike','deriv',
;           'pwrspc','wave','hpfilt']
; dp_pars = an anonymous structure containing the input parameters for
; the task, this will be unpacked in this routine and the parameters
; are passed through. Note that, since this is only called from the
; thm_GUI_new routine, there is no error checking for
; content, it is expected that the calling routine passes through the
; proper parameters in each case.
;OUTPUT:
; success = a byte, 0b if the process was unsuccessful or cancelled,
;           1b if the process was completed
;KEYWORDS:
; out_string = an output string, contains either an error message,
;           or names of tplot variables that have been creeated
; names_out = the tplot names of the created data variables
; no_setactive = if set, the new variables will no be set to active at
;                the end of the process.
; hwin, sbar = history window and status bar objects for updates
;HISTORY:
; 16-oct-2008, jmm, jimm@ssl.berkeley.edu
; switched output from message to byte, 29-oct-2008,jmm
; 12-Dec-2008,prc Fixed bug where dproc was not reading data stored in
; loaded data,but instead was reading non-gui-data.
; Fixed bug where data produced by dproc was not inheriting any meta-data.
; 23-jan-2009, jmm, deletes any tplot variables that are created
;                   during processing, added catch, so that deletion
;                   of tplot variables is done if an error bonks a
;                   process.
; 10-Feb-2009, jmm, Added hwin, sbar keywords
;
;$LastChangedBy: pcruce $
;$LastChangedDate: 2009-03-26 10:41:56 -0700 (Thu, 26 Mar 2009) $
;$LastChangedRevision: 5428 $
;$URL: svn+ssh://thmsvn@ambrosia.ssl.berkeley.edu/repos/thmsoc/tags/tdas_5_00/idl/themis/thm_ui_new/objects/thm_ui_loaded_data__dproc.pro $
Function thm_ui_loaded_data::dproc, dp_task, dp_pars, names_out = names_out, $
                           out_string = out_string, no_setactive = no_setactive, $
                           hwin = hwin, sbar = sbar, _extra = _extra

err = 0
catch, err
If(err Ne 0) Then Begin
    catch, /cancel
    Help, /Last_Message, Output=err_msg
    If(is_string(out_string)) Then out_string = [out_string, err_msg] $
    Else out_string = err_msg
    ok = error_message(traceback = 1, /noname, title = 'Error in Data Processing: ')
    Goto, return_sequence
Endif

;Initialize output
out_string = ''
names_out = ''
otp = 0b
;First get the active data names:
active_v = self -> getactive(/parent) ;all you want is parents
If(is_string(active_v) Eq 0) Then Begin
    out_string = 'No active data, Returning'
    Return, otp
Endif
nav = n_elements(active_v)
dpt = strtrim(strlowcase(dp_task), 2)
;keep track of tnames created in process
tnames_in = tnames()
For j = 0, nav-1 Do Begin
    varname = active_v[j]
;Get the data -- first check to see if there are more than 2D. Most of
;                these routines will not work for more than 2d data.

    ;This call to getTvarObject is important for proper function of
    ;the gui. Do not remove.
    ;#1 It reads data from loadedData, not from tplot variables that may be deleted.
    ;#2 It stores all the associated metadata with the object, which is needed to properly classify data.
    tvarObject = self->getTvarObject(varname)
    get_data, varname, data = d
    If(is_struct(d)) Then Begin
        sy = size(d.y, /n_dim)
        If(sy Gt 2) And (dpt Ne 'subavg') And (dpt Ne 'submed') Then Begin
            out_string = [out_string, varname+': data has too many dimensions']
            yes_data = 0b
        Endif Else yes_data = 1b
;Test for spectrogram--if there are more than 6 traces, you'll not do
;power spectra or wavelets
        If(dpt Eq 'pwrspc' Or dpt Eq 'wave') Then Begin
            If(sy Ge 2) && (n_elements(d.y[0, *]) Gt 6) Then Begin
                out_string = [out_string, varname+': data is a spectrogram and cannot be processed']
                yes_data = 0b
            Endif
        Endif
    Endif Else Begin
        out_string = [out_string, varname+': has no data']
        yes_data = 0b
    Endelse

    If(yes_data) Then Begin
        nn0 = varname
;Now process the data
        Case dpt Of
            'subavg': Begin
                tsub_average, varname, nn, new_name = nn0+'-d'
                nn = tnames(nn0+'-d')
            End
            'submed': Begin
                tsub_average, varname, nn, new_name = nn0+'-m', /median
                nn = tnames(nn0+'-m')
            End
            'deriv': Begin
                deriv_data, varname, newname = nn0+'-ddt'
                nn = tnames(nn0+'-ddt')
            End
            'spike': Begin
                clean_spikes, varname, new_name = nn0+'-despiked'
                nn = tnames(nn0+'-despiked')
            End
            'smooth': Begin
                dt = dp_pars.dt
                If(dt Lt 1.0) Then dtchar = strcompress(/remove_all, dt) $
                Else dtchar = strcompress(/remove_all, fix(dt))
                tsmooth_in_time, varname, dt, newname = nn0+'-sm_'+dtchar
                nn = tnames(nn0+'-sm_'+dtchar)
            End
            'blkavg': Begin
;need another sanity test
                get_data, nn0, t
                av_dt = median(t[1:*]-t)
                If(av_dt Gt dp_pars.dt) Then Begin
                    lbl = ['Note that the median value of the time resolution for '+nn0+' is:'+strcompress(string(av_dt))+' sec.', $
                           'The value that you have chosen for the averaging time resolution is smaller: '+$
                           strcompress(string(dp_pars.dt))+' sec.', $
                           'This will have non-intuitive and possibly non-plottable results. Do you want to continue?']
                    ok = yesno_widget_fn('THEMIS GUI BLK_AVG TEST', label = lbl)
                Endif Else ok = 1b
                If(ok) Then Begin
                    dt = dp_pars.dt
                    If(dt Lt 1.0) Then dtchar = strcompress(/remove_all, dt) $
                    Else dtchar = strcompress(/remove_all, fix(dt))
                    avg_data, varname, dt, newname = nn0+'-av_'+dtchar
                    nn = tnames(nn0+'-av_'+dtchar)
                Endif Else Begin
                    out_string = 'Block Average process for: '+nn0+' cancelled. '
                Endelse
            End
            'clip': Begin
                tclip, varname, dp_pars.amin, dp_pars.amax, newname = nn0+'-clip'
                nn = tnames(nn0+'-clip')
            End
            'deflag': Begin
                tdeflag, varname, dp_pars.method, $
                  newname = nn0+'-deflag'+'-'+dp_pars.method
                nn = tnames(nn0+'-deflag'+'-'+dp_pars.method)
            End
            'degap': Begin
;need another sanity test
                get_data, nn0, t
                av_dt = median(t[1:*]-t)
                If(av_dt Gt dp_pars.degap_dt) Then Begin
                    lbl = ['Note that the median value of the time resolution for '+nn0+' is:'+strcompress(string(av_dt))+' sec.', $
                           'The value that you have chosen for the degap time resolution is smaller: '+$
                           strcompress(string(dp_pars.degap_dt))+' sec.', $
                           'This will have non-intuitive and possibly non-plottable results. Do you want to continue?']
                    ok = yesno_widget_fn('THEMIS GUI DEGAP TEST', label = lbl)
                Endif Else ok = 1b
                If(ok) Then Begin
                    tdegap, varname, dt = dp_pars.degap_dt, margin = dp_pars.margin, $
                      maxgap = dp_pars.maxgap, newname = nn0+'-degap'
                    nn = tnames(nn0+'-degap')
                Endif Else Begin
                    out_string = 'Degap process for: '+nn0+' cancelled. '
                Endelse
            End
            'pwrspc': Begin
                thm_ui_pwrspc, nn0, nn, /dynamic
                nn = tnames(nn0+'_?_dpwrspc')
                store_data, tnames(nn0), /delete
                store_data, tnames(nn0+'_?'), /delete
            End
            'wave': Begin
                copy_data, varname, nn0
;Test for memory
                get_data, nn0, t
                sstx = where(t Ge dp_pars.trange[0] And $
                             t Lt dp_pars.trange[1], nsstx)
                If(nsstx Eq 0) Then Begin
                  out_string = 'No Data In Time Range'
                  Return, -1
                Endif Else Begin
                  t = t[sstx]
                  memtest = (wv_memory_test(temporary(t)))/1.0e6
                  mem_av = get_max_memblock2()

                  if double(memtest)/mem_av gt .01 then begin       
                    lbl = ['Estimated Memory usage for Wavelet for '+nn0+' is: '+string(memtest, '(f10.2)')+' Mbytes, ', $
                         'You have an estimated '+string(mem_av, '(f10.2)')+' Mbytes, Continue?']
                    ok = yesno_widget_fn('THEMIS GUI WAVELET MEMORY CHECK', label = lbl,/center)
                  endif else begin
                    ok = 1
                  endelse
                  
                  If(ok) Then Begin
                    If(obj_valid(hwin)) Then hwin -> update, 'Processing Wavelet for: '+nn0
                    If(obj_valid(sbar)) Then sbar -> update, 'Processing Wavelet for: '+nn0
                    thm_ui_wavelet, nn0, nn, dp_pars.trange
                    nn = tnames(nn0+'_?_wv_pow')
                    store_data, tnames(nn0+'_?'), /delete
                  Endif Else Begin
                    out_string = 'Wavelet process for: '+nn0+' cancelled. '
                  Endelse
                  store_data, tnames(nn0), /delete
                Endelse
            End
            'hpfilt': Begin
                dt = dp_pars.dt
                If(dt Lt 1.0) Then dtchar = strcompress(/remove_all, dt) $
                Else dtchar = strcompress(/remove_all, fix(dt))
                thigh_pass_filter, varname, dt, newname = nn0+'-hpf'+dtchar
                nn = tnames(nn0+'-hpf'+dtchar)
            End
        Endcase
    Endif Else nn = ''
;varname is no longer needed ?
;    store_data, varname, /delete
    If(is_string(nn)) Then Begin
;Add the new variables into the loaded_data object, this will create a
;new group
        For k = 0, n_elements(nn)-1 Do Begin
            If(~otp) Then Begin ;only need 1 success for otp to be set to 1
                get_data, nn[k], data = dtest
                If(is_struct(temporary(dtest))) Then otp = 1b
            Endif
            ;The following 3 lines are important for ensuring that metadata
            ;associated with the old variables is also associated with the
            ;new variables.  Do not remove.

            if dpt eq 'wave' || dpt eq 'pwrspc' then begin
              isSpect = 1
            endif else begin
              tvarObject->getProperty,isSpect=isSpect
            endelse

            newObject = tvarObject->copy()
            newObject->setProperty,name=nn[k],isSpect=isSpect
            if self->addTvarObject(newObject,added_name=added_name) then begin
              out_string = [out_string, 'Added variable: '+nn[k]]
            endif else begin
              out_string = [out_string, 'Failed to add variable: '+nn[k]]
            endelse
        Endfor
        names_out = [names_out, nn]
    Endif Else Begin
        out_string = [out_string, varname+' Not processed']
    Endelse
    heap_gc                     ;clean-up memory
Endfor
;Reset active data to new variables, if there are any
If(otp) Then Begin
    If(is_string(names_out, names_out_ok)) Then Begin
        names_out = names_out_ok
        If(~keyword_set(no_setactive)) Then Begin
            self -> clearallactive
            For j = 0, n_elements(names_out)-1 Do Begin
                self -> setactive, names_out[j]
            Endfor
        Endif
    Endif
Endif

return_sequence:
;Remove nulls from output string
If(is_string(out_string, out_string_ok)) Then out_string = out_string_ok

;dump any tplot variables that you didn't start with
tnames_out = tnames()
If(is_string(tnames_out)) Then Begin
    If(is_string(tnames_in)) Then Begin
        For j = 0, n_elements(tnames_out)-1 Do Begin
            xx = where(tnames_in Eq tnames_out[j], nxx)
            If(nxx Eq 0) Then store_data, tnames_out[j], /delete
        Endfor
    Endif Else del_data, '*'    ;no variables when we started, so delete all
Endif

Return, otp
End

