;+ 
;NAME: 
; thm_ui_message_bar__define
;
;PURPOSE:
; This is a text bar object used to display textual information for the user
; (such as a status bar for current states, message bar, or informational bar)
;
;CALLING SEQUENCE:
; To Create:    myStatusBar = Obj_New("THM_UI_MESSAGE_BAR", myWidgetBase)
; To Use:       myStatusBar->Update, 'This is a test'
; Or:           result = myStatusBar->GetState()
;
;INPUT:
; parent:       id for the parent widget (must be a base)
;
;KEYWORDS:
; name:   optional name
; state:  set this to one to display
; value:  text to be displayed in the bar 
; xSize:  size of bar in x direction
; ySize:  size of bar in y direction
; debug:  set this value to one for debugging
;
;OUTPUT:
; message bar object reference
;
;METHODS:
; Draw         creates/displays the bar (automatically called by INIT)
; Delete       removes bar from display (object persists)
; Update       updates bar with new message
; SetProperty  procedure to set keywords 
; GetProperty  procedure to get keywords 
; GetState     returns the current state of the bar (on/off) (this is a function)

;HISTORY:
;
;$LastChangedBy: pcruce $
;$LastChangedDate: 2008-11-05 13:25:13 -0800 (Wed, 05 Nov 2008) $
;$LastChangedRevision: 3926 $
;$URL: svn+ssh://thmsvn@ambrosia.ssl.berkeley.edu/repos/thmsoc/tags/tdas_5_00/idl/themis/thm_ui_new/objects/thm_ui_message_bar__define.pro $
;-----------------------------------------------------------------------------------


PRO THM_UI_MESSAGE_BAR::Draw
IF self.id EQ 0 THEN BEGIN
   self.id = WIDGET_TEXT(self.parent, value=self.value, xsize = self.xsize, ysize = self.ysize,/editable)
   self.state = 1
ENDIF
END ;--------------------------------------------------------------------------------



PRO THM_UI_MESSAGE_BAR::Delete
IF self.id NE 0 THEN BEGIN
   WIDGET_CONTROL, self.id, /Destroy
   self.state = 0
   self.id = 0
ENDIF
END ;--------------------------------------------------------------------------------



PRO THM_UI_MESSAGE_BAR::Update, value
IF self.id NE 0 THEN BEGIN
   WIDGET_CONTROL, self.id, SET_VALUE=value
   self.value=value
ENDIF 
END ;--------------------------------------------------------------------------------



FUNCTION THM_UI_MESSAGE_BAR::GetState
RETURN, self.state
END ;--------------------------------------------------------------------------------



PRO THM_UI_MESSAGE_BAR::SetProperty,  $ ; The property set method for the object
            XSize=xsize,              $ ; size of bar in x direction
            YSize=ysize,              $ ; size of bar in y direction
            Refresh=refresh             ; set this to one to redisplay  

   Catch, theError
   IF theError NE 0 THEN BEGIN
      Catch, /Cancel
      ok = Error_Message(Traceback=Keyword_Set(self.debug))
      RETURN
   ENDIF

   ; Check for undefined variables.

   IF N_Elements(xsize) NE 0 THEN self.xsize = xsize
   IF N_Elements(ysize) NE 0 THEN self.ysize = ysize
   
   IF Keyword_Set(refresh) THEN BEGIN
      self->Delete
      self->Draw
   ENDIF

END ;--------------------------------------------------------------------------------



PRO THM_UI_MESSAGE_BAR::GetProperty,  $
            id=id,                   $ ; widget id
            Name=name,               $ ; optional name of bar
            Value=value,             $ ; text to be displayed in the bar 
            State=state,             $ ; flag to indicate whether bar is diplayed
            XSize=xsize,             $ ; size of bar in x direction
            YSize=ysize                ; size of bar in y direction

   Catch, theError
   IF theError NE 0 THEN BEGIN
      Catch, /Cancel
      ok = Error_Message(Traceback=Keyword_Set(self.debug))
      RETURN
   ENDIF

   IF arg_present(id) then id = self.id
   IF Arg_Present(name) THEN name = self.name
   IF Arg_Present(value) THEN value = self.value
   IF Arg_Present(state) THEN state = self.state
   IF Arg_Present(xsize) THEN xsize = self.xsize
   IF Arg_Present(ysize) THEN ysize = self.ysize

END ;--------------------------------------------------------------------------------



;PRO THM_UI_MESSAGE_BAR::Cleanup 
;   nothing to clean up (yet)
;END ;--------------------------------------------------------------------------------



FUNCTION THM_UI_MESSAGE_BAR::Init,    $ ; The INIT method of the bar object.
            parent,                  $ ; id of parent, required
            Name=name,               $ ; optional name of bar
            Value=value,             $ ; text to be displayed in the bar 
            State=state,             $ ; flag to indicate whether bar is diplayed
            XSize=xsize,             $ ; size of bar in x direction
            YSize=ysize,             $ ; size of bar in y direction
            Debug=debug                ; set to one for debugging

   Catch, theError
   IF theError NE 0 THEN BEGIN
      Catch, /Cancel
      ok = Error_Message(Traceback=Keyword_Set(debug))
      RETURN, 0
   ENDIF

   self.debug = Keyword_Set(debug)
 
   ; Check that all parameters have values
   
   IF N_Elements(name) EQ 0 THEN name=' '
   IF N_Elements(value) EQ 0 THEN value='Message Bar '
   IF N_Elements(state) EQ 0 THEN state=1
   IF N_Elements(xsize) EQ 0 THEN xsize = 10
   IF N_Elements(ysize) EQ 0 THEN ysize = 1

  ; Set all parameters
  
   self.parent = parent
   self.name = name
   self.value = value
   self.state = state
   self.xsize = xsize
   self.ysize = ysize

  ; If bar is displayed then create the widget
   
   IF self.state EQ 1 THEN self->Draw

   RETURN, 1
END ;--------------------------------------------------------------------------------



PRO THM_UI_MESSAGE_BAR__DEFINE

   struct = { THM_UI_MESSAGE_BAR,   $
              parent: 0L,           $ ; id for the parent widget (must be a base)
              id: 0L,               $ ; widget id for the bar
              name: ' ',            $ ; optional name
              state: 0,             $ ; on/off flag (for display)
              value: ' ',           $ ; text to be displayed in the bar 
              xSize: 0,             $ ; size of bar in x direction
              ySize: 0,             $ ; size of bar in y direction
              debug: 0              $ ; set this value to one for debugging

}

END
