;+ 
;NAME: 
; thm_ui_panel__define
;
;PURPOSE:  
; generic object for a panel
;
;CALLING SEQUENCE:
; panel = Obj_New("THM_UI_PANEL")
;
;INPUT:
; none
;
;ATTRIBUTES:

;traceSettings IDL_Container object storing trace settings for each set of data quantities to be plotted
;windowID    ID for parent window (defaults to -1)
;name        name for this panel
;id          unique identifier for this panel
;settings    property object for this panel
;xAxis       x axis properties object
;yAxis       y axis properties object
;zAxis       z axis properties object
;tracking    flag set if tracking is on
;isActive    flag set if panel is displayed
;syncflag    flag set if label and trace colors are synced
;variables   idl container of variable objects
;showvariables flag indicates whether variables should be displayed or not 
;markers     idl container of marker objects
;
;OUTPUT:
; panel object reference
;
;METHODS:
; GetProperty
; GetAll
; GetLayoutStructure
; SetLayoutStructure
; SetProperty
;
;HISTORY:
;
;$LastChangedBy: aaflores $
;$LastChangedDate: 2009-03-11 15:28:37 -0700 (Wed, 11 Mar 2009) $
;$LastChangedRevision: 5258 $
;$URL: svn+ssh://thmsvn@ambrosia.ssl.berkeley.edu/repos/thmsoc/tags/tdas_5_00/idl/themis/thm_ui_new/objects/thm_ui_panel__define.pro $
;-----------------------------------------------------------------------------------

FUNCTION THM_UI_PANEL::Copy
   out = Obj_New("THM_UI_PANEL", self.id,/nosave)
   Struct_Assign, self, out
   ; copy panel settings
   newSettings=Obj_New("THM_UI_PANEL_SETTINGS")
   IF Obj_Valid(self.Settings) THEN newSettings=self.settings->Copy() ELSE $
      newSettings=Obj_New()
   out->SetProperty, Settings=newSettings
   ; copy x axis
   newAxis=Obj_New("THM_UI_AXIS_SETTINGS")
   IF Obj_Valid(self.xAxis) THEN newAxis=self.xAxis->Copy() ELSE $
      newAxis=Obj_New()
   out->SetProperty, XAxis=newAxis
   ; copy y axis
   newAxis=Obj_New("THM_UI_AXIS_SETTINGS")
   IF Obj_Valid(self.yAxis) THEN newAxis=self.yAxis->Copy() ELSE $
      newAxis=Obj_New()
   out->SetProperty, YAxis=newAxis
   ; copy z axis
   newAxis=Obj_New("THM_UI_ZAXIS_SETTINGS")
   IF Obj_Valid(self.zAxis) THEN newAxis=self.zAxis->Copy() ELSE $
      newAxis=Obj_New()
   out->SetProperty, ZAxis=newAxis
   ; copy trace settings
   IF Obj_Valid(self.traceSettings) THEN BEGIN
     origSettings=self.traceSettings->Get(/all)  
     newSettings=Obj_New("IDL_Container")
     nSettings = N_Elements(origSettings)
     IF nSettings GT 0 THEN BEGIN
       FOR i=0, nSettings-1 DO BEGIN
         IF Obj_Valid(origSettings[i]) THEN BEGIN
            newSetting=origSettings[i]->Copy()
            newSettings->Add, newSetting
         ENDIF 
       ENDFOR
     ENDIF
   ENDIF
   
   out->SetProperty, TraceSettings=newSettings
   ; copy variables
   newVariables=Obj_New("IDL_Container")
   newVariable=Obj_New("THM_UI_VARIABLE")
   IF Obj_Valid(self.variables) THEN origVariables=self.variables->Get(/all)        
   nVariables = N_Elements(origVariables)
   IF nVariables GT 0 THEN BEGIN
      FOR i=0, nVariables-1 DO BEGIN
         IF Obj_Valid(origVariables[i]) THEN BEGIN
            newVariable=origVariables[i]->Copy()
            newVariables->Add, newVariable
         ENDIF 
      ENDFOR
   ENDIF
   out->SetProperty, Variables=newVariables
   ; copy markers
   newMarkers=Obj_New("IDL_Container")
   newMarker=Obj_New("THM_UI_MARKER")
   IF Obj_Valid(self.markers) THEN origMarkers=self.markers->Get(/all)        
   numMarkers = N_Elements(origMarkers)
   IF numMarkers GT 0 THEN BEGIN
      FOR i=0, numMarkers-1 DO BEGIN
         IF Obj_Valid(origMarkers[i]) THEN BEGIN
            newMarker=origMarkers[i]->Copy()
            newMarkers->Add, newMarker
         ENDIF 
      ENDFOR
   ENDIF
   out->SetProperty, Markers=newMarkers
   RETURN, out
END ;--------------------------------------------------------------------------------



PRO THM_UI_PANEL::SetProperty,  $
        traceSettings=traceSettings, $   ; IDL_Container object storing trace settings for each set of data quantities to be plotted
        WindowID=windowid,      $ ; ID for parent window (no longer required)
        Name=name,              $ ; name for this panel
        ID=id,                  $ ; unique identifier for this panel
        Settings=settings,      $ ; property object for this panel
        XAxis=xaxis,            $ ; x axis properties object
        YAxis=yaxis,            $ ; y axis properties object
        zAxis=zAxis,            $ ; z axis properties object
        Tracking=tracking,      $ ; flag set if tracking is on
        IsActive=isactive,      $ ; flag set if panel is displayed
        SyncFlag=syncflag,      $ ; flag set if labels and trace colors are synced
        Variables=variables,    $ ; idl container of variable objects
        showvariables=showvariables, $ ; flag indicates whether variables should be displayed or not 
        labelmargin=labelmargin,$ ; label margin for panel
        Markers=markers           ; idl container of variable objects


   Catch, theError
   IF theError NE 0 THEN BEGIN
      Catch, /Cancel
      ok = Error_Message(Traceback=1)
      RETURN
   ENDIF
   
   if obj_valid(traceSettings) && Size(tracesettings, /type) eq 11 then self.traceSettings = traceSettings
   
   IF N_Elements(windowid) NE 0 THEN self.windowID = windowid
   IF N_Elements(name) NE 0 THEN self.name = name
   IF N_Elements(id) NE 0 THEN self.id = id
   IF N_Elements(rownumber) NE 0 THEN self.rownumber = rownumber
   IF N_Elements(colnumber) NE 0 THEN self.colnumber = colnumber
   IF N_Elements(settings) NE 0 && Size(settings, /type) eq 11 THEN self.settings = settings
   IF N_Elements(xaxis) NE 0 && Size(xaxis, /type) EQ 11 THEN self.xAxis = xaxis
   IF N_Elements(yaxis) NE 0 && Size(yaxis, /type) EQ 11 THEN self.yAxis = yaxis
   IF N_Elements(zaxis) NE 0 && Size(zaxis, /type) EQ 11 THEN self.zAxis = zaxis
   IF N_Elements(tracking) NE 0 THEN self.tracking = tracking
   IF N_Elements(isactive) NE 0 THEN self.isActive = isactive
   IF N_Elements(syncflag) NE 0 THEN self.syncFlag = syncflag
   IF N_Elements(labelmargin) NE 0 THEN self.labelmargin = labelmargin
   IF Obj_Valid(variables) && obj_isa(variables,'IDL_Container') THEN self.variables = variables
   if n_elements(showvariables) ne 0 then self.showvariables = showvariables
   IF Obj_Valid(markers) && obj_isa(markers,'IDL_Container') THEN self.markers = markers
   
RETURN
END ;--------------------------------------------------------------------------------


;returns [xpos,ypos,xsize,ysize] in points
FUNCTION THM_UI_PANEL::GetPanelCoordinates

   ; get units and values for each coordinate, bottom, left, width, height
  
  coord = dblarr(4)
   
  self.settings->getProperty,lunit=unit,lvalue=value,left=flag
  coord[0] = (~flag || value eq -1)?-1:self.settings->convertunit(value,unit,0)
  
  self.settings->getProperty,bunit=unit,bvalue=value,bottom=flag
  coord[1] = (~flag || value eq -1)?-1:self.settings->convertunit(value,unit,0)
  
  self.settings->getProperty,wunit=unit,wvalue=value,width=flag
  coord[2] = (~flag || value eq -1)?-1:self.settings->convertunit(value,unit,0)

  self.settings->getProperty,hunit=unit,hvalue=value,height=flag
  coord[3] = (~flag || value eq -1)?-1:self.settings->convertunit(value,unit,0)

RETURN, coord 
END ;--------------------------------------------------------------------------------

;coord = [xpos,ypos,xsize,ysize] in points
pro THM_UI_PANEL::setPanelCoordinates,coord
 
  self.settings->getProperty,lunit=lunit
  lvalue = self.settings->convertunit(coord[0],0,lunit)
  self.settings->setProperty,lvalue=lvalue
  
  self.settings->getProperty,bunit=bunit
  bvalue = self.settings->convertunit(coord[1],0,bunit)
  self.settings->setProperty,bvalue=bvalue
  
  self.settings->getProperty,wunit=wunit
  wvalue = self.settings->convertunit(coord[2],0,wunit)
  self.settings->setProperty,wvalue=wvalue
  
  self.settings->getProperty,hunit=hunit
  hvalue = self.settings->convertunit(coord[3],0,hunit)
  self.settings->setProperty,hvalue=hvalue
 

END ;--------------------------------------------------------------------------------

;function constructs a panel name for display in droplists
function thm_ui_panel::constructPanelName

  compile_opt idl2

  name = 'Panel '
  
  name += strtrim(string(self.id + 1),2)

  self.settings->getProperty,row=row,col=col,titleObj=panelTitle
  
  panelTitle->getProperty, value=title    
  if strlen(title) gt 45 then title = strmid(title, 0, 45) + ' ...'
  
  name += ' (' + strtrim(string(row),2) + ', ' + strtrim(string(col),2) + ')  -  ' + $
           title
   
  return,name

end


;function constructs an array of panel's trace names for display in droplists
function thm_ui_panel::constructTraceNames

  compile_opt idl2
  
  if obj_valid(self.traceSettings) AND $
       ~array_equal((self.traceSettings->get(/all)), -1, /no_typeconv) then begin
  
    objs = self.traceSettings->get(/all)
    names = strarr(n_elements(objs))
    
      for i = 0,n_elements(objs)-1 do begin
      
        if obj_isa(objs[i],'thm_ui_line_settings') then begin
          objs[i]->getProperty,dataX=dataX,dataY=dataY
          
          names[i] = '  - ' + dataX + ' -vs- ' + dataY
        
        endif else if obj_isa(objs[i],'thm_ui_spectra_settings') then begin
        
          objs[i]->getProperty,dataX=dataX,dataY=dataY,dataZ=dataZ
          
          names[i] = '  - ' + dataX + ' -vs- ' + dataY + ' -vs- ' + dataZ 
           
        endif
      
      endfor
  endif
  
  return,names

end

PRO THM_UI_PANEL::GetProperty,  $
        traceSettings=traceSettings, $   ; IDL_Container object storing trace settings for each set of data quantities to be plotted
        WindowID=windowid,      $ ; ID for parent window
        Name=name,              $ ; name for this panel
        ID=id,                  $ ; unique identifier for this panel
        Settings=settings,      $ ; property object for this panel
        XAxis=xaxis,            $ ; x axis properties object
        YAxis=yaxis,            $ ; y axis properties object
        zAxis=zaxis,            $ ; z axis properties object
        Tracking=tracking,      $ ; flag set if tracking is on
        IsActive=isactive,      $ ; flag set if panel is displayed
        SyncFlag=syncflag,      $ ; flag set if labels and trace colors are synced
        Variables=variables,    $ ; idl container of variable objects
        showvariables=showvariables, $ ; flag indicates whether variables should be displayed or not 
        labelmargin=labelmargin,$ ; label margin for panel
        Markers=markers           ; idl container of marker objects


   Catch, theError
   IF theError NE 0 THEN BEGIN
      Catch, /Cancel
      ok = Error_Message(Traceback=1)
      RETURN
   ENDIF
   
   if arg_present(traceSettings) then traceSettings = self.traceSettings
   IF Arg_Present(windowid) THEN windowid = self.windowID
   IF Arg_Present(name) THEN name = self.name
   IF Arg_Present(id) THEN id = self.id
   IF Arg_Present(settings) THEN settings = self.settings
   IF Arg_Present(xaxis) THEN xaxis = self.xAxis
   IF Arg_Present(yaxis) THEN yaxis = self.yAxis
   IF Arg_Present(zaxis) THEN zaxis = self.zAxis
   IF Arg_Present(tracking) THEN tracking = self.tracking
   IF Arg_Present(isactive) THEN isactive = self.isActive
   IF Arg_Present(syncflag) THEN syncflag = self.syncFlag
   IF Arg_Present(variables) THEN variables = self.variables
   if arg_present(showvariables) then showvariables = self.showvariables
   IF Arg_Present(labelmargin) THEN labelmargin = self.labelmargin
   IF Arg_Present(markers) THEN markers = self.markers
   
RETURN
END ;--------------------------------------------------------------------------------

;this routine will update references to a data quantity
;This should be used if a name has changed while traces are
;already in existence .
pro thm_ui_panel::updatedatareference,oldnames,newnames

  compile_opt idl2
  
  ref_changed = 0
  
  self->getProperty,traceSettings=traceSettings,variables=variables
  
  if obj_valid(traceSettings) then begin
  
    traces = traceSettings->get(/all)
  
    if obj_valid(traces[0]) then begin
      for i = 0,n_elements(traces)-1 do begin
      
        traces[i]->updatedatareference,oldnames,newnames,changed=changed
        if changed then ref_changed = 1
        
      endfor
    endif
    
  endif
  
  if ref_changed then self->synclabelstolines
  
  if obj_valid(variables) then begin
  
    var_list = variables->get(/all)
    
    if obj_valid(var_list[0]) then begin
      for i = 0,n_elements(var_list)-1 do begin
      
        var_list[i]->updatedatareference,oldnames,newnames
        
      endfor
    endif
    
  endif
  
end

PRO THM_UI_PANEL::SyncLinesToLabels
   IF ~Obj_Valid(self.traceSettings) THEN RETURN
   settings = self.traceSettings->Get(/All)
   nLines = N_Elements(settings)
   
   IF ~Obj_Valid(self.yAxis) THEN RETURN
   self.yAxis->GetProperty, Labels=labels
   labels1 = labels->Get(/All)
   nLabels = N_Elements(labels1)
   IF nLabels LT 1 THEN RETURN
   IF ~Obj_Valid(labels1[0]) THEN RETURN
   IF nLines GT nLabels THEN nlines = nLabels
   IF nLines LT 1 THEN RETURN
 
   FOR i=0,nLines-1 DO BEGIN

      IF Obj_Isa(settings[i], 'THM_UI_LINE_SETTINGS') THEN BEGIN   ;is a line

         settings[i]->GetProperty, LineStyle=lineStyle
         IF Obj_Valid(lineStyle) THEN BEGIN
            labels1[i]->GetProperty, Color=color
            lineStyle->SetProperty, Color=color
        ENDIF
     ENDIF
      
       ;I'm not sure what this is supposed to do, but whatever it is, it doesn't work. 
       ;ELSE BEGIN  ; is a spectra
       ;IF ~Obj_Valid(self.zAxis) THEN RETURN
       ;self.zAxis->GetProperty, LabelTextObject=label
       ;IF ~Obj_Valid(label) THEN RETURN
       ;label->SetProperty, Color=color
       ; ENDELSE
   ENDFOR
END ;--------------------------------------------------------------------------------



PRO THM_UI_PANEL::SyncLabelsToLines
   IF ~Obj_Valid(self.traceSettings) THEN RETURN
   lineSettings = self.traceSettings->Get(/All)
   nLines = N_Elements(lineSettings)
   IF ~Obj_Valid(self.yAxis) THEN RETURN
   self.yAxis->GetProperty, Labels = labels
   if ~obj_valid(labels[0]) then return
   labels1 = labels->Get(/All)
   nLabels = N_Elements(labels1)
   IF nLabels LT 1 THEN RETURN
   IF ~Obj_Valid(labels1[0]) THEN RETURN
   IF nLabels GT nLines THEN nlabels = nLines
   IF nLabels LT 1 THEN RETURN
   FOR i=0,nLabels-1 DO BEGIN
   
     if obj_isa(linesettings[i],'thm_ui_spectra_settings') then begin
       labels1[i]->setProperty,color=[0,0,0]
     endif else begin
       lineSettings[i]->GetProperty, LineStyle=lineStyle
       lineSettings[i]->GetProperty, dataY=dataY
       IF Obj_Valid(lineStyle) THEN BEGIN
         lineStyle->GetProperty, Color=color
         labels1[i]->SetProperty, Color=color
       ENDIF
       if size(dataY,/type) eq 7 then labels1[i]->SetProperty,value=dataY
     endelse
     
   ENDFOR
END ;--------------------------------------------------------------------------------



FUNCTION THM_UI_PANEL::GetLayoutStructure
self.settings->GetProperty, Row=row, Col=col
RETURN, {layout, row:row, col:col, id:self.id}
END ;--------------------------------------------------------------------------------


pro THM_UI_PANEL::SetLayoutStructure, Row=row, Col=col
self.settings->SetProperty, Row=row, Col=col
END ;--------------------------------------------------------------------------------


FUNCTION THM_UI_PANEL::GetAll

   str = create_struct(name="THM_UI_PANEL")
   struct_assign,self,str
   
RETURN, str
END ;--------------------------------------------------------------------------------

PRO THM_UI_PANEL::SetAll, str

   struct_assign,str,self

RETURN
END ;--------------------------------------------------------------------------------

PRO THM_UI_PANEL::Save

  copy = self->copy()
  if ptr_valid(self.origsettings) then begin
    ptr_free,self.origsettings
  endif
  self.origsettings = ptr_new(copy->getall())

RETURN
END ;--------------------------------------------------------------------------------

PRO THM_UI_PANEL::Reset

   if ptr_valid(self.origsettings) then begin
      self->SetAll,*self.origsettings
      self->save
   endif

RETURN
END ;--------------------------------------------------------------------------------


;PRO THM_UI_PANEL::Cleanup 
;   Obj_Destroy, self.settings
;   Obj_Destroy, self.xAxis
;   Obj_Destroy, self.yAxis
;   Obj_Destroy, self.zAxis
;   Obj_Destroy, self.traceSettings
;   Obj_Destroy, self.variables 
;   Obj_Destroy, self.markers 
;RETURN    
;END ;--------------------------------------------------------------------------------


  
FUNCTION THM_UI_PANEL::Init,    $ ; The INIT method of the line style object
        id,                     $ ; numerical identifier for this panel     
        traceSettings=traceSettings, $ ; IDL_Container object storing trace settings for each set of data quantities to be plotted
        windowID=windowID,      $ ; ID for parent window (now optional, defaults to -1)
        Name=name,              $ ; name for this panel
        Settings=settings,      $ ; property object for this panel
        XAxis=xaxis,            $ ; x axis properties object
        YAxis=yaxis,            $ ; y axis properties object
        zAxis=zaxis,            $ ; z axis properties object
        Tracking=tracking,      $ ; flag set if tracking is on
        Variables=variables,    $ ; idl container of variable objects
        showvariables=showvariables, $ ; flag indicates whether variables should be displayed or not
        Markers=markers,        $ ; idl container of variable objects
        labelmargin=labelmargin,$ ; label margin for panel
        IsActive=isactive,      $ ; flag set if panel is displayed
        SyncFlag=syncflag,      $ ; flag set if labels and trace colors are synced
        Debug=debug,            $ ; flag to debug
        nosave=nosave             ; won't save a copy on startup 

   Catch, theError
   IF theError NE 0 THEN BEGIN
      Catch, /Cancel
      ok = Error_Message(Traceback=Keyword_Set(debug))
      RETURN, 0
   ENDIF
  
   ; Check that all parameters have values

   IF N_Elements(id) EQ 0 THEN id = -1
   IF N_Elements(windowid) EQ 0 THEN windowid = -1
   IF N_Elements(name) EQ 0 THEN name = 'Panel: ' + strtrim(id+1, 2) 
   IF N_Elements(labelmargin) EQ 0 THEN labelmargin = 0
   IF NOT Obj_Valid(settings) THEN settings = Obj_New('THM_UI_PANEL_SETTINGS')
   if ~obj_valid(xaxis) then xaxis = Obj_New()
   if ~obj_valid(yaxis) then yaxis = Obj_New()
   if ~obj_valid(zaxis) then zaxis = obj_new()
   if ~obj_valid(traceSettings) then traceSettings = obj_new('IDL_Container')
   IF N_Elements(tracking) EQ 0 THEN tracking = 1
   IF N_Elements(isactive) EQ 0 THEN isactive = 1
   IF N_Elements(syncflag) EQ 0 THEN syncflag = 1
   IF NOT Obj_Valid(variables) THEN variables = Obj_New('IDL_Container')
   IF NOT Obj_Valid(markers) THEN markers = Obj_New('IDL_Container')
   if n_elements(showvariables) eq 0 then showvariables = 1

  ; Set all parameters

   self.traceSettings = traceSettings
   self.windowID = windowid
   self.name = name
   self.id = id
   self.settings = settings
   self.xAxis = xaxis
   self.yAxis = yaxis
   self.zAxis = zaxis
   self.tracking = tracking
   self.isActive = isactive
   self.syncFlag = syncflag
   self.variables = variables
   self.markers = markers
   self.showvariables = showvariables
   self.labelmargin = labelmargin
   
   if ~keyword_set(nosave) then begin
      self->save
   endif
  
   RETURN, 1
END ;--------------------------------------------------------------------------------                 

PRO THM_UI_PANEL__DEFINE

  ;note that id is no longer unique, if we want to add code to make it unique, 
  ;I think we can use shmmap & shmvar, or get_var & set_var
   struct = { THM_UI_PANEL,           $

              name: '',               $ ; name for this panel              
              id: 0,                  $ ; numerical identifier for this panel
              windowID: 0,            $ ; ID for window this panel is displayed on
              settings: Obj_New(),    $ ; setting object for this panel           
              traceSettings:Obj_New(),$ ; IDL_Container object storing trace settings for each set of data quantities to be plotted
              xAxis: Obj_New(),       $ ; x axis properties object
              yAxis: Obj_New(),       $ ; y axis properties object
              zAxis: Obj_new(),       $ ; z axis properties object
              tracking: 0,            $ ; flag set if tracking is on
              isActive: 0,            $ ; flag set if panel is displayed
              variables: Obj_New(),   $ ; idl container of variables
              markers: Obj_New(),     $ ; idl container of markers
              showvariables:0,        $ ; flag indicates whether variables should be displayed or not
              labelmargin: 0,         $ ; label margin for panel
              syncFlag: 0,            $ ; flag indicating whether to sync labels and traces
              origsettings: ptr_new(),$ ; pointer to original settings in case of reset
              INHERITS THM_UI_READWRITE $ ; generalized read/write methods
                                     
}

END
