;+
;NAME:
; thm_ui_widget_tree__define.pro
;
;PURPOSE:
;  Object representing the loaded data tree compound widget.
;
;CALLING SEQUENCE:
;  wt = obj_new('thm_ui_widget_tree',parentid,uvalue,loadedData,xsize=200,ysize=200,mode=1)
;
;Examples:
; 1.
;   widget_control,event.id,get_value=val
;   print, val->getvalue()
;
; 2. obj = obj_new('thm_ui_widget_tree,tlb,'TREE',loadedData,uname='TREE_NAME')
;    id = widget_info(find_by_uname='TREE_NAME')
;    widget_control,id,get_value=val
;    print,val->getvalue()
;    val->setProperty,multi=0
;
; Attributes:
;
;  uvalue: the user value that will be returned by event handler
;  uname: the user name that can be used to identify the widget
;  xsize: the x axis length of the tree viewing area(scrollbars will be added automatically)
;  ysize: the y axis length of the tree viewing area(scrollbars will be added automatically)
;  mode: the visualization/selection mode(see NOTES below)
;  multi: whether multiple selections with ctrl/shift click are allowed
;  leafonly: 1: indicates selection may be only made at the lowest level of the tree
;            0: indicates selections may be done at any point in the tree
;
;NOTES:
;
;  There are 4-different selection modes for this widget.  Each selection
;  mode will be appropriate for different panels.
;  Mode 0: tplot-layout selection,
;          data-processing selection,
;          loaded-data-selection
;
;  Mode 1 : variable-selection,
;           x-layout selection,
;           y-lineplot-layout selection
;           line-trace selection.
;           save-data-as
;
;  Mode 2 : y-spectra-layout selection
;           z-spectra-layout selection
;           (possibly data-processing)
;
;  Mode 3 : Calculate Panel
;           Possibly future versions of data analysis
;
;  Mode Descriptions
;  Mode 0 is a tplot-like selection.  It provides access to
;  only the groupname and doesn't let the user drill down into internal
;  quantities.
;
;  When values are returned from mode 0, an array of pointers to
;  structs that store that names of the contained variables will be returned.
;  (or 0 on fail)
;
;  Mode 1 is a 1-D component selection.  It allows the user to
;  look at and select internal 1-d quantities,
;
;  When values are returned from mode 1, an array of strings will be
;  returned. (or 0 on fail)
;
;  Mode 2 is a 2-D component selection.  It allows the user to look at
;  and select any 2-d quantities.  In mode 2 yaxis-values will
;  not be grouped inside their containing group, instead they will be grouped
;  side-by-side with their containing group.
;
;  Values returned from mode 2 will be an array of strings of groupnames.
;  (or 0 on fail)
;
;  Mode 3 allows selection of either group quantities('tha_state_pos') or
;  components('tha_state_pos_x'), but does not allow branch selection.
;  (ie Cannot grab themis and get all quantities.)
;
;  Public Methods:
;  update: Call this routine after loaded data has been changed by
;          some other process
;  getProperty: Use this to get the current widget_tree settings
;  setProperty: Use this to change the current widget_tree settings
;  getValue(): Use this to return a list of the current selections.
;
;  The other methods should NOT be called by external code.  Making code
;  that uses them will make this code very difficult to maintain. So if
;  you need a feature, just request that it be added instead.
;
;
;HISTORY:
;$LastChangedBy: pcruce $
;$LastChangedDate: 2009-04-02 09:37:05 -0700 (Thu, 02 Apr 2009) $
;$LastChangedRevision: 5517 $
;$URL: svn+ssh://thmsvn@ambrosia.ssl.berkeley.edu/repos/thmsoc/tags/tdas_5_00/idl/themis/thm_ui_new/objects/thm_ui_widget_tree__define.pro $
;
;--------------------------------------------------------------------------------


;Public Method
;call this routine to update tree after changes have been made in:
;(1) The data objects(particularly: name,mission,instrument,observatory fields)
;(2) The loadeddata object
pro thm_ui_widget_tree::update,selected=selected,from_copy=copy
  compile_opt idl2

  widget_control,self.parent,update=0

  if keyword_set(copy) && widget_valid(copy) then begin
    copy_root = widget_info(copy,/child)
    oldtree = widget_info(copy_root,/child)
  endif else begin
    oldtree = self.root
  endelse

  destroytree = self.root

  self->populateTree

  self->copyexpansion,oldtree,self.root

  if keyword_set(selected) && is_string(selected[0]) then begin
    self->setSelected,self.root,selected
  endif

  widget_control,destroytree,/destroy

  widget_control,self.parent,update=1

end

;Public Method
;  uvalue: the user value that will be returned by event handler
;  uname: the user name that can be used to identify the widget
;  xsize: the x axis length of the tree viewing area
;  ysize: the y axis length of the tree viewing area
;  mode: the visualization/selection mode(see top documentation)
;  multi: whether multiple selections are allowed within the tree or not
;  leafonly: Set this flag to 1 if you want selections to be allowed
;            only on the leaf of the tree
;  showdatetime: If this boolean is 1 the date time of a variable will
;             be printed with its name
pro thm_ui_widget_tree::getProperty,$
                                uvalue=uvalue,$
                                uname=uname,$
                                xsize=xsize,$
                                ysize=ysize,$
                                mode=mode,$
                                multi=multi,$
                                leafonly=leafonly,$
                                showdatatime=showdatetime

  compile_opt idl2

  uvalue = self.uvalue
  uname = self.uname
  xsize = self.xsize
  ysize = self.ysize
  mode = self.mode
  multi = self.multi
  leafonly = self.leafonly
  showdatatime = self.showdatetime

end

;Public Method
;  uvalue: the user value that will be returned by event handler
;  uname: the user name that can be used to identify the widget
;  xsize: the x axis length of the tree viewing area
;  ysize: the y axis length of the tree viewing area
;  mode: the visualization/selection mode(see top documentation)
;  multi: whether multiple selections are allowed within the tree or not
;  leafonly: Set this flag to 1 if you want selections to be allowed
;            only on the leaf of the tree
;  showdatetime: If this boolean is 1 the date time of a variable will
;             be printed with its name
pro thm_ui_widget_tree::setProperty,$
                                uvalue=uvalue,$
                                uname=uname,$
                                xsize=xsize,$
                                ysize=ysize,$
                                mode=mode,$
                                multi=multi,$
                                leafonly=leafonly,$
                                showdatetime=showdatetime
  compile_opt idl2

  if keyword_set(uvalue) then begin
    self.uvalue = uvalue
    widget_control,self.id,set_uvalue=uvalue
  endif

  if keyword_set(uname) then begin
    self.uname = uname
    widget_control,self.id,set_uname=uname
  endif

  ;the x/y changes are done simultaneously, if possible
  ;this should prevent screen flicker when resizing
  if keyword_set(xsize) && keyword_set(ysize) then begin
    self.xsize=xsize
    self.ysize=ysize
    widget_control,self.root,xsize=xsize,ysize=ysize
  endif else if keyword_set(xsize) then begin
    self.xsize=xsize
    widget_control,self.root,xsize=xsize
  endif else if keyword_set(ysize) then begin
    self.ysize=ysize
    widget_control,self.root,ysize=ysize
  endif

  if is_num(mode) then begin
    self.mode = mode
  endif

  if is_num(multi) then begin
    self.multi = multi
  endif

  if is_num(leafonly) then begin
    self.leafonly = leafonly
  endif

  if is_num(showdatetime) then begin
    self.showdatetime = showdatetime
  endif

  if is_num(multi) || is_num(mode) || $
     is_num(leafonly) || is_num(showdatetime) then begin
    self->update
  endif


end

;copy the tree
function thm_ui_widget_tree::getCopy

  compile_opt idl2

  new_base = widget_base()

  new_root = widget_tree(new_base)

  widget_tree_move,self.root,new_root,/copy

  return,new_base

end

;Public method
;returns a list of the current selections in the tree
;at the requested dimensionality
;returns 0 if no selection exists.
function thm_ui_widget_tree::getValue

  compile_opt idl2

  if ~keyword_set(time) then time = 0
  if ~keyword_set(yaxis) then yaxis = 0

  if is_num(data) then begin
    data = data
  endif else begin
    data = 1
  endelse

  ;return,''

  return,self->getValueRecursive(self.root,0)

end

;Private Method: Not to be called by external code
;method for internal use by object.  Used to traverse tree in the
;getValue method
function thm_ui_widget_tree::getValueRecursive,widget,selected

  compile_opt idl2

  widget_control,widget,get_uvalue=state

  if widget eq self.root then begin
    sel = 0
  endif else begin
    sel = selected || widget_info(widget,/tree_select)
  endelse

  val = 0

  ;if we are at an interior node that will never have a value that is returned
  ;then we just aggregate any values from recursive calls to any children
  if state.value eq 'INTERIOR' then begin

    children = widget_info(widget,/all_children)

    if children[0] eq 0 then return,val

    for i = 0,n_elements(children)-1 do begin

      res = self->getValueRecursive(children[i],sel)

      if ~is_num(res) then begin
        if ~is_num(val) then begin
          val = [val,res]
        endif else begin
          val = [res]
        endelse
      endif

    endfor

  ;if we are at a group, we call the routine that handles the messiness
  ;of figuring out which values should be returned, given the set of options
  endif else if state.value eq 'GROUP' then begin

   return, self->getValueGroup(widget,sel)

  endif

  return,val

end

;Private Method: Not to be called by external code
;gets the value after the value routine has recursed down to a group
function thm_ui_widget_tree::getValueGroup,widget,selected

  sel = widget_info(widget,/tree_select)

  if self.mode eq 0 then begin

    if sel || (~self.leafonly && selected) then begin
      widget_control,widget,get_uvalue=state

      return,ptr_new({groupname:state.name,timename:state.timename,datanames:state.datanames,yaxisname:state.yaxisname})

    endif

  endif else if self.mode eq 1 then begin

    return,self->getGroupValueRecursive(widget,selected || sel)

  endif else if self.mode eq 2 then begin

    if sel || (~self.leafonly && selected) then begin

      widget_control,widget,get_uvalue=state

      return,state.name

    endif

  endif else if self.mode eq 3 then begin

     return,[self->getGroupValueRecursive(widget,0)]

  endif

  return,0

end

;Private Method: Not to be called by external code
;used to walk through a group and return a list of leaf names
function thm_ui_widget_tree::getGroupValueRecursive,widget,selected

  compile_opt idl2

  if widget_info(widget,/tree_folder) then begin

    widget_control,widget,get_uvalue=state

    if self.mode eq 3 && widget_info(widget,/tree_select) && state.value ne 'COMPONENT' then begin
      out = [state.name]
    endif else begin
      out = 0
    endelse

    children = widget_info(widget,/all_children)

    sel = selected || widget_info(widget,/tree_select)

    if self.mode eq 3 then sel = 0

    if children[0] ne 0 then begin

      for i = 0,n_elements(children)-1 do begin

        res = self->getGroupValueRecursive(children[i],sel)

        if ~is_num(res) then begin
          if ~is_num(out) then begin
            out = [out,res]
          endif else begin
            out = [res]
          endelse
        endif

      endfor

      return,out

    endif

  endif else if widget_info(widget,/tree_select) || (selected && ~self.leafonly) then begin

    widget_control,widget,get_uvalue=state

    return,state.name

  endif

  return,0

end

;Private Method: Not to be called by external code
;method walks down the tree and unexpands nodes
;Used in event handler after an expand event is
;returned
pro thm_ui_widget_tree::collapseNode,node

  children = widget_info(node,/all_children)

  if children[0] eq 0 then return

  for i = 0,n_elements(children)-1 do begin

    if widget_info(children[i],/tree_expanded) then begin

      widget_control,children[i],set_tree_expanded=0

      ;unselect?

      self->collapseNode,children[i]

    endif

  endfor

end

;Private Method: not to be called by external code
;This walks down the trees and expands all the nodes
;in the current tree that were expanded in the old tree
pro thm_ui_widget_tree::copyexpansion,oldnode,newnode

  compile_opt idl2

  oldchildren = widget_info(oldnode,/all_children)
  newchildren = widget_info(newnode,/all_children)

  if oldchildren[0] eq 0 then return
  if newchildren[0] eq 0 then return

  for i = 0,n_elements(newChildren)-1 do begin

    widget_control,newChildren[i],get_value=newname

    if ~keyword_set(newnames) then begin
      newnames = [newname]
    endif else begin
      newnames = [newnames,newname]
    endelse

  endfor

  for i = 0,n_elements(oldchildren)-1 do begin

    widget_control,oldchildren[i],get_value=oldname

    idx = where(oldname eq newnames,c)

    if c eq 1 && widget_info(oldchildren[i],/tree_expanded) then begin

      ;For some reason, windows sometimes like to activate the selection on
      ;this node when we expand it.
      tree_select = widget_info(newchildren[idx],/tree_select)

      widget_control,newchildren[idx],set_tree_expanded=1

      if widget_info(newchildren[idx],/tree_select) ne tree_select then begin
        widget_control,newchildren[idx],set_tree_select=tree_select
      endif

      self->copyexpansion,oldchildren[i],newchildren[idx]

    endif

  endfor

end

;Private Method: Not to be called by external code
;This routine constructs the assembles the idl tree widgets
;from the contents of the loadedData object
pro thm_ui_widget_tree::populateTree

  compile_opt idl2

  self.root = widget_tree(self.stash,xsize=self.xsize,ysize=self.ysize,UVALUE={value:'INTERIOR',children:''},multiple=self.multi,bitmap=self.nodemap,/mask)

  ldTreeData = self.ld->getTreeData()

  if is_struct(ldTreeData) then begin

    dim = dimen(ldTreeData.names)

    ;this loops overall names, adding a branch for each one
    for i = 0,dim[0]-1 do begin

      self->addBranch,ldTreeData.names[i,*],ldTreeData.objs[i]

    endfor

  endif

end

;Private Method: Not to be called by external code
;walks down to a particular leaf, adding widgets as needed
pro thm_ui_widget_tree::addBranch,names,obj

  compile_opt idl2

  current = self.root

  ;this block walks down the tree, adding interior nodes if necessary
  ;on its way to a leaf
  if n_elements(names) gt 1 then begin
    for i = 0,n_elements(names)-2 do begin

      name = names[i]

      current = self->expandNode(current,name)

      ;error
      if current eq 0 then begin
        return
      endif

    endfor
  endif

  ;if present, the last section of each branch will
  ;have a structurally similar subtree
  ;NOTE: the visibility of components is dependent on what self.mode is set to

  self->addGroup,current,obj

end

;Private Method: Not to be called by external code
;modularized group adding code, adds all the widgets that exist within a group.
;Two benefits:
;#1 This is where the messiest code that is likely to be subject to change will reside
;#2 This code is used to generate data groups AND to generate yaxis groups, preventing code duplication
pro thm_ui_widget_tree::addGroup,parent,obj,yaxis=yaxis,groupid=groupid

  compile_opt idl2

  if ~obj_valid(obj) then return

  name = obj->getname()
  datanames = obj->getDataNames()
  timeName = obj->getTimeName()
  yaxisname = obj->getYaxisName()
  isyaxis = obj->getisyaxis()

  if keyword_set(yaxis) then begin
    yaxis = 1
  endif else begin
    yaxis = 0
  endelse

  ts = ''

  if self.showdatetime then begin
    self.ld->getVarData,name=timename,data=d
    if ptr_valid(d) then begin
      low = min(*d,/nan,max=high)
      ts = ' [ ' + time_string(low) + ' to ' + time_string(high) + ' ] '
    endif
  endif

  if self.mode eq 0 then begin

    if isyaxis then return

    groupid = widget_tree(parent,$
                        value=name+ts,$
                        uvalue={value:'GROUP',name:name,timename:timename,datanames:datanames,yaxisname:yaxisname},$
                        bitmap=self.leafmap,$
                        /mask)

  endif else if self.mode eq 1 || self.mode eq 3 then begin

    if isyaxis && ~yaxis then return

    componentname = name + '_data'

    if ~keyword_set(timename) && $
       ~keyword_set(datanames) && $
       ~keyword_set(yaxisname) then begin

      groupid = widget_tree(parent,$
                          value=name+ts,$
                          uvalue={value:'GROUP',name:name},$
                          bitmap=self.leafmap,$
                          /mask)
    endif else begin

      state = {value:'GROUP',name:name}

      groupid = widget_tree(parent,$
                        value=name+ts,$
                        uvalue=state,$
                        bitmap=self.nodemap,$
                        /folder,$
                        /mask)

      if keyword_set(timeName) then begin
        timeid = widget_tree(groupid,value=timeName+ts,uvalue={value:'TIME',name:timename},bitmap=self.leafmap,/mask)
      endif

      if keyword_set(datanames) then begin

       ; componentid = widget_tree(groupid,value=componentname,uvalue={value:'COMPONENT',name:componentname},/FOLDER,bitmap=self.nodemap,/mask)
        for i = 0,n_elements(datanames)-1 do begin
          data = widget_tree(groupid,value=datanames[i]+ts,uvalue={value:'DATA',name:datanames[i]},bitmap=self.leafmap,/mask)
        endfor
      endif

      if ~keyword_set(yaxis) then begin

        if keyword_set(yaxisname) then begin

          ygroup = self.ld->getGroup(yaxisname)

          if obj_valid(ygroup) then begin
            self->addGroup,groupid,ygroup,/yaxis,groupid=yaxisid
          endif

        endif
      endif

    endelse

  endif else if self.mode eq 2 then begin

    group = widget_tree(parent,$
                         value=name+ts,$
                         uvalue={value:'GROUP',name:name},$
                         bitmap=self.leafmap,$
                         /mask)
  endif

end

;Private Method: Not to be called by external code
;adds a widget if widget not present, otherwise returns pre-existing widget
function thm_ui_widget_tree::expandNode,widget,name

  compile_opt idl2

  widget_control,widget,get_uvalue=state

  if state.value ne 'INTERIOR' then begin
    ok = error_message('Illegal attempt to expand node at leaf',/traceback)
    return,0
  endif

  if ~keyword_set(state.children) then begin
    state = {value:'INTERIOR',children:[name],name:name}
    out = widget_tree(widget,value=name,uvalue={value:'INTERIOR',children:[''],name:name},/folder,bitmap=self.nodemap,/mask)
  endif else begin

    idx = where(name eq state.children,c)

    if c eq 0 then begin
      out = widget_tree(widget,value=name,uvalue={value:'INTERIOR',children:[''],name:name},/folder,bitmap=self.nodemap,/mask)
      state = {value:'INTERIOR',children:[state.children,name],name:name}
    endif else if c gt 1 then begin
      ok = error_message('Non-unique child should not exist')
      return,0
    endif else begin
      children = widget_info(widget,/all_children)
      out  = children[idx]
    endelse

  endelse

  widget_control,widget,set_uvalue=state

  return,out

end

pro thm_ui_widget_tree::setSelected,node,selected

  compile_opt idl2

  if widget_info(node,/tree_root) ne node then begin

    widget_control,node,get_uvalue=state

    idx = where(state.name eq selected,c)

    if c gt 0 then begin
      widget_control,node,set_tree_select=1
      widget_control,node,set_tree_visible=1
    endif

  endif

  children = widget_info(node,/all_children)

  if children[0] ne 0 then begin

    for i = 0,n_elements(children)-1 do begin
      self->setSelected,children[i],selected
    endfor

  endif


end



;Private Method: Not to be called by external code
;This is the true object based event handler
;it handles internal events from the compound
;widgets
function thm_ui_widget_tree::handleEvent,event

  compile_opt idl2

 ; help,event,/str

  if self.skip_event then begin
    self.skip_event = 0
    return,0
  endif

  if Tag_Names(event, /Structure_Name) EQ 'WIDGET_TREE_EXPAND' then begin

    if ~widget_info(event.id,/tree_expanded) then begin
      self->collapseNode,event.id
    endif

    if !version.os_family ne 'Windows' then begin
      self.skip_event = 1
    endif

;    return,{THM_UI_TREE_EXPAND,ID:self.id,TOP:event.top,HANDLER:0L}
    return,0

  endif else if Tag_Names(event,/Structure_Name) eq 'WIDGET_TREE_SEL' then begin

    widget_control,event.id,get_uvalue=str

    if self.leafonly && $
       (self.mode eq 0 || self.mode eq 2) && $
       str.value eq 'INTERIOR' then begin

      widget_control,event.id,set_tree_select=0

    endif else if self.mode eq 1 && $
                  self.leafonly  && $
                  (str.value eq 'INTERIOR' || $
                   str.value eq 'GROUP' || $
                   str.value eq 'COMPONENT') then begin

      widget_control,event.id,set_tree_select=0

    endif

    return,{THM_UI_TREE_EVENT,ID:self.id,TOP:event.top,HANDLER:0L}

  endif

return,0

end

function thm_ui_widget_tree::cleanup

  compile_opt idl2

  Widget_Control,self.id,/Destroy

end

;Private Method: Not to be called by external code
;intializes the widget tree
;parentid:  the id of the parent base that this is part of
;uvalue:  the value that will be called in parent event handler
;loadedData: the loadedData object, used to populate tree
;xsize=xsize: the size of the tree widget area along the x-axis
;ysize=ysize: the size of the tree widget area along the y-axis
;mode=mode: indicates the viewing/selection mode.  See above for
;a description of the modes
;multi=multi: indicates whether multiple selections can be made in the tree(with shift/control click)
;leafonly=leafonly: Set this flag to 1 if you want selections to be allowed
;only on the leaf of the tree
;showdatetime: If this boolean is 1 the date time of a variable will
;             be printed with its name
;selected=selected: Pass in a list of variable names that you would like selected
function thm_ui_widget_tree::init,$
                           parentid,$
                           uvalue,$
                           loadedData,$
                           uname=uname,$
                           xsize=xsize,$
                           ysize=ysize,$
                           mode=mode,$
                           multi=multi,$
                           selected=selected,$
                           leafonly=leafonly,$
                           showdatetime=showdatetime

  compile_opt idl2

  if ~keyword_set(parentid) then begin
    ok = error_message('Need required argument: parentID',/traceback)
    return,0
  endif

  if ~keyword_set(uvalue) then begin
    ok = error_message('Need required argument: uvalue',/traceback)
    return,0
  endif

  if ~keyword_set(uvalue) || ~obj_valid(loadedData) then begin
    ok = error_message('Need required argument: loadedData',/traceback)
    return,0
  endif

  if ~keyword_set(uname) then begin
    uname = ''
  endif

  if ~keyword_set(xsize) then begin
    xsize = 300
  endif

  if ~keyword_set(ysize) then begin
    ysize = 300
  endif

  if ~is_num(mode) then begin
    mode = 0
  endif

  if ~is_num(multi) then begin
    multi = 1
  endif

  if ~keyword_set(leafonly) then begin
    leafonly = 0
  endif

  if ~keyword_set(showdatetime) then begin
    showdatetime = 0
  endif

  self.parent = parentid
  self.uvalue = uvalue
  self.ld = loadedData
  self.uname = uname
  self.xsize = xsize
  self.ysize = ysize
  self.mode = mode
  self.multi = multi
  self.leafonly = leafonly
  self.showdatetime = showdatetime
  self.nodemap = bytarr(16,16,3)
  self.nodemap[*] = 196
  self.nodemap[indgen(8)*2,7,*] = 0
  self.leafmap = self.nodemap
  self.nodemap[7:11,5:9,*] = 0
  self.nodemap[[7,7,11,11],[5,9,5,9],*] = 196
  ;self.bitmap[9,indgen(8)*2,*] = 0

  self.id = widget_base(self.parent, $
                uvalue=uvalue, $
                uname=uname, $
                event_func='thm_ui_widget_tree_event',$
                func_get_value='thm_ui_widget_tree_get_value')

  self.stash = widget_base(self.id,uvalue=self)

  widget_control,self.id,update=0

  self->populateTree

  if keyword_set(selected) && is_string(selected[0]) then begin

    self->setSelected,self.root,selected

  endif

  widget_control,self.id,update=1

  return,1

end

;Private Function: Not to be called by external code
;the event handler for the widget tree
;this is not a method of the widget object
;but it works as a means to create object-based compound widgets
;NOTE: this routine should only be called internally,
;it should not be called by the end-user.
function thm_ui_widget_tree_event,event

  compile_opt hidden,idl2

  store = widget_info(event.handler,/child)
  widget_control,store,get_uvalue=obj

  return,obj->handleEvent(event)

end

;Private Function: Not to be called by external code
;the getvalue function for the widget tree
;this is not a method of the widget object
;but it works as a means to create object-based compound widgets
;It is not as important as the event_handler callback,
;but it allows a user to easily query the widget value without
;needing to unpack the state(ie you can use widget_control,id,get_value=gv)
;NOTE: this routine should only be called internally,
;it should not be called by the end-user.
function thm_ui_widget_tree_get_value,id

 COMPILE_OPT hidden,idl2

  ; Recover the state of this compound widget
  stash = WIDGET_INFO(id, /CHILD)
  WIDGET_CONTROL, stash, GET_UVALUE=state

  return,state

end

pro thm_ui_widget_tree__define

  compile_opt idl2

  struct = { THM_UI_WIDGET_TREE,     $
              parent: 0,             $ ; id for the parent widget (must be a base)
              id: 0,                 $ ; widget id for the tree widget
              stash:0,               $ ; widget id for the tree widget stash widget
              root:0,                $ ; widget id for the tree widget root
              uvalue: '',            $ ; the user value to be called in parent event handler
              uname:'',              $ ; the name of the widget, can be used with find_by_name call to widget_info
              xSize: 0,              $ ; size of tree area in x direction
              ySize: 0,              $ ; size of tree area in y direction
              mode:0,                $ ; Indicates the viewing/selection mode for the data
              multi:0,               $ ; 0 means one selection only, 1 means multiple selections allowed
              leafonly:0,            $ ; 0 means selections can be made anywhere in the tree, 1 means only leaves may be selected
              showdatetime:0B,       $ ; if this boolean is 1 then the start/stop times of variables will be printed out with their names.
              nodemap:bytarr(16,16,3),$ ; the bitmap that is used as the tree icon for interior nodes
              leafmap:bytarr(16,16,3),$ ; the bitmap that is used as the tree icon for interior nodes
              ld:obj_new(),           $ ; the loaded data object
              skip_event:0B           $ ; set this flag to make the handler skip the next event

}

end