;+ 
;NAME:  
;  thm_ui_make_default_lineplot
;  
;PURPOSE:
;  Routine that creates a default lineplot on particular panel
;  
;CALLING SEQUENCE:
;  thm_ui_make_default_lineplot, loadedData, panel, xvar, yvar
;  
;INPUT:
;  loadedData: the loadedData object
;  panel:  the panel on which the plot should be placed.
;  xvar:  the name of the variable storing the x-data.
;  yvar:  the name of the variable storing the y-data.
;            
;KEYWORDS:
;  none
;        
;OUTPUT:
;  none
;
;--------------------------------------------------------------------------------

pro thm_ui_make_default_lineplot, loadedData, panel, xvar, yvar

compile_opt idl2, hidden

panel->GetProperty, traceSettings=traceSettings, xAxis=xAxis, yAxis=yAxis
ntraces = traceSettings->count()

; check if x,y-axis objects exist, create if not
if ~obj_valid(xAxis) then xAxis = obj_new('thm_ui_axis_settings', blacklabels=1)
if ~obj_valid(yAxis) then yAxis = obj_new('thm_ui_axis_settings', blacklabels=0)

; get and check if y-axis labels exist, create if not
yAxis->getProperty, labels=ylabels
if ~obj_valid(ylabels) then ylabels = obj_new('IDL_Container')
xAxis->getProperty, labels=xlabels
if ~obj_valid(xlabels) then xlabels = obj_new('IDL_Container')

; setup default color order ['k','m','r','g','c','b','y']
;def_colors = [[0,0,0],[255,0,255],[255,0,0],[0,255,0],[0,255,255],[0,0,255],[255,255,0]]
def_colors = [[0,0,255],[0,255,0],[255,0,0],[0,0,0],[255,255,0],[255,0,255],[0,255,255]]

factors = [[1,0],[1,1],[1,3],[3,1],[1,7],[5,3],[3,5],[7,1]]

color_idx1 = ntraces mod 7
color_idx2 = (ntraces +1) mod 7
factor1 = factors[0,(ntraces/7) mod 8]
factor2 = factors[1,(ntraces/7) mod 8]


color=(factor1*def_colors[*,color_idx1]+factor2*def_colors[*,color_idx2])/(factor1+factor2)

; get data object for x and y quantities
if loadedData->isParent(xvar) then begin ;sometimes name can be a group and data name, this makes sure we get the data
  group = loadedData->getGroup(xvar)
  xObj = group->getObject(xvar)
endif else begin
  xObj = loadedData->getObjects(name=xvar)
endelse

if loadedData->isParent(yvar) then begin ;sometimes name can be a group and data name, this makes sure we get the data
  group = loadedData->getGroup(yvar)
  yObj = group->getObject(yvar)
endif else begin
  yObj = loadedData->getObjects(name=yvar)
endelse

; get limits/dlimits and merge them
xObj->getProperty, dlimitptr=dlptr, limitptr=lptr
extract_tags,*dlptr,*lptr
dl = *dlptr

if (size(dl,/type) eq 8) then begin

  ; set y-axis scaling according to limits/dlimits
  if in_set('ylog', strlowcase(tag_names(dl))) then begin
    yAxis->setProperty, scaling=dl.ylog
  endif; else yAxis->setProperty, scaling=0
  
  if in_set('ystyle', strlowcase(tag_names(dl))) then ystyle=dl.ystyle else ystyle=0

  ; set y-axis limits and turn on fixed yrange
  if in_set('yrange', strlowcase(tag_names(dl))) then begin
    if dl.yrange[0] ne dl.yrange[1] then begin ; make sure the range isn't zero
      yaxis->getproperty, minfixedrange=minfixedrange, maxfixedrange=maxfixedrange
      if (minfixedrange eq 0) AND (maxfixedrange eq 0) then begin
        minfixedrange = dl.yrange[0]
        maxfixedrange = dl.yrange[1]
      endif
      if dl.yrange[0] lt minfixedrange then minfixedrange = dl.yrange[0]
      if dl.yrange[1] gt maxfixedrange then maxfixedrange = dl.yrange[1]
      yAxis->setProperty, minfixedrange=minfixedrange, maxfixedrange=maxfixedrange, $
                          rangeoption=2
    endif
  endif
  
  ; set tight y-limits if requested
  if in_set('ystyle', strlowcase(tag_names(dl))) then begin
    if ystyle then yaxis->setproperty, rangeoption=0, rangemargin=0
  endif
  
;  ;set labels
;  if in_set('labels', strlowcase(tag_names(dl))) then begin
;    if ntraces gt n_elements(dl.labels) then ylabel=yvar $
;      else ylabel=dl.labels[ntraces]
;  endif
endif

; check whether or not data is time quantity
xObj->getProperty, istime=xistime
yObj->getProperty, istime=yistime

ctobj = obj_new('IDLgrPalette')
;ctobj->loadct, file=


; add trace to traceSettings object
traceSettings->add,obj_new('thm_ui_line_settings',dataX=xvar, dataY=yvar, $
                           linestyle=obj_new('thm_ui_line_style', color=color, $
                           thick=1.0))

; add label for trace to labels object
;ylabels->add,obj_new('thm_ui_text', value=ylabel, font=2, format=3, size=8, $
ylabels->add,obj_new('thm_ui_text', value=yvar, font=2, format=3, size=8, $
                    color=color)
xlabels->add,obj_new('thm_ui_text',value ='',font=2,format=3,size=8,$
                    color=[0,0,0],show=0)

; set x and y axis properties
xAxis->SetProperty, labels=xlabels, isTimeAxis=xistime

;Set number of minor ticks(xAxis)
if xistime then begin
  xAxis->setProperty,numMinorTicks = 5
endif else begin
  xAxis->getProperty,scaling=xscale
  if xscale eq 1 then begin
    xAxis->setProperty,numMinorTicks = 8
  endif else if xScale eq 2 then begin
    xAxis->setProperty,numMinorTicks = 0
  endif else begin
    xAxis->setProperty,numMinorTicks = 4
  endelse
endelse

yAxis->SetProperty, labels=ylabels, isTimeAxis=yistime

;Set number of minor ticks(yAxis)
if yistime then begin
  yAxis->setProperty,numMinorTicks = 5
endif else begin
  yAxis->getProperty,scaling=yscale
  if yscale eq 1 then begin
    yAxis->setProperty,numMinorTicks = 8
  endif else if yScale eq 2 then begin
    yAxis->setProperty,numMinorTicks = 0
  endif else begin
    yAxis->setProperty,numMinorTicks = 4
  endelse
endelse

panel->SetProperty, traceSettings=traceSettings, xAxis=xAxis, yAxis=yAxis

end