;+
;NAME:
; thm_ui_draw_object
;
;PURPOSE:
; This object generates an object graphics tree using the IDLgr* objects that are built
; into IDL.  It generates this tree by querying the loaded data object and the window storage
; object to get the settings generated by the panel.  This object also performs other utility
; operations related to drawing.  These include animations of vertical bars and markers, updating
; the legend by performing a lookup in a hashed version of the plotted data, and reporting information
; about the current display.(Cursor segment selected,  range of displayed data, etc...)
;
; There are a few major functions in this object.
; #1 'update': will update the draw area to correspond to new settings
; #2 'setCursor' : actually makes the draw object do all the animation updates
; #3 'draw' : use it to force the object to redraw the draw area.
;
;$LastChangedBy: pcruce $
;$LastChangedDate: 2009-04-22 18:11:55 -0700 (Wed, 22 Apr 2009) $
;$LastChangedRevision: 5722 $
;$URL: svn+ssh://thmsvn@ambrosia.ssl.berkeley.edu/repos/thmsoc/tags/tdas_5_00/idl/themis/thm_ui_new/display/thm_ui_draw_object__define.pro $
;-

;Makes a vertical bar, updates legends, draws rubber band,
;draws markers(during draw animation), highlights markers
;during mouseover. 'Location' is the cursor location in coordinates
;normalized to the draw area size. This should be called pretty much
;any time the cursor moves in the draw area.
pro thm_ui_draw_object::setCursor,location

  compile_opt idl2
  
 ; tm = systime(/seconds)
  
  self.cursorloc = location
  
  self->updateVBar,location,self.panelInfo
  self->updateHBar,location,self.panelInfo
  
  ;If the rubber band is turned on then update its position when
  ;the cursor position is updated
  if self.rubberOn then begin
  
    if ~self->rubberBand(self.rubberStart,self.cursorLoc - self.rubberStart) then begin
      self.statusBar->update,'Problem drawing rubber band'
    ;t=error_message('Problem drawing rubber band',/traceback)
    endif
    
  endif
  
  ;Single panel legend update block
  if self.legendOn eq 1 && ptr_valid(self.panelInfo) then begin
  
    for i = 0,n_elements(*self.panelInfo)-1 do begin
    
      panel = ((*self.panelInfo)[i])
      
      if self->inBounds(panel) then begin
      
        xposnorm = (location[0] - panel.xplotpos[0])/(panel.xplotpos[1]-panel.xplotpos[0])
        yposnorm = (location[1] - panel.yplotpos[0])/(panel.yplotpos[1]-panel.yplotpos[0])
        
        if obj_valid(panel.legendModel) then begin
          panel.legendModel->setProperty,hide=0
        endif
        
        if obj_valid(panel.legendAnnoModel) then begin
          panel.legendAnnoModel->setProperty,hide=0
        endif
        
        self->setVarHide,panel,0
        
        self->updatelegend,xposnorm,yposnorm,panel
        
      endif else begin
      
        if obj_valid(panel.legendModel) then begin
          panel.legendModel->setProperty,hide=1
        endif
        
        if obj_valid(panel.legendAnnoModel) then begin
          panel.legendAnnoModel->setProperty,hide=1
        endif
        
        self->setVarHide,panel,1
      
        ;self->updateLegend,0,0,panel,/blank
        
      endelse
      
    endfor
    
  ;multiple panel legend update block
  endif else if self.legendOn eq 2 && ptr_valid(self.panelInfo) then begin
  
    drawlegend = 0
    
    for i = 0,n_elements(*self.panelInfo)-1 do begin
    
      panel = ((*self.panelInfo)[i])
      
      if self->inBounds(panel) then begin
      
        drawlegend = 1
        inBoundsIndex = i
        xposnorm = (location[0] - panel.xplotpos[0])/(panel.xplotpos[1]-panel.xplotpos[0])
        yposnorm = (location[1] - panel.yplotpos[0])/(panel.yplotpos[1]-panel.yplotpos[0])
        
      endif
      
    endfor
    
    if drawlegend then begin
      for i = 0,n_elements(*self.panelInfo)-1 do begin
      
        panel = ((*self.panelInfo)[i])
        
        if i eq inBoundsIndex then begin
          self->updatelegend,xposnorm,yposnorm,panel
        endif else begin
          self->updatelegend,xposnorm,yposnorm,panel,/noyvalue  
        endelse
        
      endfor
    endif else begin ;if we aren't drawing blank the text
    
      for i = 0,n_elements(*self.panelInfo)-1 do begin
      
        panel = ((*self.panelInfo)[i])
        
        self->updateLegend,0,0,panel,/blank
      endfor
      
    endelse
  endif
  
  ;Determine if marker should be drawn on cursor update
  ;Single panel marking
  if self.markerOn eq 1 && $
    ptr_valid(self.panelInfo) && $
    ptr_valid(self.currentMarkers) then begin
    
    panels = *self.panelInfo
    
    for i=0,n_elements(panels)-1 do begin
      if self->inBounds(panels[i]) && $
        self->inBounds(panels[i],location=self.markerStart) then begin
        
        xstart = (self.markerStart[0] - panels[i].xplotpos[0])/(panels[i].xplotpos[1]-panels[i].xplotpos[0])
        xstop = (self.cursorloc[0] - panels[i].xplotpos[0])/(panels[i].xplotpos[1]-panels[i].xplotpos[0])
        
        self->drawMarker,xstart,xstop,panels[i],(*self.currentMarkers)[0]
        
      endif
      
    endfor
    
  ;multiple panel marking
  endif else if self.markerOn eq 2 && $
    ptr_valid(self.panelInfo) && $
    ptr_valid(self.currentMarkers) then begin
    
    panels = *self.panelInfo
    
    inBounds = 0
    
    for i = 0,n_elements(panels)-1 do begin
      if self->inBounds(panels[i]) && $
        self->inBounds(panels[i],location=self.markerstart) then begin
        inBounds = 1
        xstart = (self.markerStart[0] - panels[i].xplotpos[0])/(panels[i].xplotpos[1]-panels[i].xplotpos[0])
        xstop = (self.cursorloc[0] - panels[i].xplotpos[0])/(panels[i].xplotpos[1]-panels[i].xplotpos[0])
      endif
    endfor
    
    if inBounds then begin
    
      markers = *self.currentMarkers
      
      for i = 0,n_elements(panels)-1 do begin
        self->drawMarker,xstart,xstop,panels[i],markers[i]
      endfor
    endif
    
  endif
  
  ;highlight markers during mouseover
  if ptr_valid(self.panelInfo) then begin
  
    panels = *self.panelInfo
    
    for i = 0,n_elements(panels) -1 do begin
    
      if ptr_valid(panels[i].markerInfo) then begin
      
        xloc = (location[0] - panels[i].xplotpos[0])/(panels[i].xplotpos[1]-panels[i].xplotpos[0])
        
        marked = 0 ; to make sure only one panel gets marked
        
        for j = n_elements(*panels[i].markerInfo)-1,0,-1 do begin
        
          marker = (*panels[i].markerInfo)[j]
          
          if marker.displayed then begin
          
            if (xloc ge marker.pos[0] && xloc le marker.pos[1] && ~marked && self->inBounds(panels[i])) && marker.selected then begin
              marked = 1
              marker.frames[0]->setProperty,color=self->convertColor(self->hueRotation(marker.color))
              marker.frames[1]->setProperty,color=self->convertColor(self->hueRotation(marker.color))
              panels[i].markerIdx = j
            endif else if xloc ge marker.pos[0] && xloc le marker.pos[1] && ~marked && self->inBounds(panels[i]) then begin
              marked = 1
              marker.frames[0]->setProperty,color=self->convertColor(self->hueRotation(marker.color))
              marker.frames[1]->setProperty,color=self->convertColor(self->hueRotation(marker.color))
              panels[i].markerIdx = j
            endif else if marker.selected then begin
              marker.frames[0]->setProperty,color=self->convertColor(self->huerotation(self->hueRotation(marker.color)))
              marker.frames[1]->setProperty,color=self->convertColor(self->huerotation(self->hueRotation(marker.color)))
            endif else begin
              marker.frames[0]->setProperty,color=self->convertColor(marker.color)
              marker.frames[1]->setProperty,color=self->convertColor(marker.color)
            endelse
          endif
          
        endfor
        
        if ~marked then begin
          panels[i].markerIdx = -1
        endif
        
      endif
      
    endfor
    
    ptr_free,self.panelInfo
    self.panelInfo = ptr_new(panels)
    
  endif
  
  ;print,'TM1: ' + strtrim(string((systime(/seconds)-tm)),2)
  
  ;self.destination->draw,self.scene,/draw_instance
  self->draw
  
  ;print,'TM2: ' + strtrim(string((systime(/seconds)-tm)),2)
  
end

pro thm_ui_draw_object::updateVBar,location,panelInfo

  compile_opt idl2

 ;single panel vBar draw mode
  if self.vBarOn eq 1 && ptr_valid(self.panelInfo) then begin
  
    for i = 0,n_elements(*self.panelInfo)-1 do begin
    
      xpos = ((*self.panelInfo)[i]).xplotpos
      ypos = ((*self.panelInfo)[i]).yplotpos
      vBar = ((*self.panelInfo)[i]).vBar
      anno = ((*self.panelInfo)[i]).annotation
      view = ((*self.panelInfo)[i]).view
      
      ;anno->setProperty,hide=0
      old = vBar->get(/all)
      if obj_valid(old) then obj_destroy,old
      vBar->remove,/all
      
      if self->inBounds((*self.panelInfo)[i]) then begin
      
        xobjpos = (location[0] - xpos[0])/(xpos[1]-xpos[0])
        
        vBar->add,obj_new('IDLgrPolyline',[xobjpos,xobjpos],[0.,1.],[.5,.5],color=self->convertColor([0,0,0]),hide=0)
      endif
      
    ;self.destination->draw,view
    ;self.destination->draw,anno
      
    endfor
    
  ;multiple panel vBar draw mode
  endif else if self.vBarOn eq 2 && ptr_valid(self.panelInfo) then begin
  
    drawbar = 0
    
    ;this section determines if the cursor is within the area of *any*
    ;panel
    for i = 0,n_elements(*self.panelInfo)-1 do begin
    
      xpos = ((*self.panelInfo)[i]).xplotpos
      ypos = ((*self.panelInfo)[i]).yplotpos
      vBar = ((*self.panelInfo)[i]).vBar
      anno = ((*self.panelInfo)[i]).annotation
      view = ((*self.panelInfo)[i]).view
      
      ;anno->setProperty,hide=0
      old = vBar->get(/all)
      if obj_valid(old) then obj_destroy,old
      vBar->remove,/all
      
      if self->inBounds((*self.panelInfo)[i]) then begin
      
        xobjpos = (location[0] - xpos[0])/(xpos[1]-xpos[0])
        
        drawbar = 1
        
      ;vBar->add,obj_new('IDLgrPolyline',[xobjpos,xobjpos],[0.,1.],[.5,.5],color=self->convertColor([0,0,0]),hide=0)
      endif
      
    endfor
    
      ;If it is in bounds, then draw the vbar proportional to each panel.
    if drawBar then begin
    
      for i = 0,n_elements(*self.panelInfo)-1 do begin
      
        xpos = ((*self.panelInfo)[i]).xplotpos
        ypos = ((*self.panelInfo)[i]).yplotpos
        vBar = ((*self.panelInfo)[i]).vBar
        anno = ((*self.panelInfo)[i]).annotation
        view = ((*self.panelInfo)[i]).view
        
        ;anno->setProperty,hide=0
        ;vBar->remove,/all
        vBar->add,obj_new('IDLgrPolyline',[xobjpos,xobjpos],[0.,1.],[.5,.5],color=self->convertColor([0,0,0]),hide=0)
        
      endfor
      
    endif
    
  endif
  
end

pro thm_ui_draw_object::updateHBar,location,panelInfo

  compile_opt idl2

 ;single panel hBar draw mode
  if self.hBarOn eq 1 && ptr_valid(self.panelInfo) then begin
  
    for i = 0,n_elements(*self.panelInfo)-1 do begin
    
      xpos = ((*self.panelInfo)[i]).xplotpos
      ypos = ((*self.panelInfo)[i]).yplotpos
      hBar = ((*self.panelInfo)[i]).hBar
      anno = ((*self.panelInfo)[i]).annotation
      view = ((*self.panelInfo)[i]).view
      
      ;anno->setProperty,hide=0
      old = hBar->get(/all)
      if obj_valid(old) then obj_destroy,old
      hBar->remove,/all
      
      if self->inBounds((*self.panelInfo)[i]) then begin
      
        yobjpos = (location[1] - ypos[0])/(ypos[1]-ypos[0])
        
        hBar->add,obj_new('IDLgrPolyline',[0.,1.],[yobjpos,yobjpos],[.5,.5],color=self->convertColor([0,0,0]),hide=0)
      endif
      
    ;self.destination->draw,view
    ;self.destination->draw,anno
      
    endfor
    
  ;multiple panel hBar draw mode
  endif else if self.hBarOn eq 2 && ptr_valid(self.panelInfo) then begin
  
    drawbar = 0
    
    ;this section determines if the cursor is within the area of *any*
    ;panel
    for i = 0,n_elements(*self.panelInfo)-1 do begin
    
      xpos = ((*self.panelInfo)[i]).xplotpos
      ypos = ((*self.panelInfo)[i]).yplotpos
      hBar = ((*self.panelInfo)[i]).hBar
      anno = ((*self.panelInfo)[i]).annotation
      view = ((*self.panelInfo)[i]).view
      
      ;anno->setProperty,hide=0
      old = hBar->get(/all)
      if obj_valid(old) then obj_destroy,old
      hBar->remove,/all
      
      if self->inBounds((*self.panelInfo)[i]) then begin
      
        yobjpos = (location[1] - ypos[0])/(ypos[1]-ypos[0])
        
        drawbar = 1
        
      ;vBar->add,obj_new('IDLgrPolyline',[xobjpos,xobjpos],[0.,1.],[.5,.5],color=self->convertColor([0,0,0]),hide=0)
      endif
      
    endfor
    
      ;If it is in bounds, then draw the vbar proportional to each panel.
    if drawBar then begin
    
      for i = 0,n_elements(*self.panelInfo)-1 do begin
      
        xpos = ((*self.panelInfo)[i]).xplotpos
        ypos = ((*self.panelInfo)[i]).yplotpos
        hBar = ((*self.panelInfo)[i]).hBar
        anno = ((*self.panelInfo)[i]).annotation
        view = ((*self.panelInfo)[i]).view
        
        ;anno->setProperty,hide=0
        ;vBar->remove,/all
        hBar->add,obj_new('IDLgrPolyline',[0.,1.],[yobjpos,yobjpos],[.5,.5],color=self->convertColor([0,0,0]),hide=0)
        
      endfor
      
    endif
    
  endif
  
end

;this code performs an update on the legend text values for the provided panel
;it fills them in with the normalized data values provided as arguments
pro thm_ui_draw_object::updateLegend,xposnorm,yposnorm,panel,blank=blank,noyvalue=noyvalue

  compile_opt idl2
  
  ;update panel legend
  ;blank panel, empty legend
  if keyword_set(blank) && ptr_valid(panel.traceInfo) then begin
    panel.xobj->setProperty,strings=''
    
  ;  if panel.hasSpec then begin
    panel.yobj->setProperty,strings=''
  ;  endif
    
    traces = *panel.traceInfo
    
    for j = 0,n_elements(traces)-1 do begin
      if obj_valid(traces[j].textObj) then begin
        traces[j].textObj->setProperty,strings=''
      endif
    endfor
    
  ;inside panel range, update with data
  endif else if ptr_valid(panel.traceinfo) then begin
  
    location = self.cursorloc
    
    if panel.locked then begin
      xrange = panel.lockedRange
      xscale = panel.lockedScale
    endif else begin
      xrange = panel.xrange
      xscale = panel.xscale
    endelse
    
    xdatapos = xrange[0]+(xrange[1]-xrange[0])*xposnorm
    formatdata={timeAxis:panel.xIsTime,formatid:panel.xIsTime?15:4,scaling:xscale,exponent:~xscale?0:2}
    panel.xobj->setProperty,strings=formatannotation(0,0,xdatapos,data=formatdata)
    
    if ~keyword_set(noyvalue) then begin
      ydatapos = panel.yrange[0]+(panel.yrange[1]-panel.yrange[0])*yposnorm
      formatdatay={timeAxis:panel.yIsTime,formatid:panel.yIsTime?15:4,scaling:panel.yscale,exponent:~panel.yscale?0:2}
     ; if panel.hasSpec then begin
      panel.yobj->setProperty,strings=formatannotation(0,0,ydatapos,data=formatdatay)
      ;endif
    endif else begin
      panel.yobj->setProperty,strings=''
    endelse
    
    formatdataz={timeAxis:panel.zIsTime,formatid:panel.zIsTime?15:4,scaling:panel.zscale,exponent:~panel.zscale?0:2}
    
    traces = *panel.traceInfo
    
    for  i= 0,n_elements(traces)-1 do begin
    
      if ~ptr_valid(traces[i].refData) then begin
        dataString = 'NaN'
      endif else begin
      
        if traces[i].isSpec then begin
        
          refdim = dimen(*traces[i].refData)
          if n_elements(refDim) eq 1 then begin
            refDim = [refDim,1]
          endif
          
          refxidx = (round(xposnorm*refdim[0]) < (refdim[0]-1)) > 0
          refyidx = (round(yposnorm*refdim[1]) < (refdim[1]-1)) > 0
          
       ;   if finite((*traces[i].refData)[refxidx,refyidx],/inf) then stop
          
          dataString = formatannotation(0,0,(*traces[i].refData)[refxidx,refyidx],data=formatdataz)
          
       ;   print,dataString,refdim[0],refxidx,(*traces[i].refData)[refxidx,refyidx]
        endif else begin
        
          if ~ptr_valid(traces[i].abcissa) then begin
          
            refdim = dimen(*traces[i].refData)
            refxidx = (round(xposnorm*refdim[0]-1) < (refdim[0]-1)) > 0
            formatdatay={timeAxis:panel.yIsTime,formatid:panel.yIsTime?15:4,scaling:panel.yscale,exponent:~panel.yscale?0:2}
            
          endif else begin
          
            tmp = min(abs(*traces[i].abcissa - xposnorm),refxidx)
            formatdatay={timeAxis:panel.yIsTime,formatid:panel.yIsTime?15:4,scaling:panel.yscale,range:panel.yrange,exponent:~panel.yscale?0:2}
            
          endelse
          
          dataString = formatannotation(0,0,(*traces[i].refData)[refxidx],data=formatdatay)
          
        endelse
        
      endelse
      
      if obj_valid(traces[i].textObj) then begin
        traces[i].textObj->setProperty,strings=dataString
      endif
      
   ;   print,dataString,(dimen(*traces[i].refdata))[0],refxidx,(*traces[i].refData)[refxidx]
      
    endfor
    
  endif
  
  ;now update variables using the same logic
  if keyword_set(blank) && ptr_valid(panel.varInfo) then begin
 
    vars = *panel.varInfo
    
    for i = 0,n_elements(vars)-1 do begin
      vars[i].textObj->setProperty,strings=''
    endfor
    
  endif $
   else if ptr_valid(panel.varInfo) then begin
  
    vars = *panel.varInfo
    
    for i = 0,n_elements(vars)-1 do begin
    
      nx  = n_elements(*vars[i].dataY)
    
      idx = (round(xposnorm*(nx-1)) > 0) < (nx-1)  
    
      value = (*vars[i].dataY)[idx]
      
      formatdata={timeAxis:vars[i].isTime,formatid:vars[i].annotateStyle,scaling:0,exponent:0}
      str = formatannotation(0,0,value,data=formatdata)
      
      if value ge 0 then begin
        str = ' ' + str
      endif
      
      vars[i].textObj->setProperty,strings=str
      
    endfor
    
  endif
  
  
end

;draws marker and updates the range in the marker object
pro thm_ui_draw_object::drawMarker,xstart,xstop,panel,marker,remove=remove

  compile_opt idl2
  
  zstack = .21
  
  xbegin = min([xstart,xstop],/nan)
  xend = max([xstart,xstop],/nan)
  
  ; print,xbegin,xend
  
  ;get rid of old partial markers
  old = panel.marker->get(/all)
  if obj_valid(old[0]) then obj_destroy,old
  panel.marker->remove,/all
  
  if keyword_set(remove) then return
  
  marker->getProperty,settings=settings
  
  settings->getProperty,$
    fillColor=fillColor,$
    lineStyle=lineStyle,$
    drawOpaque=drawOpaque
    
  lineStyle->getProperty,$
    id=lineid,$
    show=lineshow,$
    color=linecolor,$
    thickness=linethick
    
  lineleft = obj_new('IDLgrPolyline',$
    [xbegin,xbegin],$
    [0.,1.],$
    replicate(zstack,2)+.00001,$
    linestyle=lineid,$
    hide=~lineshow,$
    color=self->convertColor(linecolor),$
    thick=linethick)
    
  lineright = obj_new('IDLgrPolyline',$
    [xend,xend],$
    [0.,1.],$
    replicate(zstack,2)+.00001,$
    linestyle=lineid,$
    hide=~lineshow,$
    color=self->convertColor(linecolor),$
    thick=linethick)
    
  poly = obj_new('IDLgrPolygon',$
    [xbegin,xend,xend,xbegin],$
    [0.,0.,1.,1.],$
    replicate(zstack,4),$
    color=self->convertColor(fillColor),$
    alpha_channel=drawOpaque)
    
  panel.marker->add,lineleft
  panel.marker->add,lineright
  panel.marker->add,poly
  
  if panel.locked then begin
    range = ([xbegin,xend]*(panel.lockedrange[1]-panel.lockedrange[0]))+panel.lockedrange[0]
  endif else begin
    range = ([xbegin,xend]*(panel.xrange[1]-panel.xrange[0]))+panel.xrange[0]
  endelse
  
  ;   print,time_string(range)
  
  marker->setProperty,range=range
  
end

;returns true if the current cursor loc is within the bounds of the panel
;location is an optional keyword.  If set, the location passed via the keyword
;will be used rather than the current cursor loc
function thm_ui_draw_object::inBounds,panelInfo,location=location

  compile_opt idl2
  
  if ~keyword_set(location) then location = self.cursorloc
  
  ;print,'x',panelInfo.xplotpos,'y',panelInfo.yplotpos
  
  return,location[0] ge panelInfo.xplotpos[0] && $
    location[0] le panelInfo.xplotpos[1] && $
    location[1] ge panelInfo.yplotpos[0] && $
    location[1] le panelInfo.yplotpos[1]
    
end

;returns an array of structs that indicate the panels overlapped by the rubber band
;structs have the form {idx:0,xrange:[0,1],yrange[0,1],vars:ptr_new()}
;idx is the panel index in the current display
;xrange is the xrange of the panel in non-logarithmic(normal) space
;yrange is the yrange of the panel in non-logarithmic(normal) space
;vars is either a null pointer or an array of structs of the form:
;{range:[0,1]} which stores the range of each variable on the panel
;returns 0 on fail
function thm_ui_draw_object::GetrubberBandPos

  compile_opt idl2
  
  ;prototype output structs
  str = {idx:0,xrange:[0D,1D],yrange:[0D,1D],vars:ptr_new()}
  varstr = {range:[0D,1D]}
  
  if ~ptr_valid(self.panelInfo) then return,0
  
  str_arr = replicate(str,n_elements(*self.panelInfo))
  
  arr_num = 0
  
  ;calculate the borders of the rubberband box
  ;because the box can be oriented in various ways
  ;the start & end pos may not be the expected corner
  left = self.cursorloc[0] < self.rubberstart[0]
  right = self.cursorloc[0] > self.rubberstart[0]
  bottom = self.cursorloc[1] < self.rubberstart[1]
  top = self.cursorloc[1] > self.rubberstart[1]
  
  ;loop over panels
  for i = 0,n_elements(*self.panelInfo)-1 do begin
  
    xplotpos = ((*self.panelInfo)[i]).xplotpos
    yplotpos = ((*self.panelInfo)[i]).yplotpos
    
    if ((*self.panelInfo)[i]).locked then begin
  ;  if 0 then begin
      xrange = ((*self.panelInfo)[i]).lockedRange
    endif else begin
      xrange = ((*self.panelInfo)[i]).xrange
    endelse
    
    yrange = ((*self.panelInfo)[i]).yrange
    
    ;determine if rubber band intersects with panel
    if (right  ge  xplotpos[0] && $
      left   le  xplotpos[1] && $
      top    ge  yplotpos[0] && $
      bottom le  yplotpos[1]) then begin
      
      ;determine the dimensions of the overlapping box
      bx_right    = right < xplotpos[1]
      bx_left     = left > xplotpos[0]
      bx_top      = top < yplotpos[1]
      bx_bottom   = bottom > yplotpos[0]
      
      ;now calculate ranges
      
      bx_left_norm = (bx_left - xplotpos[0])/(xplotpos[1]-xplotpos[0])
      bx_right_norm = (bx_right - xplotpos[0])/(xplotpos[1]-xplotpos[0])
      bx_bottom_norm = (bx_bottom - yplotpos[0])/(yplotpos[1]-yplotpos[0])
      bx_top_norm = (bx_top - yplotpos[0])/(yplotpos[1]-yplotpos[0])
      
      
      rng_left   = bx_left_norm*(xrange[1]-xrange[0])+xrange[0]
      rng_right  = bx_right_norm*(xrange[1]-xrange[0])+xrange[0]
      rng_bottom = bx_bottom_norm*(yrange[1]-yrange[0])+yrange[0]
      rng_top    = bx_top_norm*(yrange[1]-yrange[0])+yrange[0]
      
      outxrange = [rng_left,rng_right]
      outyrange = [rng_bottom,rng_top]
      
      ;now translate from logarithmic to normal space(if necessary)
      if ((*self.panelInfo)[i]).xscale eq 1 then $
        outxrange = 10D ^ outxrange $
      else if  ((*self.panelInfo)[i]).xscale eq 2 then $
      outxrange = exp(outxrange)
      
    if ((*self.panelInfo)[i]).yscale eq 1 then $
      outyrange = 10D ^ outyrange $
    else if  ((*self.panelInfo)[i]).yscale eq 2 then $
    outyrange = exp(outyrange)
    
  str_arr[arr_num].idx = i
  str_arr[arr_num].xrange = outxrange
  str_arr[arr_num].yrange = outyrange
  
  if ptr_valid(((*self.panelInfo)[i]).varInfo) then begin
  
    varInfoArray = *((*self.panelInfo)[i]).varInfo
    
    varstr_arr = replicate(varstr,n_elements(varInfoArray))
    
    for j = 0,n_elements(varInfoArray) - 1 do begin
    
      varrange = varInfoArray[j].range
      
      varlow = bx_left_norm*(varrange[1]-varrange[0])+varrange[0]
      varhigh = bx_right_norm*(varrange[1]-varrange[0])+varrange[0]
      
      outvarrange = [varlow,varhigh]
      
      if varInfoArray[j].scaling eq 1 then begin
        outvarRange = 10. ^ outvarrange
      endif else if varInfoArray[j].scaling eq 2 then begin
        outvarRange = exp(outvarrange)
      endif
      
      varstr_arr[j].range=outvarrange
      
    endfor
    
    str_arr[arr_num].vars = ptr_new(varstr_arr)
    
  endif
  
  arr_num++
endif

endfor

if arr_num eq 0 then return,0 else return,str_arr[0:arr_num-1]

end

;Return a bunch of information about a panel with a particular index
;returns 0 on fail
function thm_ui_draw_object::GetPanelInfo,index

  compile_opt idl2
  
  info = {$
    xpos:[0D,1D],$  ;the normalized x position of the plot within the panel
    ypos:[0D,1D],$  ;the normalized y position of the plot within the panel
    xrange:[0D,1D],$ ;the x data range of the panel(If xscale= 1 or 2 it will be the log of the range)
    yrange:[0D,1D],$ ;the y data range of the panel(If yscale= 1 or 2 it will be the log of the range)
    zrange:[0D,1D],$ ;the z data range of the panel(If zscale= 1 or 2 it will be the log of the range)
    xmajorSize:0D,$  ;the size of a major tick on the x axis(If xscale= 1 or 2 it will be in logarithmic space)
    ymajorSize:0D,$  ;the size of a major tick on the y axis(If yscale= 1 or 2 it will be in logarithmic space)
    xscale:0D,$  ;indicates scaling option for the x axis 0:linear,1:log10,2:logN
    yscale:0D,$  ;indicates scaling option for the y axis 0:linear,1:log10,2:logN
    zscale:0D,$  ;indicates scaling option for the z axis 0:linear,1:log10,2:logN
    hasspec:0B,$ ;boolean indicates whether the panel has at least one spectrogram, if yes Z info will apply
    hasLine:0B } ;boolean indicates whether the panel has at least one line, if yes Z info will apply
    
  if ~ptr_valid(self.panelInfo) || $
    n_elements(index) ne 1 || $
    index ge n_elements(*self.panelInfo) then begin
    return,0
  endif
  
  panel = (*self.panelInfo)[index]
  
  info.xpos = panel.xplotpos
  info.ypos = panel.yplotpos
  info.xrange = panel.xrange
  info.yrange = panel.yrange
  info.zrange = panel.zrange
  info.xmajorsize = panel.xmajorsize
  info.ymajorsize = panel.ymajorsize
  info.xscale = panel.xscale
  info.yscale = panel.yscale
  info.zscale = panel.zscale
  info.hasSpec = panel.hasSpec
  info.hasLine = panel.hasLine
  
  return,info
  
end

;returns an array that shows panel size.
;Array units are points.
;array format = [xpos,ypos,width,height]
function thm_ui_draw_object::getPanelSize,xdims,ydims

  compile_opt idl2
  
  xpos = self->norm2pt(xdims[0],0)
  xlen = self->norm2pt(xdims[1]-xdims[0],0)
  ypos = self->norm2pt(ydims[0],1)
  ylen = self->norm2pt(ydims[1]-ydims[0],1)
  
  return,[xpos,ypos,xlen,ylen]
  
end

;Returns the number of panels currently displayed
function thm_ui_draw_object::getPanelNumber

  compile_opt idl2
  
  if ~ptr_valid(self.panelInfo) then return,0
  
  return,n_elements(*self.panelInfo)
  
end

;Returns the marker or list of markers that were created
;by the most recent markeron/markeroff call
;if no markers available, returns 0
;returns an array of structs of the form
;{idx:0,marker:obj_new('thm_ui_marker')}
;idx is the panel index
;the object is the marker
function thm_ui_draw_object::getMarkers

  compile_opt idl2
  
  if ~ptr_valid(self.currentMarkers) then return,0
  
  markerStruct = {idx:0,marker:obj_new()}
  
  markers = replicate(markerStruct,n_elements(*self.currentMarkers))
  
  for i = 0,n_elements(markers)-1 do begin
    markers[i].marker = ((*self.currentMarkers)[i])->copy()
  endfor
  
  markers[*].idx = *self.markerIdx
  
  return,[markers]
  
end

;returns the a struct that identifies that panel and the
;the component of the panel that was clicked {panelidx:0,component:0,marker:-1}
;components are numbered:  0=plot,1=xaxis,2=yaxis,3=zaxis,4=variables
;marker: the index of any selected markers, is -1 if none
;returns 0L  if click is nothing(ie page)
;resolution of position is only approximate at this point
function thm_ui_draw_object::getClick

  compile_opt idl2
  
  if ~ptr_valid(self.panelInfo) then return,0
  
  panels = *self.panelInfo
  
  loc = self.cursorloc
  
  markeridx = -1L
  panelidx = -1L
  
  for i = 0,n_elements(panels)-1 do begin
  
    xpos = panels[i].xplotpos
    ypos = panels[i].yplotpos
    margins = panels[i].margins
    place = panels[i].zplacement
    
    if loc[0] ge xpos[0] && $  ;panel
      loc[0] le xpos[1] && $
      loc[1] ge ypos[0] && $
      loc[1] le ypos[1] then  begin
      return,{panelidx:i,component:0,marker:panels[i].markeridx}
    endif else if loc[0] ge xpos[0] - margins[0] && $ ;left side
      loc[0] le xpos[0] && $
      loc[1] ge ypos[0] && $
      loc[1] le ypos[1] then begin
      
      if place eq 2 then begin
        return,{panelidx:i,component:3,marker:-1} ;zaxis
      endif else begin
        return,{panelidx:i,component:2,marker:-1} ;yaxis
      endelse
      
    endif else if loc[0] ge xpos[1] && $ ;right side
      loc[0] le xpos[1] + margins[1] && $
      loc[1] ge ypos[0] && $
      loc[1] le ypos[1] then begin
      
      if place eq 3 then begin
        return,{panelidx:i,component:3,marker:-1} ;zaxis
      endif else begin
        return,{panelidx:i,component:2,marker:-1} ; yaxis
      endelse
      
    endif else if loc[0] ge xpos[0] && $ ;top side
      loc[0] le xpos[1] && $
      loc[1] ge ypos[1] && $
      loc[1] le ypos[1] + margins[2] then begin
      
      if place eq 0 then begin
        return,{panelidx:i,component:3,marker:-1} ;zaxis
      endif else begin
        return,{panelidx:i,component:1,marker:-1} ;xaxis
      endelse
      
    endif else if loc[0] ge xpos[0] && $ ;bottom side
      loc[0] le xpos[1] && $
      loc[1] ge ypos[0] - margins[3] && $
      loc[1] le ypos[0] then begin
      
      if place eq 1 then begin
        return,{panelidx:i,component:3,marker:-1} ;zaxis
      endif else begin
        return,{panelidx:i,component:1,marker:-1} ;xaxis
      endelse
      
    endif else if loc[0] ge xpos[0] && $
      loc[0] le xpos[1] && $
      loc[1] ge ypos[0] - margins[3] - margins[4] && $
      loc[1] le ypos[0] - margins[3] then begin
      
      return,{panelidx:i,component:4,marker:-1} ; vars
      
    endif
    
  endfor
  
  return,0
  
end

;This routine performs the draw work necessary to display a marker click
;basically this means mutating the selected flag on markerinfo
pro thm_ui_draw_object::setClick,infostruct

  compile_opt idl2
  
  if keyword_set(infostruct) && $
    is_struct(infostruct) && $
    infostruct.marker ne -1 && $
    ptr_valid(self.panelInfo) then begin
    
    panels = *self.panelInfo
    
    for i = 0,n_elements(panels)-1 do begin
    
      if ptr_valid(panels[i].markerInfo) then begin
      
        markers = *panels[i].markerInfo
        
        if i eq infostruct.panelidx then begin
          old_selected = markers[infostruct.marker].selected
        endif
        
        markers[*].selected = 0
        
        if i eq infostruct.panelidx then begin
        
          if old_selected eq 1 then begin
            markers[infostruct.marker].selected = 0
          endif else begin
            markers[infostruct.marker].selected = 1
          endelse
          
        endif
        
        ptr_free,panels[i].markerInfo
        panels[i].markerInfo = ptr_new(markers)
        
      endif
      
    endfor
    
    ptr_free,self.panelInfo
    self.panelInfo = ptr_new(panels)
    
  endif
  
  self->setCursor,self.cursorloc
end

;This function updates the entire display, it will be pretty
;slow so it should only be called after panel applies, but not
;during common widget events.
pro thm_ui_draw_object::update,windowStorage,loadedData,postscript=postscript,error=error

  compile_opt idl2
  
  error = 1
  
  lockedVerticalSpacing = 5
  
  widget_control,/hourglass
  
  self.staticViews->remove,/all
  self.dynamicViews->remove,/all
  
  heap_gc
  
  self.dynamicViews->add,self.rubberView
  
  if keyword_set(postscript) then begin
    self.postscript=1
  endif else begin
    self.postscript=0
  endelse
  
  ;use copy to prevent mutation of input settings
  activeWindow = (windowStorage->getActive())->copy()
  
  activeWindow->getProperty,settings=pageSettings,panels=panels,nRows=nRows,nCols=nCols,locked=locked
  
  pageSettings->getProperty,ypanelspacing=yverticalspacing
  
  if locked ne -1 && yverticalspacing ge 5 then begin
  
    pageSettings->setProperty,ypanelspacing=5
    
  endif
  
  ;first draw page
  if ~self->updatePage(pageSettings) then begin
    self.statusbar->update,'Draw Object Error: Invalid page settings when drawing page'
    self.historyWin->update,'Draw Object Error: Invalid page settings when drawing page'
    self->nukeDraw
    self->createInstance
    return
  endif
  
  ;needed for creating the illusion of transparent ticks, without using true transparency
  pageSettings->getProperty,backgroundcolor=backgroundcolor
  
  margins = activeWindow->getMargins()
  
  panelObjs = panels->get(/ALL)
  
  if ~obj_valid(self.panelViews) then begin
    self.panelViews = obj_new('IDLgrViewGroup')
    self.scene->add,self.panelViews
  endif
  
  ;self.staticViews->add,self.panelViews
  
  ;this code would properly deallocate memory from previous call
  ;but is probably unnecessary because IDL has garbage collection
  ;If we are finding problems with memory leaks, this code should
  ;be uncommented
  ; if ptr_valid(self.panelInfo) then begin
  ;
  ;
  ;    for i = 0,n_elements(*self.panelInfo) -1 do begin
  ;
  ;      panelInfo = (*self.panelInfo)[i]
  ;
  ;      if ptr_valid(panelInfo.traceInfo) then begin
  ;        for j = 0,n_elements(*panelInfo.traceInfo)-1 do begin
  ;
  ;          traceInfo = (*panelInfo.traceInfo)[j]
  ;          ptr_free,traceInfo.refData
  ;          if obj_valid(traceinfo.textObj) then obj_destroy,traceinfo.textObj
  ;
  ;        endfor
  ;
  ;        ptr_free,panelInfo.traceInfo
  ;      endif
  ;
  ;      if ptr_valid(panelInfo.varInfo) then begin
  ;        for j = 0,n_elements(*panelInfo.varInfo)-1 do begin
  ;          varInfo = (*panelInfo.varInfo)[j]
  ;          ptr_free,varInfo.dataX
  ;          if obj_valid(traceinfo.textobj) then obj_destroy,varInfo.textobj
  ;        endfor
  ;
  ;        ptr_free,panelInfo.varInfo
  ;      endif
  ;
  ;      if ptr_valid(panelInfo.markerInfo) then begin
  ;        for j = 0,n_elements(*panelInfo.markerInfo)-1 do begin
  ;          markerInfo = (*panelInfo.markerInfo)[j]
  ;          if obj_valid(markerInfo.frames[0]) then obj_destroy,markerInfo.frames[0]
  ;          if obj_valid(markerInfo.frames[1]) then obj_destroy,markerInfo.frames[1]
  ;        endfor
  ;        ptr_free,panelInfo.markerInfo
  ;      endif
  ;
  ;      if obj_valid(panelInfo.vBar) then obj_destroy,panelInfo.vBar
  ;      if obj_valid(panelInfo.marker) then obj_destroy,panelInfo.marker
  ;      if obj_valid(panelInfo.annotation) then obj_destroy,panelInfo.annotation
  ;      if obj_valid(panelInfo.view) then obj_destroy,panelInfo.view
  ;      if obj_valid(panelInfo.xobj) then obj_destroy,panelInfo.xobj
  ;      if obj_valid(panelInfo.yobj) then obj_destroy,panelInfo.yobj
  ;      if obj_valid(panelInfo.legendModel) then obj_destroy,panelInfo.legendModel
  ;    endfor
  ;    heap_free,self.panelInfo
  ;
  ; endif
  
  ptr_free,self.panelInfo
  self.panelViews->remove,/all
  
  ;no panels in the current display.  Terminate without error
  if ~obj_valid(panelObjs[0]) then begin
    self->createInstance
    error = 0
    return
  endif
  
  layoutDims = [nRows,nCols]
  ;locked=-1
  if ~self->updatePanels(layoutDims,margins,panelObjs,loadedData,backgroundcolor,locked,activeWindow) then begin
    self.statusbar->update,'Draw Object Error: Updating panels, Please check History'
    self.historyWin->update,'Draw Object Error: Updating panels, Please check History'
    self->nukeDraw
    self->createInstance
    return
  endif
  
  ;self.destination->draw,self.scene,/create_instance
  
  ;self.pageView->setProperty,hide=1
  ;self.panelViews->setProperty,hide=1
  
  self->createInstance
  
  error = 0
  
end

;This routine
;1. translates data from linear to log space, if necessary
;2. 
;
;
;xPtr,yPtr,zPtr are arrays of pointers to triplets and couplets of data values, from various traces 
;If zPtr is null, this is a line trace, if not, this is a spectral trace
;
;NOTES:
; This routine is a partial replacement for thm_ui_xyclip, which became unwieldy to maintain
; as the reponsibilities of the routine diverged.
pro thm_ui_draw_object::xclip,xPtr,yPtr,zPtr,range,scale,mirrorptr=mirrorptr,fail=fail

  compile_opt idl2

  fail = 1
  
  ;allocate output arrays
  outx = ptrarr(n_elements(xPtr))
  outy = ptrarr(n_elements(yPtr))
  outz = ptrarr(n_elements(zptr))
  
  if keyword_set(mirrorptr) then begin
    outmirror = ptrarr(n_elements(mirrorptr))
  endif
  
  ;loop over traces
  for i = 0,n_elements(xPtr)-1 do begin
  
    ;skip any traces that are invalid for any reason
    if ~ptr_valid(xPtr[i]) || $
       ~ptr_valid(yPtr[i]) then continue
   
    ;temporary allocations for the purpose of this algorithm
    x = temporary(*xPtr[i])
    y = temporary(*yPtr[i])
  
    if keyword_set(mirrorptr) && $
       ptr_valid(mirrorptr[i]) then begin
       mirror = temporary(*mirrorptr[i])
    endif else begin
       mirror = 0
    endelse  
    
    if ptr_valid(zPtr[i]) then begin
      z = temporary(*zPtr[i])
    endif else begin
      z = 0
    endelse
    
    ;apply logarithmic scaling to x values, out of range values will be marked by translation in -Inf, then removed when we screen out of range values
    if scale eq 1 then begin
      x = alog10(x)
    endif else if scale eq 2 then begin
      x = alog(x)
    endif
    
    ;sort x if spectral quantity
    if keyword_set(z) then begin
    
      sortidx = bsort(x)
      x = x[sortidx]
      
      if ndimen(y) eq 2 then begin
        y = y[sortidx,*]
      endif
      
      z = z[sortidx,*]
    
    endif
  
    ;identify out of range values
    idx = where(x lt range[0],c)
    
    if ~keyword_set(z) then begin
      ;add some margin, if possible
      if c gt 1 then begin
        idx = idx[0:n_elements(idx)-2]
        x[idx] = !values.d_nan
      endif
;      if c gt 0 then begin
;        x[idx] = !values.d_nan
;      endif
    endif else begin
      if c eq n_elements(x) then begin
        x[idx] = !values.d_nan
      endif else if c gt 1 then begin
        ;if we can, add a little bit of margin to the spectral plot
        ;this way we won't have blank edges when we zoom
        idx = idx[0:c-2]
        x[idx] = !values.d_nan
      endif
    endelse
    
    idx = where(x gt range[1],c)
    
    if ~keyword_set(z) then begin
;     if c gt 0 then begin
;       x[idx] = !values.d_nan
;     endif
      ;this code adds a little bit of margin
      ;to prevent gaps from clipping
      if c gt 1 then begin
        idx = idx[1:n_elements(idx)-1]
        x[idx] = !values.d_nan
      endif
    endif else begin
      if c eq n_elements(x) then begin
        x[idx] = !values.d_nan
      endif else if c gt 1 then begin
        ;if we can, add a little bit of margin to the spectral plot
        ;this way we won't have blank edges when we zoom
        idx = idx[1:c-1]
        x[idx] = !values.d_nan
      endif
    endelse
     
    ;now clip out of range values
    
    idx = where(finite(x),c)
     
    ;if no legitimate values are available, then the outptrs are left as null
    if c gt 0 then begin
    
      x = x[idx]
    
      if keyword_set(z) then begin
         
        if ndimen(y) eq 2 then begin
          y = y[idx,*]
        endif

        z = z[idx,*]
        
        outz[i] = ptr_new(temporary(z))
    
      endif else begin
   
        y = y[idx]
        
        if keyword_set(mirror) then begin
          mirror = mirror[idx]
          outmirror[i] = ptr_new(temporary(mirror))
        endif
    
      endelse
      
      outx[i] = ptr_new(temporary(x))
      outy[i] = ptr_new(temporary(y))
    endif
    
  endfor
  
  ptr_free,xPtr,yPtr,zPtr
  
  if keyword_set(mirrorptr) then begin
    ptr_free,mirrorPtr
    mirrorPtr = outmirror
  endif
  
  xPtr = outx
  yPtr = outy
  zPtr = outz
  
  fail = 0
       
end

;This function uses some index magic to turn all
;The data values greater(or less) than the limit
;into NaNs,  data is assumed to be a 2-dimensional array
;each element of data[i,*] should be sorted in descending order.
;The magic comes in because the index smallest index greater than the limit
;for each column should not be turned into NaN to give a small margin
;on spectral plots, thus preventing the gaps at the edge of the plot from clipping.
;This is quite tricky to do without looping for i, because the number of elements
;that are over the limit in each i-column will vary for each i.
;It might be possible to select a limit large that the available, such that
;the same margin is added, but I've determined the identifying this number for each
;column would be even harder than index-magic.
;
;The margin is not overly problematic because it is resolved by clipping
;on the final spectral image.  So a little bit of margin only increases the
;computation time of the final image by a managable amount.  The computational 
;burden is only prohibitive when there is extreme zoom on a single point.  Future
;versions of makeSpec should resolve this problem using sampling rather than 
;interpolation.
;You might ask, "why clip at all", but if we did no clipping the interpolation could
;be much more costly.
function thm_ui_draw_object::indexMagic,data,idx,less=less

  compile_opt idl2
  
  ;if we have 1 offending value, it is swallowed by the margin
  ;any more than that, and there is a chance that we'll need
  ;to NaN-clip
  if n_elements(idx) gt 1 then begin

    ;turn the 1-d indices into pairs of 2-d indices
    idx2 = array_indices(data,idx)
    
    ;sort on x-value
    ;This is the central trick. A side effect of a stable sort
    ;on pre-sorted inputs, is that the values for each individual
    ;x-will now be sorted.  Because IDL:'sort' is not stable, 'bsort'
    ;must be used by this algorithm
    srt2 = bsort(idx2[0,*])
    
    ;now reorder indices in order of ascending x indices
    idx2 = idx2[*,srt2]
    
    ;this block identifies all the indicies that are not the minimum(or max) in a column
    if keyword_set(less) then begin
      idx3 = where((shift(idx2[0,*],1)-idx2[0,*]) eq 0,c)
    endif else begin
      idx3 = where((shift(idx2[0,*],-1)-idx2[0,*]) eq 0,c)
    endelse
    
    ;if any exist we return our magic indexes
    if c gt 0 then begin
   
      return,idx2[*,idx3]
  
    endif
    
  endif
  
  return,-1

end

pro thm_ui_draw_object::yclip,xPtr,yPtr,zPtr,range,scale,mirrorptr=mirrorptr,fail=fail

  compile_opt idl2
  
  fail = 1
  
  ;allocate output arrays
  outx = ptrarr(n_elements(xPtr))
  outy = ptrarr(n_elements(yPtr))
  outz = ptrarr(n_elements(zptr))
  
  if keyword_set(mirrorptr) then begin
    outmirror = ptrarr(n_elements(mirrorptr))
  endif
  
  for i = 0,n_elements(yPtr)-1 do begin
  
    ;skip any traces that are invalid for any reason
    if ~ptr_valid(xPtr[i]) || $
       ~ptr_valid(yPtr[i]) then continue
       
    ;temporary allocations for the purpose of this algorithm
    x = temporary(*xPtr[i])
    y = temporary(*yPtr[i])
  
    if keyword_set(mirrorptr) && $
       ptr_valid(mirrorptr[i]) then begin
       mirror = temporary(*mirrorptr[i])
    endif else begin
       mirror = 0
    endelse   
    
    if ptr_valid(zPtr[i]) then begin
      z = temporary(*zPtr[i])
    endif else begin
      z = 0
    endelse
    
    ;sort 1 dimensional y-axis....I'm not sure what to do with 2-d ones here
    ;This is descending, as per the convention in tplot/themis data
    ;If the y is ascending, the data will have vertical inversion and probably other problems
    if keyword_set(z) && ndimen(y) eq 1 then begin
    
      idx = reverse(bsort(y))
      
      y = y[idx]
      z = z[*,idx]
    
    endif
       
    ;apply logarithmic scaling to x values, out of range values will be marked by translation in -Inf, then removed when we screen out of range values
    if scale eq 1 then begin
      y = alog10(y)
    endif else if scale eq 2 then begin
      y = alog(y)
    endif 
    
    if keyword_set(mirror) then begin
      if scale eq 1 then begin
        mirror = alog10(mirror)
      endif else if scale eq 2 then begin
        mirror = alog(mirror)
      endif
      
;      idx = where(mirror lt range[0] or mirror gt range[1],c)
;      
;      if c ne 0 then begin
;        mirror[idx] = !VALUES.D_NAN
;      endif
      
      outmirror[i] = ptr_new(temporary(mirror))
      
    endif

    if ~keyword_set(z) then begin
    
;      idx = where(y lt range[0] or y gt range[1],c)
;      if c gt 0 then begin
;        y[idx] = !VALUES.D_NAN
;      endif
      
    endif else begin
    
      ;Spectral plots are created
      ;using bin-centers.  So if we clip to the range
      ;we clip out 1/2 a bin.  This code will(eventually) clip the
      ;range, but it will leave 1-bin of margin

      ;this code assumes that the y-axis is sorted and descending, which happens to
      ;be the convention used with tplot variables.    
      idx = where(y lt range[0],c)
      
      if c eq n_elements(y) then continue
      
      if c gt 1 then begin
      
        if ndimen(y) eq 1 then begin
        
          idx = idx[1:c-1]    
          y[idx] = !VALUES.D_NAN
          z[*,idx] = !VALUES.D_NAN
        
        endif else begin
          ;this code should perform the same operation as was performed on the 1-d data
          ;but it performs it for each column of y's
          idx2 = self->indexMagic(y,idx,/less)
          
          ;it returns 2-d indices, so we should expect at least 2 elements if we have any clipping to do.
          if n_elements(idx2) gt 1 then begin
         
            y[idx2[0,*],idx2[1,*]] = !VALUES.D_NAN
            z[idx2[0,*],idx2[1,*]] = !VALUES.D_NAN
            
          endif
        
        endelse
      
      endif
      
      idx = where(y gt range[1],c)
      
      if c eq n_elements(y) then continue
      
      if c gt 1 then begin
      
        if ndimen(y) eq 1 then begin
        
          idx = idx[0:c-2]    
          y[idx] = !VALUES.D_NAN
          z[*,idx] = !VALUES.D_NAN
        
        endif else begin
        
         ;this code should perform the same operation as was performed on the 1-d data
          ;but it performs it for each column of y's
          idx2 = self->indexMagic(y,idx)
          
          ;it returns 2-d indices, so we should expect at least 2 elements if we have any clipping to do.
          if n_elements(idx2) gt 1 then begin
         
            y[idx2[0,*],idx2[1,*]] = !VALUES.D_NAN
            z[idx2[0,*],idx2[1,*]] = !VALUES.D_NAN
            
          endif
          
        endelse
           
      endif
 

      
      ;now clip.  Why clip?  Performance improves significantly if we can pare down the data-set
      if ndimen(y) eq 1 then begin
      
        idx = where(finite(y),c)
           
        ;don't draw if there are no valid points
        if c eq 0 then continue
      
        y = y[idx]
        z = z[*,idx]
      
      endif else begin
      
        ;this expression will only have a nan at any position if every value in the row at that position is NaN 
        max_val = max(y,dimension=1,/nan)
      
        ;interior rows should not be removed, unless their neighboring rows are also all NaNs
        ;this fun code identifies these rows
        ;This vector technique operates something like the game minesweeper
        idx = where(finite(max_val),c,complement=comp)
        
        if c ne 0 then begin
          max_val[idx] = 1
        endif
        
        if c ne n_elements(max_val) then begin
          max_val[comp] = 0
        endif
        
        ;0 pad to prevent the computation from getting messed up
        row_indicator = [0,max_val,0]
        
        row_indicator = shift(row_indicator,1)+row_indicator+shift(row_indicator,-1)
        
        ;clip off the padding
        row_indicator = row_indicator[1:n_elements(row_indicator)-2]
         
        valid = where(row_indicator gt 0,c)
        
        if c eq 0 then begin
          ;don't draw if there are no valid points
          continue
        endif
        
        ;remove any leading rows that are all nans
        if valid[0] eq 0 && max_val[0] eq 0 && n_elements(valid) gt 1 then begin
          valid = valid[1:n_elements(valid)-1]
        endif 
        
        ;remove any trailing rows that are all nans
        if valid[n_elements(valid)-1] eq n_elements(max_val)-1 && max_val[n_elements(max_val)-1] eq 0 && n_elements(valid) gt 1 then begin
          valid = valid[0:n_elements(valid)-2]
        endif
        
        ;clip the rows off
        y = y[*,valid]
        z = z[*,valid]
         
      endelse
    
      outz[i] = ptr_new(temporary(z))
    
    
    endelse   
    
    outx[i] = ptr_new(temporary(x))
    outy[i] = ptr_new(temporary(y))
  
  endfor
  
  ptr_free,xPtr,yPtr,zPtr
  
  if keyword_set(mirrorptr) then begin
    ptr_free,mirrorPtr
    mirrorPtr = outmirror
  endif
  
  xPtr = outx
  yPtr = outy
  zPtr = outz
  
  fail = 0

end

pro thm_ui_draw_object::lockTop,panel

  compile_opt idl2

  panel->getProperty,xaxis=xaxis,markers=markers
  
 ; panel->setProperty,showvariables = 0
  
  if obj_valid(xaxis) then begin
  
    xaxis->getProperty,placeAnnotation=pa
    
    
    if ~pa then begin
      xaxis->setProperty,showlabels = 0
      xaxis->setProperty,annotateAxis = 0
    endif 
  
    xaxis->setProperty,$ ; tickStyle=0, $
                       showdate=0 
   
  endif
  
  ;If marker title is below the axis, then move it to the middle
  if obj_valid(markers) then begin
    marker_list = markers->get(/all)
    
    if ~is_num(marker_list[0]) then begin
      for i = 0,n_elements(marker_list)-1 do begin
      
        marker_list[i]->getProperty,settings=msettings
        
        if obj_valid(msettings) then begin
          msettings->getProperty,vertPlacement=vert
          
          if vert eq 6 then begin
            msettings->setProperty,vertPlacement=3
          endif
        endif
        
      endfor
    endif
  endif
  
end

pro thm_ui_draw_object::lockMiddle,panel

  compile_opt idl2

  panel->getProperty,xaxis=xaxis,settings=settings,markers=markers
  
 ; panel->setProperty,showVariables = 0
  
  if obj_valid(xaxis) then begin

    xaxis->setProperty,$ ;tickStyle=0, $
                     showdate=0,$
                     showlabels=0,$
                     annotateAxis=0
  endif

  if obj_valid(settings) then begin
    settings->getProperty,titleobj=title
   
    if obj_valid(title) then begin
      title->setProperty,show=0
    endif
  
  endif 
  
   ;If marker title is below the axis, then move it to the middle
  if obj_valid(markers) then begin
    marker_list = markers->get(/all)
    
    if ~is_num(marker_list[0]) then begin
      for i = 0,n_elements(marker_list)-1 do begin
      
        marker_list[i]->getProperty,settings=msettings
        
        if obj_valid(msettings) then begin
          msettings->getProperty,vertPlacement=vert
          
          if vert eq 6 || vert eq 0 then begin
            msettings->setProperty,vertPlacement=3
          endif
        endif
        
      endfor
    endif
  endif
  
end

;helper method.  Locks the bottom panel in a layout
pro thm_ui_draw_object::lockBottom,panel

  compile_opt idl2
  
  panel->getProperty,xaxis=xaxis,settings=settings,markers=markers
  
  if obj_valid(xaxis) then begin
  
    xaxis->getProperty,placeAnnotation=pa
  
    if pa then begin
      xaxis->setProperty,placeAnnotation=0
    endif 
  
  ;  xaxis->setProperty,tickStyle=0
    
   ; xaxis->setProperty,
   ; xaxis->setProperty,annotateAxis = 1   
   ; xaxis->setProperty,showdate=1
    
  endif
  
  
  if obj_valid(settings) then begin
    settings->getProperty,titleobj=title
    
    if obj_valid(title) then begin    
      title->setProperty,show=0
    endif
  
  endif
  
  ;If marker title is below the axis, then move it to the middle
  if obj_valid(markers) then begin
    marker_list = markers->get(/all)
    
    if ~is_num(marker_list[0]) then begin
      for i = 0,n_elements(marker_list)-1 do begin
      
        marker_list[i]->getProperty,settings=msettings
        
        if obj_valid(msettings) then begin
          msettings->getProperty,vertPlacement=vert
          
          if vert eq 6 || vert eq 0 then begin
            msettings->setProperty,vertPlacement=3
          endif
        endif
        
      endfor
    endif
  endif
  
end

;draws the panels for this window
;margins is a 5 element array with elements
;[left,right,top,bottom,internal] in pixels
;layoutdims = [nRows,nCols]
;panelObjs is a list of the panel objects to be drawn
;loaded data is a reference to the loaded data object
;NOTE:  The order in which various elements are added to the display is IMPORTANT
;If you change the ordering be sure to check that this change hasn't oscured some
;important feature.
function thm_ui_draw_object::updatePanels,layoutDims,margins,panelObjs,loadedData,backgroundcolor,locked,window

  compile_opt idl2
  
  systm = systime(/seconds)
  
  panelTitleVposFactor = .1
  
  dataXptr = ptr_new()
  dataYptr = ptr_new()
  dataZptr = ptr_new()
  
  marginsNorm = [self->pt2norm(margins[0],0),$ ;left
    self->pt2norm(margins[1],0),$ ;right
    self->pt2norm(margins[2],1),$ ;top
    self->pt2norm(margins[3],1),$ ;bottom
    self->pt2norm(margins[4],0),$ ;internal margin x
    self->pt2norm(margins[5],1)]  ;internal margin y
    
  ;these structures serve as a quick reference for performing fast graphical operations
  panelInfoStruct = {xplotpos:[0D,1D],$ ;position of the plot
    yplotpos:[0D,1D],$
    margins:dblarr(5),$ ; size of edge components.  Used for identifying location of click in panel [left,right,top,bottom,vars]
    locked:0,$
    lockedRange:[0D,1D],$ ; alternative range to use for plot scaling, in the event of locked panels
    lockedScale:0,$; alternative scaling to use
    xrange:[0D,1D],$  ;data range of the plot
    yrange:[0D,1D],$
    zrange:[0D,1D],$
    xmajorSize:0D,$ ;size of a major tick in converted data space(log applied if logarithimic)
    ymajorSize:0D,$
    xscale:0,$ ;indicates log or normal scaling
    yscale:0,$
    xisTime:0,$ ;indicates whether the axis is time
    yisTime:0,$
    hasSpec:0,$ ;boolean, indicates whether a spectrogram is present
    hasLine:0,$ ;boolean, indicates whether a line is present
    zscale:0,$ ;indicates log/lin scaling for z axis
    zisTime:0,$ ;indicates whether the z axis is a time(I'm not sure a z time axis is possible)
    zplacement:4,$ ;0:top,1:bottom,2:left,3:right,4:no zaxis
    markerIdx:-1,$ ; indicates the index the highlighted marker(if any) on this panel
    traceInfo:ptr_new(),$ ;ptr to array of info for each trace
    varInfo:ptr_new(),$ ; ptr to array of variable info
    markerInfo:ptr_new(),$ ; ptr to array of marker info
    vBar:obj_new(),$ ;ptr to vertical bar model
    hBar:obj_new(),$ ;ptr to horizontal bar model
    marker:obj_new(),$ ;ptr to currently drawing marker model
    annotation:obj_new(),$ ;ptr to annotation view
    view:obj_new(),$ ;ptr to main view
    xobj:obj_new(),$ ;ptr to independent variable 1 legend text object
    yobj:obj_new(),$ ;ptr to independent variable 2 legend text object
    legendModel:obj_new(), $ ;the model in which the legend is stored
    legendAnnoModel:obj_new() $ ; the model in which the legend annotations are stored
    }
    
  traceInfoStruct = { $
    isSpec:0,$
    ;                     dataX:ptr_new(),$
    ;                     dataY:ptr_new(),$
    ;                     dataZ:ptr_new(),$
    refData:ptr_new(),$
    abcissa:ptr_new(),$
    plotData:ptr_new(), $ ;pointer to a struct in which some information is passed internally. 
    dataName:'',$
    color:[0,0,0],$
    textObj:obj_new() $   ;text object that displays dependent variable text output
    }
    
  varInfoStruct = { $
    dataX:ptr_new(),$ ;ptr to the x data quantity(note that this is normalized data, not real data)
    dataY:ptr_new(),$ ; ptr to the y data quantity
    textObj:obj_new(),$ ; ptr to the text object to be updated during tracking
    isTime:0B, $ ; indicates whether the data y is a time
    annotateStyle:0, $ ; the annotation style for the output
    scaling:0, $ ;0=linear,1=log10,2=log
    range:[0D,1D] $
    }
    
  markerInfoStruct = { $
    pos:[0D,1D], $ ;the x-start & x-stop of the marker in normalized panel coordinates
    color:[0B,0B,0B], $ ; the original color of the frame
    frames:[obj_new(),obj_new()], $ ;the IDLgrPolyline objects for the left and right sides of the marker frame
    selected:0, $  ;indicates whether marker is selected
    displayed:0 $  ;indicates whether the marker is displayed at this level of zoom
    }
    
  panelInfoArray = replicate(panelInfoStruct,n_elements(panelObjs))
  
  variableSize = self->pt2norm(self->getVariableSize(panelObjs),1)
  cumulativeVarSize = variableSize ;needed to calculate size & position of panel(ie panel view)
  
  ;calculate the range of the locked panel
  ;so that we can use it on other panels
 ; locked = -1
  if locked ne -1 then begin
  
    if locked ge n_elements(panelObjs) then begin
      self.historyWin->update,'Warning: Locked value requests locking to nonexistent panel'
      self.statusBar->update,'Warning: Locked value requests locking to nonexistent panel'
    endif else begin
      currentPanel = panelObjs[locked]
      currentPanel->getProperty,traceSettings=traceSettings,xAxis=xAxis,yAxis=yAxis,zAxis=zAxis
  
      ;reverse the order of trace processing so that spectrograms layer correctly
      traces = reverse(traceSettings->get(/all))
      
      ;extract and process data
      if obj_valid(traces[0]) then begin
      
        yaxis->getProperty,scaling=yscale
        
        self->collateData,traces,loadedData,yscale=yscale,outXptrs=dataXptr,outYptrs=dataYptr,outZptrs=dataZptr,mirror=mirrorptr,fail=fail,dataNames=dataNames,dataidx=dataidx
        
        if fail then begin
          self.historyWin->update,'Failed to read data for locking master panel'
          return,0
        endif
        
        if dataidx[0] ne -1 then begin
        
          ;some data quantities may turn out invalid, so we remove those
          traces = traces[dataidx]
          dataXptr = dataXptr[dataidx]
          dataYptr = dataYptr[dataidx]
          dataZptr = dataZptr[dataidx]
          mirrorptr = mirrorptr[dataidx]
          datanames = datanames[dataidx]
          
          ;get plot range
          
          self->getRange,dataXptr,xAxis,range=lockedRange,scaling=lockedScale,fail=fail
          
          if fail then begin
            self.historyWin->update,'No range on locking master panel.'
            return,0
          endif else begin
            ptr_free,dataXptr,dataYptr,dataZptr,mirrorPtr
          endelse
          
         endif else begin
           self.historyWin->update,'No valid data on locking master panel.'
           lockedRange = [0D,1D]
           lockedScale = 0
         endelse
       endif else begin
         self.historyWin->update,'No valid traces on locking master panel'
         lockedRange = [0D,1D]
         lockedScale = 0
       endelse
    endelse
  endif
  
  for i = 0,n_elements(panelObjs) - 1 do begin
    
    ;print,"panel:",i,systime(/seconds)-systm
    
    currentPanel = panelObjs[i]
    
    ;if we're locked, then change settings appropriately
    if locked ne -1 then begin
    
      panelLayoutPos = window->getPanelPos(currentPanel)
      
      if panelLayoutPos eq -1 then begin
        self.historyWin->update,'Problem evaluating locked panel layout'
        self.statusBar->update,'Problem evaluating locked panel layout'
        return,0
      endif else if panelLayoutPos eq 1 then begin
        self->lockBottom,currentPanel
      endif else if panelLayoutPos eq 2 then begin
        self->lockMiddle,currentPanel
      endif else if panelLayoutPos eq 3 then begin
        self->lockTop,currentPanel
      endif 
    
    end
  
    currentPanel->getProperty,settings=settings,markers=markerContainer,variables=variableContainer,showvariables=showvariables
    settings->getProperty,row=row,col=col,rSpan=rSpan,cSpan=cSpan, $
      backgroundcolor=color,titleobj=titleobj,framecolor=framecolor, $
      framethick=framethick,titleMargin=titleMargin
      
    layoutPos = [row,col,rSpan,cSpan]
    
    pcoord = currentPanel->getPanelCoordinates()
    
    ;normalize coordinates
    pcoord = [(pcoord[0] ne -1)?self->pt2norm(pcoord[0],0):-1,$
      (pcoord[1] ne -1)?self->pt2norm(pcoord[1],1):-1,$
      (pcoord[2] ne -1)?self->pt2norm(pcoord[2],0):-1,$
      (pcoord[3] ne -1)?self->pt2norm(pcoord[3],1):-1]
      
    markerNum = 0
    if obj_valid(markerContainer) && obj_isa(markerContainer,'IDL_Container') then begin
      markerList = markerContainer->get(/all)
      
      if obj_valid(markerList[0]) then begin
        markerNum = n_elements(markerList)
        markerInfo = replicate(markerInfoStruct,markerNum)
      endif
    endif
    
    varNum = 0
    ;showvariables = 1
    
    if obj_valid(variableContainer) && $
      obj_isa(variableContainer,'IDL_Container') && $
      showvariables then begin
      varList = variableContainer->get(/all)
      
      if obj_valid(varList[0]) then begin
        varNum = n_elements(varList)
        varInfo = replicate(varInfoStruct,varNum)
        panelInfoArray[i].varInfo = ptr_new(varInfo)
      endif
    endif
    
    currentVarSize = self->pt2norm(self->getVariableSize(currentPanel),1)
    
    ;I'd very much like to create the returned views by calling this routine each time I need a view,(rather than returning multiple identical items from 1 call)
    ;but for some reason the transparency doesn't work if I do that.  It seems very odd, although maybe I'm missing something obvious.
    ;(like mutating an input so that the second call produces differing results)
    self->makeView,$
      layoutDims,$
      marginsNorm,$
      layoutPos,$
      pcoord,$
      markernum,$
      variablesize,$
      currentvarSize,$
      cumulativeVarSize,$
      view=view,$
      annotation=annotation,$
      markers=markerviews,$
      xplotPos=xplotPos,$
      yplotpos=yplotpos,$
      outmargins=outmargins,$
      fail=fail
      
    cumulativeVarSize-=currentVarSize
    
    if fail then return,0
    
    panelInfoArray[i].annotation=annotation
    panelInfoArray[i].view=view
    panelInfoArray[i].margins=outmargins
    
    newsize = self->getpanelsize(xplotpos,yplotpos)
    currentPanel->setPanelCoordinates,newsize
    
    ;add views to graphics tree
    self.panelViews->add,view
    self.panelViews->add,annotation
    
    ;add static components to the list of static views
    self.staticViews->add,view
    ;add animated components to the list of dynamic views
    self.dynamicViews->add,annotation
    
    ;title
    
    if obj_valid(titleobj) then begin
    
      grTitleObj = self->getTextObject(titleobj,[.5,self->pt2norm(titleMargin,1)/(yplotpos[1]-yplotPos[0])+1.,.5],1,0)
      
      grTitle = obj_new('IDLgrModel')
      
      grTitle->add,grTitleObj
      
      view->add,grTitle
      
    endif
    
    ;get data quantities
    
    currentPanel->getProperty,traceSettings=traceSettings,xAxis=xAxis,yAxis=yAxis,zAxis=zAxis
    
    ;reverse the order of trace processing so that spectrograms layer correctly
    traces = reverse(traceSettings->get(/all))
    
    ;extract and process data
    if obj_valid(traces[0]) then begin
    
      yaxis->getProperty,scaling=yscale
      
      self->collateData,traces,loadedData,yscale=yscale,outXptrs=dataXptr,outYptrs=dataYptr,outZptrs=dataZptr,mirror=mirrorptr,fail=fail,dataNames=dataNames,dataidx=dataidx
      
      if fail then return,0
      
      if dataidx[0] ne -1 then begin
      
        ;some data quantities may turn out invalid, so we remove those
        traces = traces[dataidx]
        dataXptr = dataXptr[dataidx]
        dataYptr = dataYptr[dataidx]
        dataZptr = dataZptr[dataidx]
        mirrorptr = mirrorptr[dataidx]
        datanames = datanames[dataidx]
        
        ;get plot range
        
        self->getRange,dataXptr,xAxis,scaling=xscaling,range=xrange,fail=fail

        ;translate the range into the scaling used by this trace
        if size(lockedRange,/type) then begin
        
          panelInfoArray[i].locked = 1
          panelInfoArray[i].lockedRange = lockedRange
          panelInfoArray[i].lockedScale = lockedScale
        
          currentRange = lockedRange
          currentScale = lockedScale
        
        endif else begin
          currentRange = xrange
          currentScale = xscaling
        endelse
        
        if fail then begin
        
          traceInfoArray = 0
          xrange = [0D,1D]
          if size(lockedRange,/type) then begin
            currentRange=lockedRange
            currentScale = lockedScale
          endif else begin
            currentRange = [0D,1D]
            currentScale = 0
          endelse
          yrange = [0D,1D]
          yscale = 0
          zrange = [0D,1D]
          traces = 0
          
        endif else begin
        
          panelInfoArray[i].xrange=xrange
          panelInfoArray[i].xscale=xscaling
          
          self->xclip,dataXptr,dataYptr,dataZptr,currentRange,currentScale,fail=fail,mirrorptr=mirrorptr
          
          ;   thm_ui_xyclip,dataXptr,dataYptr,dataZptr,xrange,xscaling,fail=fail,transposez=0,mirrorptr=mirrorptr
          
          if fail then begin
          
            traceInfoArray = 0
            yrange = [0D,1D]
            yscale = 0
            zrange = [0D,1D]
            traces = 0
            
          endif else begin
          
            self->getRange,dataYptr,yAxis,scaling=yscaling,mirror=mirrorptr,range=yrange,fail=fail
            
            if fail then begin
            
              traceInfoArray = 0
              traces = 0
              yrange = [0D,1D]
              zrange = [0D,1D]
              
            endif else begin
            
              panelInfoArray[i].yrange=yrange
              panelInfoArray[i].yscale=yscaling
              
              self->yclip,dataXptr,dataYptr,dataZptr,yrange,yscaling,fail=fail,mirrorptr=mirrorptr
          
              
              ; thm_ui_xyclip,dataYptr,dataXptr,dataZptr,yrange,yscaling,fail=fail,transposez=1,yaxis=1,mirrorptr=mirrorptr
              ;thm_ui_xyclip,mirrorptr,dataXptr,dataZptr,yrange,yscaling,fail=fail,transposez=1,keepnans=1
              
              if fail then begin
                traceInfoArray = 0
                traces = 0
                zrange = [0D,1D]
              endif else begin
              
                ;print,"gotxy range",systime(/seconds)-systm
              
                if obj_valid(zAxis) then begin
                
                  ; panelInfoArray[i].hasSpec = 1
                
                  self->getZRange,dataZptr,zAxis,range=zrange,scaling=zscaling,fixed=zFixed,fail=fail
                 
                  if fail then begin
                    traceInfoArray = 0
                    traces = 0
                    zrange = [0D,1D]
                  endif else begin
                  
                    panelInfoArray[i].zscale=zscaling
                    panelInfoArray[i].zrange=zrange
                    
                  ;  self->zclip
                    
                    thm_ui_xyzclip,dataZptr,zrange,zscaling,fail=fail
                    
                    if fail then begin
                      traceInfoArray = 0
                      traces = 0
                    endif
                  endelse
                  
                  traceInfoArray = replicate(traceInfoStruct,n_elements(traces))
                  
                  ; traceInfoArray[*].isSpec = 1
                  ;        traceInfoArray[*].dataX = dataXptr
                  ;        traceInfoArray[*].dataY = dataYptr
                  ;        traceInfoArray[*].dataZ = dataZptr
                  if obj_valid(traces[0]) then begin
                    traceInfoArray[*].dataName = dataNames
                  endif
                  
                endif else begin
                
                  traceInfoArray = replicate(traceInfoStruct,n_elements(traces))
                  
                  ; traceInfoArray[*].isSpec = 0
                  ;        traceInfoArray[*].dataX = dataXptr
                  ;        traceInfoArray[*].dataY = dataYptr
                  ;        traceInfoArray[*].dataZ = dataZptr
                  if obj_valid(traces[0]) then begin
                    traceInfoArray[*].dataName = dataNames
                  endif
                endelse
              endelse
            endelse
          endelse
        endelse
        
      endif else begin
        traceInfoArray = 0
        if size(lockedRange,/type) then begin
          currentRange=lockedRange
          currentScale = lockedScale
        endif else begin
          currentRange = [0D,1D]
          currentScale = 0
        endelse
        yrange = [0D,1D]
        yscale = 0
        zrange = [0D,1D]
        traces = 0
      endelse
      
    ;print,"got zrange",systime(/seconds)-systm
      
    endif else begin
      traceInfoArray = 0
      if size(lockedRange,/type) then begin
        currentRange=lockedRange
        currentScale = lockedScale
      endif else begin
        currentRange = [0D,1D]
        currentScale = 0
      endelse
      yrange = [0D,1D]
      yscale = 0
      zrange = [0D,1D]
    endelse
    
    ;draw permanent markers
    if markerNum gt 0 then begin
      zstack = .21 + .0001
      for j = 0,n_elements(markerList)-1 do begin
      
        self->addMarker,markerviews[j],markerList[j],currentrange,zstack,fail=fail,markerpos=markerpos,markerframes=markerframes,markercolor=markercolor,markerSelected=markerSelected
        
        if ~fail then begin
          zstack += .0001
          self.panelViews->add,markerviews[j]
          markerInfo[j].pos = markerpos
          markerInfo[j].frames = markerFrames
          markerInfo[j].color = markercolor
          markerInfo[j].displayed = 1
          markerInfo[j].selected = markerSelected
          ;adding marker view to dynamic view list
          self.dynamicViews->add,markerViews[j]
          
        endif else begin
          markerInfo[j].displayed = 0
          obj_destroy,markerviews[j]
        endelse
        
      endfor
      panelInfoArray[i].markerInfo = ptr_new(markerInfo)
      
    endif
    
    ;make axes
    
    ;DEFSYSV,'!VIEW',view
    
    self->makeXYaxisModel,0,currentrange,yrange,currentScale,xAxis,yplotpos,xplotpos,framecolor,framethick,backgroundcolor,model=xAxisModel,gridmodel=gridmodelx,majors=xAxisMajors,isTimeAxis=xistime,labelpos=labelpos,fail=fail
    
    if fail then return,0
    
    self->makeXYaxisModel,1,currentrange,yrange,yscale,yAxis,xplotpos,yplotpos,framecolor,framethick,backgroundcolor,model=yAxisModel,gridmodel=gridmodely,majors=yAxisMajors,isTimeAxis=yistime,fail=fail
    
    if fail then return,0
    
    if n_elements(xAxisMajors) lt 2 then begin
      xMajorSize = currentrange[1]-currentrange[0]
    endif else begin
      xMajorSize = (currentRange[1]-currentRange[0])*median(xAxisMajors[1:n_elements(xAxisMajors)-1]-xAxisMajors[0:n_elements(xAxisMajors)-2])
    endelse
    
    if n_elements(yAxisMajors) lt 2 then begin
      yMajorSize = yrange[1]-yrange[0]
    endif else begin
      yMajorSize = (yrange[1]-yrange[0])*median(yAxisMajors[1:n_elements(yAxisMajors)-1]-yAxisMajors[0:n_elements(yAxisMajors)-2])
    endelse
    
    panelInfoArray[i].xmajorsize=xmajorsize
    panelInfoArray[i].ymajorsize=ymajorsize
    
    panelInfoArray[i].xistime=xistime
    panelInfoArray[i].yistime=yistime
    
    if obj_valid(zAxis) then begin
    
      zaxis->getProperty,colorTable=paletteNum
  
      if ~keyword_set(zfixed) then begin
        zRangeRecalc = [!VALUES.D_NAN,!VALUES.D_NAN]
      endif
    endif
    
    ;plot
    if obj_valid(traces[0]) then begin
    
      panel_sz_pt = self->getSpecSize(xplotpos,yplotpos)
      
      if self.postscript then begin
        spec_list = obj_new('IDL_Container')
      endif
      
      for j = 0,n_elements(traces)-1 do begin
      
        if obj_isa(traces[j],'thm_ui_line_settings') && $
          ptr_valid(dataXptr[j]) && $
          ptr_valid(dataYptr[j]) then begin
          
          panelInfoArray[i].hasLine = 1
          
          linePlot = self->getLinePlot(traces[j],currentRange,yrange,xplotpos,yplotpos,currentScale,yscaling,xAxisMajors,dataXptr[j],datayPtr[j],panelInfoArray[i].xistime,linecolor=linecolor,mirrorptr=mirrorptr[j],refvar=refvar,abcissa=abcissa)
          if ~obj_valid(linePlot) then begin
            self.statusbar->update,'Error: Could not generate the plot due to internal error'
            self.historyWin->update,'Error: Could not generate the plot due to internal error'
            ;ok = error_message('Invalid line plot, indicates plot generation error',/traceback)
            return,0
          endif
          
          traceInfoArray[j].color = linecolor
          
          if size(refVar,/type) then begin
            traceInfoArray[j].refData = ptr_new(refVar)
          endif
          
          traceInfoArray[j].isSpec = 0
          
          ;only need abcissa values to determine value used for legend
          ;if the data is non-functional, All data that doesn't have
          ;x as the time axis is treated as non-functional
          if size(abcissa,/type) then begin
          ;if keyword_set(abcissa) && ~xistime then begin
            traceInfoArray[j].abcissa = ptr_new(abcissa)
          endif
          
          view->add,linePlot
          
        endif else if obj_isa(traces[j],'thm_ui_spectra_settings') && $
          ptr_valid(dataXptr[j]) && $
          ptr_valid(dataYptr[j]) && $
          ptr_valid(dataZptr[j]) then begin
          
          panelInfoArray[i].hasSpec = 1
          
         ; self->getSpecPlot,traces[j],currentRange,yrange,zrange,panel_sz_pt[0],panel_sz_pt[1],xscaling,yscaling,zscaling,paletteNum,dataXptr[j],datayptr[j],datazptr[j],refvar=refvar,polygon=specplot
          self->getSpecRef,traces[j],currentRange,yrange,panel_sz_pt[0],panel_sz_pt[1],currentScale,yscaling,zscaling,dataXptr[j],datayptr[j],datazptr[j],refvar=refvar,plotdata=plotdata
          
          if size(refvar,/type) ne 0 then begin
            traceInfoArray[j].refData = ptr_new(refVar)
        
            traceInfoArray[j].isSpec = 1
            
          endif
          
          if size(plotData,/type) ne 0 then begin
            traceInfoArray[j].plotData = ptr_new(plotData)
            
            if ~keyword_set(zfixed) then begin
              zRangeRecalc[0] = min([zrangeRecalc[0],min(plotdata.data,/nan)],/nan)
              zRangeRecalc[1] = max([zrangeRecalc[1],max(plotdata.data,/nan)],/nan)
            endif
            
          endif
          
        endif
        
      endfor
      
      ;we need to loop over the spectral data twice.
      ;we do this to recalculate the range, as the data range may be
      ;changed due to aliasing in spectral interpolation
      ;An analogous operation occurs in tplot
      if panelInfoArray[i].hasSpec then begin
      
        if ~keyword_set(zFixed) && finite(zRangeRecalc[0]) && finite(zRangeRecalc[1]) then begin
      
          zRange = zRangeRecalc
      
        endif    
      
        for j = 0,n_elements(traces)-1 do begin
          if traceInfoArray[j].isSpec && ptr_valid(traceInfoArray[j].plotData) then begin
          
            self->getSpecModel,traceInfoArray[j].plotData,zrange,paletteNum,model=specModel
          
            if obj_valid(specModel) then begin
              if ~self.postscript then begin
                view->add,specModel
              endif else begin
                spec_list->add,specModel
              endelse
            endif
          
          endif
        endfor
      
      endif
      
      if self.postscript && spec_list->count() then begin
      
        model = self->aggregateSpecplots(spec_list,panel_sz_pt,color)
        view->add,model
        
      endif
      
    endif
    
    if obj_valid(zAxis) then begin
      self->makeZAxisModel,zrange,zAxis,xplotpos,yplotpos,framecolor,framethick,model=zAxisModel
      zaxis->getProperty,placement=placement
      panelInfoArray[i].zplacement=placement     
    endif
    
    ;  print,"made plot",systime(/seconds)-systm
    
    vBarModel = obj_new('IDLgrModel')
    hBarModel = obj_new('IDLgrModel')
    markerModel = obj_new('IDLgrModel') ;This is the model for markers in the process of being drawn, not completed markers
    panelInfoArray[i].xplotpos=xplotpos
    panelInfoArray[i].yplotpos=yplotpos
    panelInfoArray[i].vBar=vBarModel
    panelInfoArray[i].hBar=hBarModel
    panelInfoArray[i].marker = markerModel
    
    currentPanel->getProperty,labelMargin=labelMargin
    
    if varNum gt 0 then begin
      self->addVariables,view,annotation,varList,panelInfoArray[i],loadedData,xAxisMajors,xAxis,labelpos,labelMargin,varptr=varptr
      ptr_free,panelInfoArray[i].varInfo
      panelInfoArray[i].varInfo=varptr
    endif
    
    ; print,"added info",systime(/seconds)-systm
    
    ;annotation->add,vBarModel
    annotation->add,vBarModel
    annotation->add,hBarModel
    annotation->add,markerModel
    
    ;background
    self->addBackGround,view,color
    self->addDateString,view,xAxis,currentRange,currentScale,xplotpos[1]-xplotpos[0],yplotpos[1]-yplotpos[0],labelMargin
    
    if keyword_set(traceInfoArray) then begin
      paTmp = panelInfoArray[i] ;error occurs if you edit the array in place
      self->addLegend,view,annotation,paTmp,traceInfoArray
      panelInfoArray[i] = paTmp
      ;print,"added other fun stuff",systime(/seconds)-systm
      panelInfoArray[i].traceInfo = ptr_new(traceInfoArray)
    endif else begin
      panelInfoArray[i].traceInfo = ptr_new()
    endelse
    
    ;add axes
    view->add,xAxisModel
    view->add,yAxisModel
    view->add,gridmodelx
    view->add,gridmodely
    
    if obj_valid(zAxis) then begin
      view->add,zAxisModel
    endif
    
    ;  ptr_free,dataXptr,dataYptr,dataZptr
    
    dataXptr = ptr_new()
    dataYptr = ptr_new()
    dataZptr = ptr_new()
    
    heap_gc
  ;print,"did everything",systime(/seconds)-systm
    
  endfor
  
  self.panelInfo = ptr_new(panelInfoArray)
  
  return,1
  
end

;this routine constructs a reference for legend lookup that has
;no more than the number of points on the screen
;x = clipped abcissa values, normalized to fractions of the total
;y = clipped data values, with the same number of elements as x
;n = the number of points requested in the output
;ref = data returned in named variable
;tolerance = the number of point before the data is considered valid.
pro thm_ui_draw_object::makeLineReference,x,y,n,ref=ref,tolerance=tolerance

  compile_opt idl2

  ;m = n_elements(x)
  
  if ~keyword_set(tolerance) then begin
    tolerance = .01
  endif 
  
  minx = min(x,/nan)
  maxx = max(x,/nan)
  
  ref = dblarr(n)
  ref[*] = !VALUES.D_NAN
  
  vals = (dindgen(n)/(n-1) > 0) < 1
  
  datx = [minx-.001,x,maxx+.001]
  
  sort_idx = bsort(datx) ;sort transform
  isort_idx = bsort(sort_idx) ;inverse sort transform
  
  datx = datx[sort_idx]
  
  close_idx = value_locate(datx,vals)
  
  lows = abs((datx)[close_idx]-vals) 
  highs = abs((datx)[close_idx+1]-vals)
  
  idx_lows = where(lows lt highs and lows lt tolerance,count_lows)
  
  if count_lows gt 0 then begin
    ref[idx_lows] = ([!VALUES.D_NAN,y,!VALUES.D_NAN])[isort_idx[close_idx[idx_lows]]]
  endif
  
  idx_highs = where(highs lt lows and highs lt tolerance,count_highs)
  
  if count_highs gt 0 then begin
    ref[idx_highs] = ([!VALUES.D_NAN,y,!VALUES.D_NAN])[isort_idx[close_idx[idx_highs]+1]]
  endif
  
 ; print,n
;  for i = 0D,n-1 do begin
;   
;    mn = min(dat-vals[i],idx,/absolute)
;   
;;    ;this filter makes sure out of range values get listed as NaNs
;    if idx gt 1 && idx le m then begin
;      ref[i] = y[idx-1]
;    endif 
;;      if keyword_set(tolerance) then begin
;;        if mn * n le tolerance then begin
;;          ref[i] = y[idx]
;;        endif
;;      endif else begin
;;        ref[i] = y[idx]
;;      endelse
;;    endif
;    
;  endfor

end

;adds thm_ui_variables to requested display
pro thm_ui_draw_object::addVariables, $
    view, $  ;the view to which they will be added
    annotation,$ ;the view to which the legend text will be added
    vars, $  ;the list of variables to be added
    panelInfo, $ ;the panel info struct
    loadedData, $ ;loaded data object
    xMajors,$  ; the positions of the major ticks on the x axis
    xAxis,$ ;the x axis of the panel
    labelPos,$ ;most extreme position of the labels
    labelMargin,$ ;the distance from the edge of the panel that labels should be placed
    varptr=varptr
    
  compile_opt idl2
  zstack = .1
  
  ;this factor will convert normalized window coords into normalized panel coords
  xpanelConv = panelInfo.xplotpos[1] - panelInfo.xplotpos[0]
  ypanelConv = panelInfo.yplotpos[1] - panelInfo.yplotpos[0]
  spacing = self->pt2norm(2,1)/ypanelConv ; the amount of space between each variable
  
  validmask = intarr(n_elements(vars))
  
  ;number of pixels across the panel * res_factor = number of points in lookup
  xpx = self->getLineSize(panelInfo.xplotpos)
  
  model = obj_new('IDLgrModel')
  anno_model = obj_new('IDLgrModel')
  
  xAxis->getProperty, $  ;should probably also account for annotation orientation, but I'm not gonna stress that at this point
    tickStyle=tickStyle,$
    bottomPlacement=bottomPlacement,$
    majorLength=majorLength,$
    minorLength=minorLength,$
    annotateAxis=annotateAxis,$
    placeAnnotation=placeAnnotation,$
    margin=margin,$
    scaling=xscaling,$
    annotateFirstTick=annotateFirstTick,$
    annotateLastTick=annotateLastTick,$
    annotateTextObject=annotateTextObject,$
    annotateExponent=annotateExponent
    
  xAxisMajors = xMajors
  

  if ~annotateFirstTick then begin
    if n_elements(xAxisMajors) gt 1 then begin
      xAxisMajors = xAxisMajors[1:n_elements(xAxisMajors)-1]
    endif else begin
      noXMajors = 1
    endelse
  endif
  
  if ~annotateLastTick then begin
    if n_elements(xAxisMajors) gt 1 then begin
      xAxisMajors = xAxisMajors[0:n_elements(xAxisMajors)-2]
    endif else begin
      noXMajors = 1
    endelse
  endif
  
  ;if ticks are outside the bottom axis, how long are they?
  if bottomPlacement && (tickStyle eq 1 || tickStyle eq 2) then begin
    tickLoc = self->pt2norm(majorLength > minorLength,1)/ypanelConv
  endif else begin
    tickLoc = 0
  endelse
  
  if annotateAxis && ~placeAnnotation && obj_valid(annotateTextObject) then begin
    labelLoc = abs(labelPos)
    annotateTextObject->getProperty,size=size
    annoLoc = tickLoc +  self->pt2norm(size,1)/ypanelConv
  endif else begin
    annoLoc = tickLoc
    labelLoc = 0
  endelse
  
  yloc = 0. - (labelloc > annoloc) - 3*spacing
  
  varInfoArray = *panelInfo.varInfo
  
  for i = 0,n_elements(vars)-1 do begin
  
    vars[i]->getProperty, $
      fieldname=fieldname,$
      controlname=controlname,$
      text=text,$
      format=format,$
      minRange=minRange,$
      maxRange=maxRange,$
      scaling=vscaling,$
      useRange=useRange,$
      annotateExponent=annotateExponent
      
    text->getProperty,size=size,show=show
    
    if ~show then continue
    
    yloc -= (spacing + self->pt2norm(size,1)/ypanelConv)
    xloc = 0. - self->pt2norm(labelMargin,0)/xpanelConv
    
    textObj = self->getTextObject(text,[xloc,yloc,zstack],1,0,justify=1)

    textObj->setproperty, enable_formatting=1
    
    model->add,textObj
    
    if keyword_set(fieldname) && loadedData->isChild(fieldname) then begin
      loadedData->getVarData,name=fieldname,data=yd,isTime=isTime,/duplicate
      varInfoArray[i].dataY = yd
      varInfoArray[i].isTime = isTime
    endif else begin
      continue
    endelse
    
    if keyword_set(controlname) && loadedData->isChild(controlname) then begin
      loadedData->getVarData,name=controlname,data=xd,/duplicate
      ;varInfoArray[i].dataX = xd
      ;xscaling = vscaling
      xrange = [min(*xd,/nan),max(*xd,/nan)]
    endif else begin
      xd2 = ptr_new(dindgen(n_elements(*yd[0]))/(n_elements(*yd[0])-1))
      varInfoArray[i].dataX = xd2
      xrange = [0D,1D]
    endelse
    
    if userange eq 2 then begin
    
      if panelInfo.locked then begin
        xrange = panelInfo.lockedRange
        xscaling = panelInfo.lockedScale
      endif else begin
        xrange = panelInfo.xrange
        xscaling = panelInfo.xscale
      endelse
      
      vars[i]->setProperty,minrange=xrange[0],maxrange=xrange[1]
    endif else if userange eq 1 then begin
      xrange = [minRange,maxRange]
      xscaling = vscaling
    endif
    
    varInfoArray[i].annotateStyle = format
    varInfoArray[i].scaling = xscaling
    varInfoArray[i].range = xrange
    
    if ptr_valid(xd) then begin
    
      ;should replace with new clip?
      ;should remove interpol?
      ;thm_ui_xyclip,xd,yd,ptr_new(),xrange,xscaling,fail=fail
      self->xclip,xd,yd,ptr_new(),xrange,xscaling,fail=fail
         
      if fail then begin
        self.statusBar->update,'Error: Could not process variable control'
        self.historyWin->update,'Error: Could not process variable control'
        ;ok = error_message('Failure to process variable control:' + controlname)
        continue
      endif
      
      xdata = (temporary(*xd[0])-xrange[0])/(xrange[1]-xrange[0])
      ydata = temporary(*yd[0])
 
      self->makeLineReference,xdata,ydata,xpx,ref=refvar
      
;      
      ;calculate an evenly spaced set of reference values to be stored, decreases runtime legend updates
  ;    abcissa_out = dindgen(xpx)/(xpx-1)
    ;  refvar = interpol([!VALUES.D_NAN,yd2,!VALUES.D_NAN],[xd2[0]-.0001D,xd2,xd2[n_elements(xd2)-1]+.0001D],abcissa_out)
      ; refvar = interpol(yd2,xd2,abcissa_out)
      
      ;strip off the extra nan padding now that interpol is done
;      if n_elements(refvar) gt 2 then begin
;        refvar = refvar[1:n_elements(refvar)-2]
;        abcissa_out = abcissa_out[1:n_elements(abcissa_out)-2]
;      endif
      
   ;   xd2 = ptr_new(abcissa_out)
  ;    yd = ptr_new(refvar)
    
   ;   varInfoArray[i].dataX = xd2
      varInfoArray[i].dataY = ptr_new(temporary(refvar))
      
    endif else begin
    
     xdata = dindgen(n_elements(ydata))/n_elements(ydata)
     ydata = temporary(*yd) 
     self->makeLineReference,xdata,ydata,xpx,ref=refvar
    
     varInfoArray[i].dataY = ptr_new(temporary(refvar))
      
    endelse
    
    textTemp = text->copy()
    textTemp->setProperty,value=''
    
    xloc = 1. + self->pt2norm(labelMargin,0)/xpanelconv
    textObj = self->getTextObject(textTemp,[xloc,yloc,zstack],1,0,justify=-1)
    textObj->setproperty, enable_formatting=1
    varInfoArray[i].textObj = textObj
    anno_model->add,textObj
    
    ; xmajornorm = (xAxisMajors-xrange[0])/(xrange[1]-xrange[0])
    annotatedata = {timeAxis:isTime,formatid:format,scaling:0,exponent:annotateExponent}
    
    if ~keyword_set(noXMajors) then begin
      for j = 0,n_elements(xAxisMajors)-1 do begin
      
        temp = min(abs(xdata - xAxisMajors[j]),idx)
        if temp lt .01 then begin ;only use real valued annotation if major tick value close to an actual value
          value = ydata[idx]
        endif else begin
          value = !VALUES.D_NAN
        endelse
      
        
        valueString = formatannotation(0,0,value,data=annotatedata)
        
        textTemp = text->copy()
        textTemp->setProperty,value=valuestring
        
        textObj = self->getTextObject(textTemp,[xAxisMajors[j],yloc,zstack],1,0)
        textObj->setproperty, enable_formatting=1
        model->add,textObj
        
      endfor
    endif
      
    validmask[i] = 1
    
  endfor
  
  idx = where(validmask,c)
  
  if c gt 0 then begin
    varptr=ptr_new(varInfoArray[idx])
    view->add,model
    annotation->add,anno_model
  endif else begin
    varptr=ptr_new()
  endelse
  
end

;calculates the space that the thm_ui_variable displays will occupy
function thm_ui_draw_object::getVariableSize,panels

  compile_opt idl2
  
  total = 0
  spacing = 2
  
  for i = 0,n_elements(panels)-1 do begin
  
    panels[i]->getProperty,variables=variables,showVariables=showvariables
    
    if ~showvariables then continue
    
    ;there is a 2 pt pad before, after,
    ;and between each variable section on the panel
    total += spacing
    
    if obj_valid(variables) && $
      obj_isa(variables,'IDL_Container') then begin
      
      varList = variables->get(/all)
      if obj_valid(varList[0]) then begin
        for j = 0,n_elements(varList)-1 do begin
        
          varList[j]->getProperty,text=text
          if obj_valid(text) then begin
            text->getProperty,size=size,show=show
            if ~show then continue
            total += size + spacing
          endif
          
        endfor
      endif
      
    endif
    
  endfor
  
  return,total
  
end

;This routine add a permanent marker to a view
;view: the view to which the marker should be added
;marker: the marker to be added
;xrange: the xrange of that view(needed to make sure positions of the marker are correctly scaled
;zstack: the height at which the marker should be placed
pro thm_ui_draw_object::addMarker,view,marker,xrange,zstack,fail=fail,markerFrames=markerFrames,markerpos=markerpos,markercolor=markercolor,markerSelected=markerSelected

  compile_opt idl2
  
  fail = 1
  
  textzStack = zstack+.05
  
  marker->getProperty,settings=settings,range=range,isSelected=isSelected
  
  settings->getProperty,$
    fillColor=fillColor,$
    lineStyle=lineStyle,$
    drawOpaque=drawOpaque,$
    label=label,$
    vertPlacement=vertPlacement
    
  ;only draw markers that are in range
  if range[0] ge xrange[1] then return
  
  range[0] = max([range[0],xrange[0]],/nan)
  
  if range[1] le xrange[0] then return
  
  range[1] = min([range[1],xrange[1]],/nan)
  
  xbegin = max([(range[0] - xrange[0])/(xrange[1]-xrange[0]),0D],/nan)
  xend = min([(range[1] - xrange[0])/(xrange[1]-xrange[0]),1D],/nan)
  
  lineStyle->getProperty,$
    id=lineid,$
    show=lineshow,$
    color=linecolor,$
    thickness=linethick
    
  if isSelected then begin
    frameColor = self->hueRotation(self->hueRotation(linecolor))
  endif else begin
    frameColor = linecolor
  endelse
  
  ;border
  lineleft = obj_new('IDLgrPolyline',$
    [xbegin,xbegin],$
    [0.,1.],$
    replicate(zstack,2)+.00001,$
    linestyle=lineid,$
    hide=~lineshow,$
    color=self->convertColor(framecolor),$
    thick=linethick)
    
  lineright = obj_new('IDLgrPolyline',$
    [xend,xend],$
    [0.,1.],$
    replicate(zstack,2)+.00001,$
    linestyle=lineid,$
    hide=~lineshow,$
    color=self->convertColor(framecolor),$
    thick=linethick)
    
    
  ;this creates the shading
  poly = obj_new('IDLgrPolygon',$
    [xbegin,xend,xend,xbegin],$
    [0.,0.,1.,1.],$
    replicate(zstack,4),$
    color=self->convertColor(fillColor),$
    alpha_channel=drawOpaque $
    )
    
  ;now create the label
    
  xpos = (xbegin+xend)/2D
  yposarray = 1D - dindgen(7)*1D/6D
  
  ypos = yposarray[vertPlacement]
  
  if vertPlacement eq 0 then begin
    offset = 1
  endif else if vertplacement eq 6 then begin
    offset = -1
  endif else begin
    offset = 0
  endelse
  
  labelObj = self->getTextObject(label,[xpos,ypos,textZstack],offset,0)
  
  model = obj_new('IDLgrModel')
  
  model->add,lineleft
  model->add,lineright
  
  if ~self.postscript then begin
    model->add,poly
  endif
  
  model->add,labelObj
  
  view->add,model
  
  markerpos = [xbegin,xend]
  markerframes = [lineleft,lineright]
  markercolor = linecolor
  markerSelected = isSelected
  
  
  fail = 0
  
end

;This routine adds a legend to a particular view/panel
;NOTE this routine is currently automatically generating labels
;IT SHOULDN'T BE DOING THIS.  When default labels are being
;correctly set, it should just use the label text object from
;the appropriate axis.
pro thm_ui_draw_object::addLegend,view,annotation,panelInfo,traceInfoArray

  compile_opt idl2
  
  textsize = 8
  spacing = 7.
  zstack = .6
  
  nchars = 0
  numBias = 10 ; number of chars that annotation will take if using numerical output
  timeBias = 17 ; number of chars that annotation will take if using time output
  extraBias = 3 ; number of chars colon and space will take
  ;nlines = 2+total(traceInfoArray[*].isSpec)
  nLines = 2+n_elements(traceInfoArray)
 
  ;calculate the size of legend in chars
  ;the the legend should be as large as the most
  ;characters in any line
  ;Here the bias is the number of characters that will be
  ;contributed by the annotated value itself.(as opposed to the label)
  if panelInfo.xIsTime then begin
    text = 'X Axis Time'
    bias = timeBias
  endif else begin
    text = 'X Axis Value'
    bias = numBias
  endelse
  
  bias += extraBias
  
  nChars = nChars > (strlen(text)+bias)
  
  if panelInfo.yIsTime then begin
    text = 'Y Axis Time'
    bias = timeBias
  endif else begin
    text = 'Y Axis Value'
    bias = numBias
  endelse
  bias += extraBias
  nChars = nChars > (strlen(text)+bias)
    
  if panelInfo.hasSpec then begin
    if panelInfo.zisTime then begin
      bias = timeBias
    endif else begin
      bias = numBias
    endelse
    bias+= extraBias
  endif else begin
    if panelInfo.yistime then begin
      bias = timeBias
    endif else begin
      bias = numBias
    endelse
    bias+= extraBias
  endelse
  
  ;Identify the maximum number of characters in any line
  for i = 0,n_elements(traceInfoArray)-1 do begin
    nChars = nChars > (strlen(traceInfoArray[i].dataName)+bias)
  endfor
  
  ;2 is for colon and one extra
  ; nChars+=annoBias+extrabias
  
  xdiv = panelInfo.xplotpos[1]-panelInfo.xplotpos[0]
  ydiv = panelInfo.yplotpos[1]-panelInfo.yplotpos[0]
  
  ;these estimates assume that a character width is 1/2 its height
  ;this may not be accurate across systems
  width = self->pt2norm(textsize * nchars/2 + 2*spacing,0)/xdiv
  height = self->pt2norm(nlines * textsize + (nlines+1)*spacing,1)/ydiv
  
  ;if the legend is going to be outside of the view, we need to calculate a bias
  ;to shift it down/left so that it will not be clipped.
  view->getProperty,viewPlane_rect=vpr
  if width/2D gt vpr[2]+vpr[0]-1D then begin
    wbias = width/2D - (vpr[2]+vpr[0]-1D)
  endif else begin
    wbias = 0D
  endelse
  
  if height/2D gt vpr[3]+vpr[1]-1D then begin
    hbias = height/2D - (vpr[3]+vpr[1]-1D)
  endif else begin
    hbias = 0D
  endelse
  
  ;
  if self.legendOn eq 1 || self.legendOn eq 2 then begin
    hide_val = 0
  endif else begin
    hide_val = 1
  endelse
  
  model = obj_new('IDLgrModel',hide=hide_val)
  anno_model = obj_new('IDLgrModel',hide=hide_val)
  
  polygon = obj_new('IDLgrPolygon',$
    [1.-width/2.,1+width/2.,1+width/2.,1-width/2.]-wbias,$
    [1.-height/2.,1-height/2.,1+height/2.,1+height/2.]-hbias,$
    [zstack,zstack,zstack,zstack],color=self->convertColor([255,255,255]))
    
  polyline = obj_new('IDLgrPolyline',$
    [1.-width/2.,1+width/2.,1+width/2.,1-width/2.,1.-width/2.]-wbias,$
    [1.-height/2.,1-height/2.,1+height/2.,1+height/2.,1-height/2.]-hbias,$
    [zstack,zstack,zstack,zstack,zstack]+.00001,color=self->convertColor([0,0,0]))
    
  model->add,polyline
  model->add,polygon
  
  xspaceNorm = self->pt2norm(spacing,0)/xdiv
  yspaceNorm = self->pt2norm(spacing,1)/ydiv
  xcharNorm = self->pt2norm(textsize,0)/xdiv
  ycharNorm = self->pt2norm(textsize,1)/ydiv
  
  labelLocation=[1.-width/2. + xspacenorm - wbias,1.+height/2 - yspacenorm - ycharnorm - hbias,zstack+.05]
  valueLocation=[1.+width/2. - xspacenorm - wbias,1.+height/2 - yspacenorm - ycharnorm - hbias,zstack+.05]
  
  if panelInfo.xIsTime then begin
    text = 'X Axis Time'
  endif else begin
    text = 'X Axis Value'
  endelse
  
  ;space pad to line up colons
  text += ' :'
  
  thm_label = obj_new('thm_ui_text',value=text,size=textsize,font=2)
  xlabel = self->getTextObject(thm_label,labelLocation,1,0,justify=-1)
  model->add,xlabel
  
  thm_value = obj_new('thm_ui_text',value='',size=textsize,font=2)
  xvalue = self->getTextObject(thm_value,valueLocation,1,0,justify=1)
  xvalue->setProperty,/enable_formatting
  panelInfo.xobj = xvalue
  anno_model->add,xvalue

  labelLocation[1] -= (yspacenorm+ycharnorm)
  valueLocation[1] -= (yspacenorm+ycharnorm)
  
  if panelInfo.yIsTime then begin
    text = 'Y Axis Time'
  endif else begin
    text = 'Y Axis Value'
  endelse
  
  text += ' :'
  
  thm_label = obj_new('thm_ui_text',value=text,size=textsize,font=2)
  ylabel = self->getTextObject(thm_label,labelLocation,1,0,justify=-1)
  model->add,ylabel
  
  thm_value = obj_new('thm_ui_text',value='',size=textsize,font=2)
  yvalue = self->getTextObject(thm_value,valueLocation,1,0,justify=1)
  panelInfo.yobj = yvalue
  yvalue->setProperty,/enable_formatting
  anno_model->add,yvalue

  ; ycnt = 1
  ; zcnt = 1
  
  ;loop backwards because traces are in reverse order to create proper layering
  for i = n_elements(traceInfoArray)-1,0,-1 do begin
   ; if traceInfoArray[i].isSpec then begin
  
      labelLocation[1] -= (yspacenorm+ycharnorm)
      valueLocation[1] -= (yspacenorm+ycharnorm)
      
      text = traceInfoArray[i].dataName + ' :'
      
      thm_label = obj_new('thm_ui_text',value=text,size=textsize,font=2,color=traceInfoArray[i].color)
      olabel = self->getTextObject(thm_label,labelLocation,1,0,justify=-1)
      model->add,olabel
      
      thm_value = obj_new('thm_ui_text',value='',size=textsize,font=2,color=traceInfoArray[i].color)
      ovalue = self->getTextObject(thm_value,valueLocation,1,0,justify=1)
      traceInfoArray[i].textobj = ovalue
      ovalue->setProperty,/enable_formatting
      anno_model->add,ovalue
  ;    thm_text = obj_new('thm_ui_text',value=text,size=textsize,font=2)
   ; endif
    
  endfor
  
  panelInfo.legendModel = model
  panelInfo.legendAnnoModel = anno_model
  
  view->add,model
  annotation->add,anno_model
  
end

;collects data into arrays and interpolates, if necessary
pro thm_ui_draw_object::collateData,traces,loadedData,yscale=yscale,outXPtrs=outXPtrs,outYPtrs=outYPtrs,outZptrs=outZPtrs,mirror=mirror,fail=fail,dataNames=dataNames,dataidx=dataidx

  compile_opt idl2
  
  fail=1
  
  outXPtrs = ptrarr(n_elements(traces))
  outYPtrs = ptrarr(n_elements(traces))
  outZPtrs = ptrarr(n_elements(traces))
  mirror=ptrarr(n_elements(traces))
  dataNames = strarr(n_elements(traces))
  specmask = intarr(n_elements(traces))
  
  for i = 0,n_elements(traces)-1 do begin
  
    traces[i]->getProperty,dataX=dataX,dataY=dataY
    
    loadedData->getvardata,name=dataX,data=xptr,/duplicate
    loadedData->getvardata,name=dataY,data=yptr,/duplicate
    
    if obj_isa(traces[i],'thm_ui_spectra_settings') then begin
      traces[i]->getProperty,dataZ=dataZ
      
      loadedData->getvardata,name=dataz,data=zptr,/duplicate
      
      if ~ptr_valid(zptr) then continue
      
      d = dimen(*zptr)
      
      ;if y doesn't exist, then create it so that plotting can still be done
      ;The value selected will just space channels evenly.
      if ~keyword_set(dataY) then begin
      
        self.statusBar->update,'Warning: No yaxis quantity is present in spectral plot of: ' + dataZ + '. Y axis will be scaled proportionally.'
        self.historyWin->update,'Warning: No yaxis quantity is present in spectral plot of: ' + dataZ + '. Y axis will be scaled proportionally.'
        
        if n_elements(d) eq 1 then d = [d,1]
        
        if keyword_set(yscale) && yscale eq 1 then begin
          yvaltemp = 10^dindgen(d[1])
        endif else if keyword_set(yscale) && yscale eq 2 then begin
          yvaltemp = exp(dindgen(d[1]))
        endif else begin
          yvaltemp = dindgen(d[1])
        endelse
        
        yptr = ptr_new(transpose(rebin(yvaltemp,d[1],d[0])))
      endif
      
    endif
    
    if ~ptr_valid(xptr) || ~ptr_valid(yptr) then continue
    
    if obj_isa(traces[i],'thm_ui_line_settings') then begin
    
      dataNames[i] = dataY
      
      traces[i]->getProperty,mirrorLine=mirrorLine
      
      outXptrs[i] = xptr
      
      if ndimen(*yptr) gt 1 then begin
        self.statusBar->update,'Cannot have multi-d y-quantities on line plots'
        self.historyWin->update,'Cannot have multi-d y-quantities on line plots'
        return
      endif
      
      if n_elements(*xptr) eq n_elements(*yptr) then begin
        outYptrs[i] = yptr
      endif else begin
      
        self.historyWin->update,'Warning: X axis quantity does not match data quantity for: ' + dataY
        self.statusBar->update,'Warning: X axis quantity does not match data quantity for: ' + dataY
      
        outYptrs[i] = ptr_new(interpol(*yptr,n_elements(*xptr)))
        ptr_free,yptr
      endelse
      
      if mirrorLine then begin
        mirror[i]=ptr_new(-(*outYptrs[i]))
        *mirror[i]=double(*mirror[i])
      endif
      
      ;all data is treated as double precision inside the draw routine
      *outXptrs[i] = double(*outXptrs[i])
      *outYptrs[i] = double(*outYptrs[i])
      
    endif else if obj_isa(traces[i],'thm_ui_spectra_settings') then begin
    
      specmask[i] = 1
      
      dataNames[i] = dataZ
      
      outZptrs[i] = zptr
      
      dim = dimen(*zptr)
      
      if n_elements(dim) eq 1 then begin
        dim = [dim,1]
      endif
      
      if dim[0] eq n_elements(*xptr) then begin
        outXptrs[i] = xptr
      endif else begin
       
        self.historyWin->update,'Warning: X axis quantity does not match data quantity for: ' + dataZ
        self.statusBar->update,'Warning: X axis quantity does not match data quantity for: ' + dataZ
      
        outXptrs[i] = ptr_new(interpol(*xptr,dim[0]))
        ptr_free,xptr
      endelse
      
      dimy = dimen(*yptr)

      ;if the y-dimension is unchanging, we can use a more efficient
      ;draw pipeline
      ;
      ;
      if n_elements(dimy) eq 2 then begin      

;using vectorized implementation
;also note, there may be a bug in this implementation
;If the first value is a NaN, it will always return false
;       change=0
;       
;       for j = 0,dim[1]-1 do begin
;          idx = where((*yptr)[0,j] ne (*yptr)[*,j],c)
;          if c ne 0 then begin
;            change=1
;          endif
;        endfor
;        

        ;if the max is the same as the min in all columns
        ;then the value never changes over time
        max_val = max(*yptr,dimension=1,/nan,min=min_val)
        
        ;remove NaNs before performing check
        ;even with the /nan switch, 'max' can have nans in the output
        ;if every data value along a dimension in NaN
        idx = where(finite(max_val),c)
        
        if c ne 0 then begin
          max_val = max_val[idx]
          min_val = min_val[idx]
        endif

        ;if the data is all NaNs or unchanging, then we can clip down to 1d
        if array_equal(max_val,min_val) || c eq 0 then begin
          tmp = ptr_new(reform((*yptr)[0,*]))
          ptr_free,yptr
          yptr=tmp
          dimy = dimen(*yptr)
        endif
        
      endif
        
      if n_elements(dimy) eq 2 then begin 
        ;if everything matches up, then store the result
        if dim[0] eq dimy[0] && dim[1] eq dimy[1] then begin
          outYptrs[i] = yptr
        endif else begin 
          self.historyWin->update,'Warning: Y axis quantity does not match data quantity for: ' + dataZ
          self.statusBar->update,'Warning: Y axis quantity does not match data quantity for: ' + dataZ
        
         ;otherwise match the elements to the contents of the z-axis data
          outYptrs[i] = ptr_new(interpolate(*yptr,dimy[0]*dindgen(dim[0])/dim[0],dimy[1]*dindgen(dim[1])/dim[1]))
          ptr_free,yptr
        endelse
        
      endif else begin
      
        if dim[1] eq dimy[0] then begin
          outYptrs[i] = yptr
        endif else begin
          outYptrs[i] = ptr_new(interpol(*yptr,dim[1]))
          ptr_free,yptr
        endelse
        
      endelse
        
      
      ;all data is treated as double precision inside the draw routine
      *outXptrs[i] = double(*outXptrs[i])
      *outYptrs[i] = double(*outYptrs[i])
      *outZptrs[i] = double(*outZptrs[i])
      
    endif else begin
      self.statusbar->update,'Error: Draw object was passed illegal trace settings'
      self.historyWin->update,'Error: Draw object was passed illegal trace settings'
      ; ok = error_message('Illegal trace settings passed to draw object',/traceback)
      return
    endelse
    
  endfor
  
  fail = 0
  
  dataidx = where(ptr_valid(outXptrs) and ptr_valid(outYptrs) and (~specmask or ptr_valid(outZptrs)))
  
end

;Generates the specplot during an update
pro thm_ui_draw_object::getSpecRef,trace,xrange,yrange,xpanel_sz_pt,ypanel_sz_pt,xscale,yscale,zscale,dx,dy,dz,refvar=refvar,plotData=plotData

  compile_opt idl2
  
  ; a = systime(/seconds)
  
  polygon = obj_new()
  
  zstack = .05
  
  x = (*dx)
  y = (*dy)
  z = (*dz)
  
  xs = min(x,/nan)
  xe = max(x,/nan)
  ys = min(y,/nan)
  ye = max(y,/nan)
  
  if keyword_set(refVar) then begin
    tmp = temporary(refVar)
  endif
  
  ;0 width range gets special treatment
  if xs eq xe then begin

    ;if the quantity takes up the entire range, then we're in business
    if xrange[0] eq xrange[1] then begin
      xstart = 0
      xsize = 1
    endif else begin
    ;if not, there is no way to figure out how to scale the quantity proportionally
      self.historyWin->update,'Cannot determine how to x-scale 0 range spectrogram'
      self.statusBar->update,'Cannot determine how to x-scale 0 range spectrogram'
      return
    endelse
  
  endif else begin
  
    ;size of the spectral data relative to the visible range
    xstart = (xs - xrange[0])/(xrange[1]-xrange[0])
  
    ;xsize can be greater than 1.  If this occurs the reference variable should have its indices clipped into range
    xsize = (xe-xs)/(xrange[1]-xrange[0])
    
  endelse
  
  ;0 width range gets special treatment
  if ys eq ye then begin
  
    ;if the quantity takes up the entire range, then we're in business
    if yrange[0] eq yrange[1] then begin
      ystart = 0
      ysize = 1
    endif else begin
      self.historyWin->update,'Cannot determine how to y-scale 0 range spectrogram'
      self.statusBar->update,'Cannot determine how to y-scale 0 range spectrogram'
    ;if not, there is no way to figure out how to scale the quantity proportionally
      return
    endelse
  
  endif else begin
  
    ystart = (ys - yrange[0])/(yrange[1]-yrange[0])
    
    ;ysize can be greater than 1.  If this occurs the reference variable should have its indices clipped into range
    ysize = (ye-ys)/(yrange[1]-yrange[0])
  
  endelse
  
  xdata_sz_pt = ceil(xsize*xpanel_sz_pt,/l64)
  ydata_sz_pt = ceil(ysize*ypanel_sz_pt,/l64)
  
  if xdata_sz_pt le 0 || ydata_sz_pt le 0 then return
  
  ;Note that is the long run, this limitation can probably be avoided by adding an algorithm to manually resample the image
  if xdata_sz_pt * ydata_sz_pt gt 2e7 then begin
  
    self.statusBar->update,'Error: Cannot generate spectrogram. The range may be too small.'
    return
    
  endif
  
  ;clip planes will cut off the image at borders
  cp = double([[-1,0,0,0],[1,0,0,-1],[0,-1,0,0],[0,1,0,-1]])
  
  refvar = dblarr(xpanel_sz_pt,ypanel_sz_pt)
  refvar[*] = !VALUES.D_NAN
  
  ;if very small scaling factors are used,
  ;refvar will sometimes lose a dimension
  if size(refvar,/n_dim) eq 1 then begin
    refvar = reform(refvar,1,1)
  endif
  
  ;self->makeSpec,x,y,z,xdata_sz_pt,ydata_sz_pt,paletteNum,zrange,image=image,refz=refz
  self->makeSpec,x,y,z,xdata_sz_pt,ydata_sz_pt,zAlpha=zAlpha,refz=refz
  
  plotData = { $
    data:refz,$
    alpha:zAlpha,$
    xstart:xstart,$
    ystart:ystart,$
    xsize:xsize,$
    ysize:ysize,$
    zstack:zstack, $
    zscale:zscale,$
    pixx:xdata_sz_pt,$
    pixy:ydata_sz_pt,$
    clip:cp $
    }
  
  ;determine indices of data in reference variable that
  ;represents the entire panel
  xStartIdx = floor(xstart*(size(refvar,/dimen))[0])
  xStopIdx = xStartIdx+xdata_sz_pt-1
  
  if xStopIdx lt 0 || xStartIdx ge xpanel_sz_pt then return
  
  if xStartIdx lt 0 then begin
    refz = refz[abs(xStartIdx):*,*]
    xStartIdx = 0
  endif
  
  if xStopIdx ge xpanel_sz_pt then begin
    refz = refz[0:xpanel_sz_pt-xStartIdx-1,*]
    xStopIdx = xpanel_sz_pt-1
  endif
  
  yStartIdx = floor(ystart*(size(refvar,/dimen))[1])
  yStopIdx = yStartIdx+ydata_sz_pt-1
  
  if yStopIdx lt 0 || yStartIdx ge ypanel_sz_pt then return
  
  if yStartIdx lt 0 then begin
    refz = refz[*,abs(yStartIdx):*]
    yStartIdx = 0
  endif
  
  if yStopIdx ge ypanel_sz_pt then begin
    refz = refz[*,0:ypanel_sz_pt-yStartIdx-1]
    yStopIdx = ypanel_sz_pt-1
  endif
  
  refvar[xStartIdx:xStopIdx,yStartIdx:yStopIdx] = refz
  
;  model = obj_new('IDLgrModel')
  
  ; if ~keyword_set(self.postscript) then begin
  ;if 1 then begin
  
  ;using a plain image doesn't layer correctly in postscript, consider generating layer code so that this option can replace polygon method
  ; imageObj = obj_new('IDLgrImage',image,location=[xstart,ystart,zstack],dimensions=[xsize,ysize],depth_test_disable=2,blend_function=[3,4])
 ; imageObj = obj_new('IDLgrImage',image)
  
;  polygon = obj_new('IDLgrPolygon', $
;    [[xstart,ystart,zstack],[xstart+xsize,ystart,zstack],[xstart+xsize,ystart+ysize,zstack],[xstart,ystart+ysize,zstack]], $
;    texture_map=imageObj,$
;    texture_coord=[[0,0],[1,0],[1,1],[0,1]], $
;    ;  texture_coord=[[0,1],[1,1],[1,0],[0,0]], $
;    color=self->convertColor([255,255,255]),$
;    shading=1,$
;    clip_planes=cp ) ;,$
;linestyle=6)
    
;endif else begin
    
;NOTE this code below has been worked around, by kludging layering issues during postscript output
;generate polygon vertices for every pixel.
;This is needed for postscript output
;
;
    
;The system I'm using is a polygon mesh, a regular arrangement of 4-gons
;The IDL documentation indicates that this is an optimized polygon
;arrangement and thus should be quicker.
;That said, we should consider generating complex n-gons for each
;contiguous colo region in future versions
    
;1d
;    x_verts = xstart + xsize* (dindgen(xdata_sz_pt+1)/xdata_sz_pt)
;
;    ;X 2d
;    x_verts = reform(rebin(x_verts,xdata_sz_pt+1,ydata_sz_pt+1),(xdata_sz_pt+1)*(ydata_sz_pt+1))
;
;    ;1d
;    y_verts = ystart + ysize*(dindgen(ydata_sz_pt+1)/ydata_sz_pt)
;
;    ;Y 2d
;    y_verts = reform(transpose(rebin(y_verts,ydata_sz_pt+1,xdata_sz_pt+1)),(xdata_sz_pt+1)*(ydata_sz_pt+1))
;
;    ;Z 2d
;    z_verts = replicate(zstack,(xdata_sz_pt+1)*(ydata_sz_pt+1))
;
;    ;generate vertex permutations to define the polygons
;
;    ;vertex number per polygon
;    v_num = lonarr(xdata_sz_pt*ydata_sz_pt)+4 ;all polygons use 4 vertices
;
;    offset = lindgen(xdata_sz_pt*ydata_sz_pt)/xdata_sz_pt
;
;    ;the permutation for each vertex of each polygon
;    ;first vertex of each polygon
;    v1 = lindgen(xdata_sz_pt*ydata_sz_pt) + offset
;    ;second vertex of each polygon
;    v2 = lindgen(xdata_sz_pt*ydata_sz_pt) + 1 + offset
;    ;third vertex of each polygon
;    v3 = lindgen(xdata_sz_pt*ydata_sz_pt) + xdata_sz_pt + 1 + offset
;    ;fourth vertex of each polygon
;    v4 = lindgen(xdata_sz_pt*ydata_sz_pt) + xdata_sz_pt + 2 + offset
;
;    v_perm = transpose([[v_num],[v1],[v2],[v4],[v3]])
;
;    ;generate vertex colors of polygons
;
;    v_r_colors = bytarr((xdata_sz_pt+1),(ydata_sz_pt+1))
;    v_r_colors[0:xdata_sz_pt-1,0:ydata_sz_pt-1] = reform(image[0,*,*])
;    v_r_colors = reform(v_r_colors,(xdata_sz_pt+1)*(ydata_sz_pt+1))
;
;    v_g_colors = bytarr((xdata_sz_pt+1),(ydata_sz_pt+1))
;    v_g_colors[0:xdata_sz_pt-1,0:ydata_sz_pt-1] = reform(image[1,*,*])
;    v_g_colors = reform(v_g_colors,(xdata_sz_pt+1)*(ydata_sz_pt+1))
;
;    v_b_colors = bytarr((xdata_sz_pt+1),(ydata_sz_pt+1))
;    v_b_colors[0:xdata_sz_pt-1,0:ydata_sz_pt-1] = reform(image[2,*,*])
;    v_b_colors = reform(v_b_colors,(xdata_sz_pt+1)*(ydata_sz_pt+1))
;
;    v_a_colors = bytarr((xdata_sz_pt+1),(ydata_sz_pt+1))
;    v_a_colors[0:xdata_sz_pt-1,0:ydata_sz_pt-1] = reform(image[3,*,*])
;    v_a_colors = reform(v_a_colors,(xdata_sz_pt+1)*(ydata_sz_pt+1))
;
;
;    v_colors = transpose([[v_r_colors],[v_g_colors],[v_b_colors],[v_a_colors]])
;
;    polygon = obj_new('IDLgrPolygon',$
;                       x_verts,$
;                       y_verts,$
;                       z_verts,$
;                       vert_colors=v_colors,$
;                       shading=0,$
;                       style=2,$
;                       polygon=v_perm)
;
;  endelse
    
;print,systime(/seconds)-a
    
; return,polygon
    
    
end

;+
;Procedure: thm_ui_draw::getSpecModel
;
;Purpose:  This procedure finishes the creation of a spectral image.
;          It uses the newly calculated range and the information in plotdata
;
;-
pro thm_ui_draw_object::getSpecModel,plotData,zrange,palette,model=model

  compile_opt idl2
  
  model = obj_new('IDLgrModel')
  
  pal = obj_new('IDLgrPalette')
  
  getctpath,ctpath
  
  pal->loadct,palette,file=ctpath
 
  pal->getProperty,red_values=rv,green_values=gv,blue_values=bv
  
  out = reform(intarr(4,(*plotData).pixx,(*plotdata).pixy),4,(*plotData).pixx,(*plotdata).pixy)
  
  if (*plotData).zscale eq 1 ||  (*plotData).zscale eq 2 then begin
  
    idx = where(finite((*plotData).data,/infinity,sign=-1),c)
    
    if c gt 0 then begin
      (*plotData).alpha[idx] = 255
      (*plotData).data[idx] = zrange[0]
    endif
  
  endif
  
  
  zval = bytscl((*plotData).data,/nan,min=zrange[0],max=zrange[1])
  
  out[0,*,*] = rv[zval]
  out[1,*,*] = gv[zval]
  out[2,*,*] = bv[zval]
  out[3,*,*] = (*plotdata).alpha 

  out = reform(out,4,(*plotData).pixx,(*plotData).pixy)
  imageObj = obj_new('IDLgrImage',out,$
    location=[(*plotData).xstart,(*plotData).ystart,(*plotData).zstack],$
    dimensions=[(*plotData).xsize,(*plotData).ysize],$
    depth_test_disable=2,$
    blend_function=[3,4])
    
  polygon = obj_new('IDLgrPolygon', $
    [[(*plotData).xstart,(*plotData).ystart,(*plotData).zstack],$
    [(*plotData).xstart+(*plotData).xsize,(*plotData).ystart,(*plotData).zstack],$
    [(*plotData).xstart+(*plotData).xsize,(*plotData).ystart+(*plotData).ysize,(*plotData).zstack],$
    [(*plotData).xstart,(*plotData).ystart+(*plotData).ysize,(*plotData).zstack]], $
    texture_map=imageObj,$
    texture_coord=[[0,0],[1,0],[1,1],[0,1]], $
    ;  texture_coord=[[0,1],[1,1],[1,0],[0,0]], $
    color=self->convertColor([255,255,255]),$
    shading=1,$
    clip_planes=(*plotData).clip ) 
    
  model->add,polygon

end

;+
;
; Function: thm_ui_draw::makeSpec
;
;Purpose:
;  helper routine for draw object.  It helps construct the image for spectrograms
;  very quickly.
;
;Inputs:
;  x: the 1-d x scaling values for the z components(x-axis)
;  y: the 1-d y scaling values for the z components
;  z: the 2-d z array of values for the image
;  pixx: the desired resolution of the output on the x-axis
;  pixy: the desired resolution of the output on the y-axis
;  palette: the palette number to be used when creating the image
;  zrange: the zrange for the whole panel(may be larger than the range of the z argument)
;
;Output:
;   An rgba image with dimensions: 4 x PixX x PixY
;   
;   Notes:
;   1. This uses an alpha channel to make all NaNs transparent.
;   2. PixX and PixY are not totally necessary because object graphics can
;      stretch an image quite well, but selecting the resolution of the screen
;      with them prevents any inadvertent errors from showing up during interpolation.
;      It might actually be better to render the image at twice the screen resolution
;      to prevent aliasing errors.
;   
;$LastChangedBy: pcruce $
;$LastChangedDate: 2009-04-22 18:11:55 -0700 (Wed, 22 Apr 2009) $
;$LastChangedRevision: 5722 $
;$URL: svn+ssh://thmsvn@ambrosia.ssl.berkeley.edu/repos/thmsoc/tags/tdas_5_00/idl/themis/thm_ui_new/display/thm_ui_draw_object__define.pro $
;-

pro thm_ui_draw_object::makeSpec,x,y,z,pixx,pixy,zAlpha=zAlpha,refz=refz,refx=refx,refy=refy

  compile_opt idl2
  
;  pal = obj_new('IDLgrPalette')
;  
;  getctpath,ctpath
;  
;  pal->loadct,palette,file=ctpath

  ;double check that data is valid
  idx = where(finite(y),c)
  
  if c eq 0 then begin
    self.statusBar->update,'Error: Unexpected invalid quantity'
    self.historyWin->update,'Error: Unexpected invalid quantity'
    return
  endif
  
  ;special case 1d Z
  if ndimen(z) eq 1 then begin
  
    if n_elements(y) eq 1 && n_elements(x) eq 1 then begin
      ;nothin can be done with one-and-ones
      return
    endif else if n_elements(y) eq 1 then begin
    
      ;refy = [0]
      dim = n_elements(z)
      xn = dim
      
      xi = (x - x[0])/(x[xn-1]-x[0]) 
      refx = dindgen(pixx)/pixx
      
      refz = interpol(z,xi,refx) # replicate(1.,pixy)
      
      refy = replicate(y,pixy)
      
    endif else if n_elements(x) eq 1 then begin
    
      ;refy = [0]
      dim = n_elements(z)
      yn = dim
      
      yi = (y - y[0])/(y[yn-1]-y[0]) 
      refy = dindgen(pixy)/pixy
      
      refz = replicate(1.,pixx) # interpol(z,yi,refy)
      
      refx = replicate(x,pixx)
    
    endif else begin
      return
    endelse
    
  ;  zout = bytscl(refz,/nan,min=zrange[0],max=zrange[1])
    
    zalpha = bytarr(pixx,pixy)
    
    idx = where(finite(refz),c)
    
    if c ne 0 then begin
      zalpha[idx] = 255
    endif
      
  ;1d y draw code
  endif else if ndimen(y) eq 1 then begin
    y = y[idx]
    z = z[*,idx]
  
    dim = dimen(z)
    
    xn = dim[0]
    yn = dim[1]
    
    alpha = dblarr(dim) + 255
    
    idx = where(~finite(z),c)
    
    if c ne 0 then alpha[idx] = 0
    
    ;sorting should maybe be dumped.
    ;it is really a preprocessing task
    idx = bsort(y)
    
    y = y[idx]
    alpha = alpha[*,idx]
    z = z[*,idx]
    
    idx = bsort(x)
    
    x = x[idx]
    alpha = alpha[idx,*]
    z = z[idx,*]
    
    xi = (x - x[0])/(x[xn-1]-x[0]) 
    xo = dindgen(pixx)/(pixx-1)
     
    refx = interpol(dindgen(xn),xi,xo)
    xpos = round(refx)
    
    ;because nans may be present, cannot use index to find min and max,
    ;despite sort
    miny = min(y,/nan)
    maxy = max(y,/nan)
    
    yi = (y - miny)/(maxy-miny)
    yo = dindgen(pixy)/(pixy-1)
  
    refy = interpol(dindgen(yn),yi,yo)
    ypos = round(refy)
    
    refz = interpolate(z,xpos,ypos,/grid,missing=!VALUES.D_NAN)
  ;  zout = bytscl(refz,/nan,min=zrange[0],max=zrange[1])
    
    zalpha = interpolate(alpha,xpos,ypos,/grid,missing=0)
    
  endif else begin
  ;2d y code

  
    dim = dimen(z)
    
    minx = min(x,/nan)
    maxx = max(x,/nan)
    
    miny = min(y,/nan)
    maxy = max(y,/nan)
    
    ;normalized inputs
    xi = (temporary(x) - minx)/(maxx - minx)
    yi = (temporary(y) - miny)/(maxy - miny)
    
    ;normalized x output
    xo = dindgen(pixx)/pixx
     
    ;index-wise scaling value
    refx = interpol(dindgen(dim[0]),temporary(xi),temporary(xo))
    
    ;rounded to prevent smoothing of the data
    xpos = round(refx)
    
    ;now make it 2d
    xpos = xpos # replicate(1.,pixy)
  
    ;normalized y output
    yo = dindgen(pixy)/pixy
  
    ;allocate storage for index-wise scaling value
    refy = dblarr(pixx,pixy)
  
    ;now sample y values along x-axis
    ;note that this technique is subject to potential aliasing errors
    ;unless pixx, is twice the width of the plotting error in pixels
    ;this can be controlled by changing the self.specres value
    for i = 0,pixx-1 do begin
      ;calculate the index of this sample
      j = (xpos[i,0] > 0) < (dim[0]-1)
      y_sample = reform(yi[j,*])
      refy[i,*] = interp(dindgen(dim[1]),y_sample,yo) 
    endfor
  
    ;use an out of range value for non-in-range values
    ;these should have already been cleared in the case of the x-axis
    idx = where(~finite(refy),c)
    
    if c ne 0 then begin
      refy[idx] = -1
    endif
    
    ;now make sure we have only exact indexes
    ypos = round(refy)
    
    refz = interpolate(z,xpos,ypos,missing=!VALUES.D_NAN)
   ; zout = bytscl(refz,/nan,min=zrange[0],max=zrange[1])
    zalpha = bytarr(pixx,pixy)
    
    idx = where(finite(refz),c)
    
    if c ne 0 then begin
      zalpha[idx] = 255
    endif
  
  endelse
  
;  pal->getProperty,red_values=rv,green_values=gv,blue_values=bv
;  
; ; catch,err
;  
;  ;here to catch potential memory overallocation
; ; if err then begin
; ;   stop
; ; endif else begin
;  out = intarr(4,pixx,pixy)
; ; endelse
;  ;catch,/cancel
;  
;  out = reform(out,4,pixx,pixy)
;  
;  out[0,*,*] = rv[zout]
;  out[1,*,*] = gv[zout]
;  out[2,*,*] = bv[zout]
;  out[3,*,*] = zalpha
;   
;  ;grid the points and scale them into the range of color indices
;  ;return, floor(interpolate(z,xpos,ypos,/grid))
;  image=out
;  
;  ;IDL will remove any 1 element dimensions on this assignment
;  ;this fixes the problem
;  image = reform(image,4,pixx,pixy)

end


;A special kluge function to get around an IDL bug, that causes improper layer in eps
;generates a composite image from a series of spectral plots to preserve layering when exporting to eps
function thm_ui_draw_object::aggregateSpecplots,spec_list,panel_sz_pt,bg_color

  compile_opt idl2
  
  zstack = .05
  
  ;clip planes will cut off the image at borders
  cp = double([[-1,0,0,0],[1,0,0,-1],[0,-1,0,0],[0,1,0,-1]])
  
  view = obj_new('IDLgrView',units=3, $
    VIEWPLANE_RECT=[0.,0.,1.,1.], $
    zclip=[1,-1], eye=5.,color=bg_color)
    
  model_list = spec_list->get(/all)
  
  for i = 0,n_elements(model_list)-1 do begin
  
    view->add,model_list[i]
    
  endfor
  
  buffer = obj_new('IDLgrBuffer',dimensions=panel_sz_pt)
  
  buffer->draw,view
  
  buffer->getProperty,image_data=image_data
  
  image = obj_new('IDLgrImage',image_data,depth_test_disable=2,clip_planes=cp,location=[0.,0.,zstack],dimensions=[1.,1.])
  
  model = obj_new('IDLgrModel')
  
  model->add,image
  
  return,model
end

;This function generates the line plot for an update
;There is actually a lot of room to increase draw speed
;by optimizing this function.  In specific we need to find
;a way to downsample a line plot, but the technique must work
;on series that are not functional(ex: circle), must not sort
;the inputs, must be imperceptible, must be fast, and must not
;expect the inputs to be spaced uniformly.  Some possibilities:
;1: uniform decimation
;2: pseudo-random decimation
;3: using pythagorean distance to rewrite non-functional series as functional series
;   then interpolate
;4: DFT with frequency cutoff, iDFT, uniform sample?
;
;Another problem involves determining the correct number of points to
;which the target should be decimated. There is a tradeoff where at one
;end you start to introduce aliasing errors and at the other end you get
;a speed slowdown.
;
;Currently the system uses the some constant factor times the number of pixels
;across the plot as the target decimation and it only decimates inputs that have
;time as the x-axis(which can be assumed to be functional).  These can be reliably
;decimated using normal interpolation.
function thm_ui_draw_object::getLinePlot,trace,xrange,yrange,plotdim1,plotdim2,xscaling,yscaling,xAxisMajors,dx,dy,xistime,linecolor=linecolor,mirrorptr=mirrorptr,refvar=refvar,abcissa=abcissa_out

  compile_opt idl2
  
  zstack = .2
  
  model = obj_new('IDLgrModel')
  
  trace->getProperty,$
    dataX=dataX,$
    dataY=dataY,$
    lineStyle=lineStyle,$
    drawBetweenPts=drawBetweenPts,$
    mirrorLine=mirrorLine,$
    symbol=symbol,$
    plotpoints=plotpoints,$
    everyother=everyother,$
    positiveEndPt=positiveEndPt,$ ;error bars not yet implemented
    negativeEndPt=negativeEndPt,$
    positiveEndRel=positiveEndRel,$
    negativeEndRel=negativeEndRel,$
    barLine=barLine,$
    markSymbol=markSymbol
    
  lineStyle->getProperty,$
    id=id,$
    show=show,$
    color=color,$
    thickness=thick
    
  linecolor=color
  
  if size(refVar,/type) then begin
    tmp = temporary(refVar)
  endif
  
  if size(abcissa_out,/type) then begin
    tmp = temporary(abcissa_out)
  endif
  
  ;number of pixels across the panel * res_factor = number of points in lookup
  xpx = self->getLineSize(plotdim1)
  
  if xrange[1] - xrange[0] eq 0 then return,model
  
  if yrange[1] - yrange[0] eq 0 then return,model
  
  x = temporary(*dx)
  y = temporary(*dy)
  
  if mirrorline then mirror = temporary(*mirrorptr)
  
  ;need at least two points to do separation check.
  if keyword_set(drawBetweenPts) && n_elements(x) gt 1 then begin
  
    separation = trace->getPtSpacing()
    
    idx = where((x[1:n_elements(x)-1]-x[0:n_elements(x)-2]) gt separation)
    
    if idx[0] ne -1 then begin
      y[idx] = !VALUES.D_NAN
    
      if keyword_set(mirrorline) then begin
        mirror[idx] = !VALUES.D_NAN
      endif
    endif 
    
  endif
  
  x = (x - xrange[0])/(xrange[1]-xrange[0])
  
  self->makeLineReference,x,y,xpx,ref=refvar
  
  y = (y - yrange[0])/(yrange[1]-yrange[0])
    
  if keyword_set(mirrorLine) then begin
  
   ; if n_elements(x) gt scale_factor*xpx then begin
    
      mirrorvar = (temporary(mirror) - yrange[0])/(yrange[1]-yrange[0])
      ;mirrorvar = interpol(mirrorvar,scale_in,scale_out)
      
   ; endif else begin
    
  ;    mirrorvar = temporary(mirror)
      
  ;  endelse
    
    plot = obj_new('IDLgrPlot',x,mirrorvar,color=self->convertColor(color),linestyle=id,hide=~show,thick=thick,xrange=[0D,1D],yrange=[0D,1D],zvalue=zstack,/use_zvalue)
    model->add,plot
    
  endif
  
  plot = obj_new('IDLgrPlot',x,y,color=self->convertColor(color),linestyle=id,hide=~show,thick=thick,xrange=[0D,1D],yrange=[0D,1D],zvalue=zstack,/use_zvalue)
  
  model->add,plot
  
  ;locations for plot symbols/error bars
  n = n_elements(x)
  
  if plotpoints eq 1 then begin
    x_sym = [x[0],x[n-1]]
    y_sym = [y[0],y[n-1]]
  endif else if plotpoints eq 2 then begin
    x_sym = [x[0]]
    y_sym = [y[0]]
  endif else if plotpoints eq 3 then begin
    x_sym = [x[n-1]]
    y_sym = [y[n-1]]
  endif else if plotpoints eq 4 then begin
  
    if n_elements(xAxisMajors) gt 1 then begin
      for i = 1,n_elements(xAxisMajors)-1 do begin
      
        t = min(abs(x-xAxisMajors[i]),idx)
        if t lt .01D then begin ;only add symbol if it is close to an actual value
          if n_elements(idx_l) eq 0 then begin
            idx_l = [idx]
          endif else begin
            idx_l = [idx_l,idx]
          endelse
        endif
        
      endfor
    endif
    
    if n_elements(idx_l) gt 0 then begin
      x_sym = x[idx_l]
      y_sym = y[idx_l]
    endif
    
  endif else if plotpoints eq 5 then begin
  
    if everyother gt 0 && everyother lt n then begin
    
      ;note that the (n mod everyother) statement is in place to prevent a 0 length lindgen
      ;It may no longer be necessary now that the draw object inputs are better error checked
      x_sym = x[lindgen(((n mod everyOther)ne 0)+n/everyOther)*everyOther]
      y_sym = y[lindgen(((n mod everyOther)ne 0)+n/everyOther)*everyOther]
    ;  x_sym = x[lindgen(n/everyOther)*everyOther]
    ;  y_sym = y[lindgen(n/everyOther)*everyOther]
      
    endif
  endif else begin
    x_sym = temporary(x)
    y_sym = temporary(y)
  endelse
  
  ;symbol plots
  
  ;note that fill is not yet implmented
  
  symbol->getProperty,id=id,show=show,color=color,fill=fill,size=size
  
  if show && size(x_sym,/type) && size(y_sym,/type) then begin
  
    ;the size appears to be double the size in points that it should be.
    xsize = .5*self->pt2norm(size,0)/(plotdim1[1]-plotdim1[0])
    ysize = .5*self->pt2norm(size,1)/(plotdim2[1]-plotdim2[0])
    
    grSymbol = obj_new('IDLgrSymbol',id,color=self->convertColor(color),size=[xsize,ysize])
    
    plot = obj_new('IDLgrPlot',x_sym,y_sym,color=self->convertColor(color),linestyle=6, $
      xrange=[0D,1D],yrange=[0D,1D], zvalue=zstack+.01,symbol=grsymbol,/use_zvalue)
      
    model->add,plot
    
  endif
  
  return,model
  
end

;model: the model
;labels: IDL_Container storing labels
;margin: margin between axis & labels
;placeAnnotation: whether to place annotation on top/bottom left/right
;dir: x/y axis flag
;orientation: horizontal/vertical text flag
;stackLabels: stacklabels or rowlabels flag
;pt1: multiplicand to change pt into view normalized coords(perp axis)
;pt2: multiplicand to change pt into view normalized coords(par axis)
;labelpos: returns the position of the most distant label from the axis
;blacklabels: indicates that default settings should be over-ridden to make all labels black
pro thm_ui_draw_object::addAxisLabels,model,labels,margin,placeAnnotation,dir,orientation,stacklabels,showlabels,pt1,pt2,blacklabels,labelPos=labelPos

  compile_opt idl2
  
  ;not completely sure about what z-values
  ;each component will need, so I set it at
  ;the top
  zstack = .1
  
  labelpos = 0
  
  ;validate input
  if ~showlabels || ~obj_valid(labels) || ~obj_isa(labels,'IDL_Container') then return
  
  objs = labels->get(/all)
  
  if ~obj_valid(objs[0]) then return
  
  ;This switch simplifies the code significantly by
  ;making the x/y axis code symmetrical.
  ;Think of orient as indicating whether the text is
  ;oriented parallel or perpedicular to the axis
  if dir eq 1 then begin
    orient = ~orientation
  endif else begin
    orient = orientation
  endelse
  
  if orientation eq 0 then begin
    baseline = [1,0,0]
    updir = [0,1,0]
  endif else begin
    baseline = [0,1,0]
    updir = [-1,0,0]
  endelse
  
  if orient eq 0 then begin
    parAlign = .5
    perAlign = .5
  ;    if placeAnnotation eq 0 then begin
  ;      if dir eq 1 then begin  ;alignments are not completely directionally symmetical
  ;        perAlign = 0.0
  ;      endif else begin
  ;        perAlign = 1.0
  ;      endelse
  ;    endif else begin
  ;      perAlign = 1.0
  ;    endelse
  endif else begin
    perAlign = .5
    if placeAnnotation eq 0 then begin
      parAlign = 1.0
    endif else begin
      parAlign = 0.0
    endelse
  endelse
  
  halign = parAlign
  valign = perAlign
  
  len = 0D
  
  n = n_elements(objs)
  
  ;get the length of the labels as a block so that the
  ;labels can be positioned properly
  for i = 0,n-1 do begin
  
    objs[i]->getProperty,value=value,size=size,show=show
    
    if ~show then continue
    
    size += 1
    
    if orient eq 0 then begin
      if stacklabels eq 0 then begin
        len += strlen(value)*pt2*size
      endif else begin
        len += pt1*size
      endelse
    endif else begin
      if stacklabels eq 0 then begin
        len += strlen(value)*pt1*size
      endif else begin
        len += pt2*size
      endelse
    endelse
  ;need to correct to account for varying orientations/pt1/pt2
    
  endfor
  
  pos = 0D
  
  for i = 0, n-1 do begin
  
    tFont = objs[i]->getGrFont()
    
    objs[i]->getProperty,color=color,value=value,show=show,size=size
    
    if size le 0 then begin
      self.statusBar->update,'Label "' + value + '" had illegal size. Skipping...'
      self.historyWin->update,'Label "' + value + '" had illegal size. Skipping...'
      continue
    endif
    
    if ~show then continue
    
    tFont->setProperty,size=size*self->getZoom()
    
    size +=1
    
    ;this code calculates position for labels of different locations/orientations
    ;It allows for the fonts of labels to be different
    if stackLabels eq 0 then begin
      if orient eq 0 then begin
        loc1 = (size*strlen(value))/(len/pt2*2D) + pos
        loc2 = -(margin*pt1+size*pt1/2D)
        pos += double(size*strlen(value))/double(len/pt2)
      endif else begin
        loc1 = .5D
        loc2 = -(margin*pt1+pos)
        pos += size*strlen(value)*pt1
      endelse
    endif else begin
    
      if orient eq 0 then begin
        loc1 = .5D
        loc2 = -(margin*pt1+pos+size*pt1/2D)
        pos += pt1*(size)
      ; pos += pt1*size
      endif else begin
        loc1 = (.5D)-(len/2D)+(size*pt2)/2D + pos
        loc2 = -(margin*pt1)
        pos += pt2*(size)
      endelse
      
      if dir eq 1 then begin
        if orient eq 1 then begin
          loc1 = 1. - loc1
        endif else begin
          loc2 = -loc2 - len - 2*(pt1*margin)
        endelse
      endif
    endelse
    
    if placeAnnotation eq 1 then begin
      loc2 = 1 - loc2
    endif
    
    if dir eq 1 then begin
      loc = [loc2,loc1,zstack]
      labelPos = loc1
    endif else begin
      loc = [loc1,loc2,zstack]
      labelPos = loc2
    endelse
   
    if keyword_set(blacklabels) then begin
      color = [0,0,0]
    endif 
   
    grText = obj_new('IDLgrText',value,$
      font=tFont,$
      color=self->convertColor(color),$
      hide=~show,$
      location=loc,$
      alignment=hAlign,$
      vertical_alignment=valign,$
      baseline=baseline,$
      recompute_dimensions=0,$
      updir=updir)
      
    model->add,grText
    
  end
  
end

;adds the dateString annotation to the axis
pro thm_ui_draw_object::addDateString,view,axisSettings,range,scaling,xplotsize,yplotsize,labelmargin

  compile_opt idl2
  
  axisSettings->getProperty,$
    showDate=showDate,$
    datestring1=datestring1,$
    datestring2=datestring2,$
    placeannotation=placeannotation,$
  ;  scaling=scaling,$
    annotateTextObject=annotateTextObject,$
    tickStyle=tickStyle,$
    bottomPlacement=bottomPlacement,$
    topPlacement=topPlacement,$
    majorLength=majorLength,$
    minorLength=minorLength,$
    isTimeAxis=isTimeAxis
    
  ; view->getProperty,viewPlane_rect=viewPlane_rect
    
  if ~showDate then begin
    return
  endif
  
  if ~isTimeAxis then begin
    return
  endif
  
  string1 = formatdate(range[0],datestring1,scaling)
  string2 = formatdate(range[0],datestring2,scaling)
  
  string1Obj = annotateTextObject->copy()
  string2Obj = annotateTextObject->copy()
  
  string1Obj->getProperty,size=size
  
  textSizeNorm = self->pt2norm(size,1)/yplotsize
  
  posoffset = textSizeNorm
  
  ;account for tick position when calculating offset for date string
  if tickStyle eq 1 || tickStyle eq 2 then begin
    if (~placeAnnotation && bottomPlacement) || $
      (placeAnnotation && topPlacement) then begin
      posoffset += (self->pt2norm(majorLength > minorLength,1)/yplotsize)
    endif
  endif
  
  xposoffset = -self->pt2norm(labelmargin,0)/xplotsize
  
  string1Obj->setProperty,value=string1
  string2Obj->setProperty,value=string2
  
  if placeAnnotation then begin
    ; location = [viewPlane_rect[0]/2.,1. + (viewPlane_rect[3]-1+viewPlane_rect[1])/4.,.1]
    location = [xposoffset,1.+posoffset,.1]
  endif else begin
    ;location = [viewPlane_rect[0]/2.,viewPlane_rect[1]/4.,.1]
    location = [xposoffset,0.-posoffset,.1]
  endelse
  
  model1 = self->getTextModel(string1Obj,location,1.,0.,justify=1)
  model2 = self->getTextModel(string2Obj,location,-1.,0.,justify=1)
  
  view->add,model1
  view->add,model2
  
end

;Adds the background to the target view
pro thm_ui_draw_object::addBackground,view,color

  compile_opt idl2
  
  zstack=0.0
  
  grBackground = obj_new('IDLgrPolygon',[0,1,1,0], $
    [0,0,1,1], $
    [zstack,zstack,zstack,zstack],$
    color=self->convertColor(color))
    
  grBModel = obj_new('IDLgrModel')
  
  grBModel->add,grBackground
  
  view->add,grBModel
  
end

;creates the views for a panel
;dims = [rownum,colnum]
;margins = [left,right,top,bottom,xSpacing,ySpacing] in normalzied
;coords
;pos = [row,col,rowSpan,colSpan]
;pcoord = [xpos,ypos,xsize,ysize]
;[xy]plotpos, 2-d array with the start and stop values of the plot in
;normalized device coordinates
pro thm_ui_draw_object::makeView,dims,margins,pos,pcoord,markernum,totalvarsize,currentvarsize,cumulvarsize,view=view,annotation=annotation,markers=markers,xplotpos=xplotpos,yplotpos=yplotpos,fail=fail,outmargins=outmargins

  compile_opt idl2
  
  ;  margins = margins_in
  ;  margins[2] = margins_in[3]
  ;  margins[3] = margins_in[2]
  
  dims = double(dims)
  margins = double(margins)
  pos = double(pos)
  pcoord = double(pcoord)
  
  ;outmargins indicate the range in normalized device coordinates to
  ;be considered part of the region(this is used for identifying context sensitive clicks)
  outmargins = dblarr(5) ; [left,right,top,bottom,var]
  
  fail = 1
  
  xused = margins[0] + margins[1] + (dims[1]-1) * margins[4]
  xsize = (1.-xused)/dims[1]
  
  yused = margins[3] + margins[2] + (dims[0]-1) * margins[5] + totalvarsize
  ysize = (1. - yused)/dims[0]
  
  if xsize lt 0 then begin
    self.statusBar->update,'Error: XMargins/Horizontal PanelSpacing too large. No room available for panels'
    self.historyWin->update,'Error: XMargins/Horinzontal PanelSpacing too large.  No room for panels'
    ; ok = error_message('XMargins/PanelSpacing too large.  No room for panels')
    return
    
  endif
  
  if ysize lt 0 then begin
    self.statusBar->update,'Error: YMargins/Vertical PanelSpacing too large.  No room for panels'
    self.historyWin->update,'Error: YMargins/Vertical PanelSpacing too large.  No room for panels'
    ;ok = error_message('YMargins/PanelSpacing too large.  No room for panels')
    return
    
  endif
  
  ;This calculates the position and length of the view within the window
  ;It also calculates the viewplane rectangle.  This can be thought of as
  ;the coordinate system local to objects placed inside the view.
  ;It is set up so that the plot itself ranges from 0 to 1,  any
  ;margins are outside of that range
  
  ;x size of the plot in normalized coordinates
  ;this accounts for colspan & any margins
  xplot_size = xsize*pos[3] + margins[4]*(pos[3]-1)
  
  ;calculate
  ;1. X size/location of view
  ;2. X size of viewplane_rect
  ;3. X Start & Stop of Plot in normalized window coordinates
  
  xpos = 0.
  xlen = 1.
  
  if pos[1] eq 1 && pos[1] + pos[3] - 1 eq dims[1] then begin ;leftmost & rightmost panel
  
    xstart = margins[0]
    outmargins[0] = margins[0]/2D
    outmargins[1] = margins[1]/2D
    
  endif else if pos[1] eq 1 then begin  ;leftmost panel
  
    xstart = margins[0]
    
    outmargins[0] = margins[0]/2D
    outmargins[1] = margins[4]/2D
    
  endif else if pos[1] + pos[3] - 1 eq dims[1] then begin ;rightmost panel
  
    xstart = 1. - margins[1] - xplot_size
    
    outmargins[0] = margins[4]/2D
    outmargins[1] = margins[1]/2D
    
  endif else begin  ;neither left nor right
  
    xstart = margins[0] + (xsize+margins[4])*(pos[1]-1.)
    
    outmargins[0] = margins[4]/2D
    outmargins[1] = margins[4]/2D
    
  endelse
  
  ;handle explicit positioning by user
  if pcoord[0] ne -1 then begin
    xstart = pcoord[0]
  endif
  
  if pcoord[2] ne -1 then begin
    xplot_size = pcoord[2]
    if xplot_size eq 0 then begin
    
      self.statusBar->update,'0 width panel cannot be created'
      self.historyWin->update,'0 width panel cannot be created'
      return
    endif
  endif
  
  xstop = xstart + xplot_size
  viewXpos = -(xstart / xplot_size)
  viewXlen = 1. / xplot_size
  
  ;dims = [rownum,colnum]
  ;margins = [left,right,top,bottom,xSpacing,ySpacing] in normalzied
  ;coords
  ;pos = [row,col,rowSpan,colSpan]
  
  ;this layout naturally orders from bottom to top
  ;the line below reverses the ordering of the rows.
  ;so that it orders from top to bottom
  pos[0] = dims[0] - pos[0] + 1
  
  ;Y size of the plot in normalized coordinates
  ;this accounts for rowspan & any interior margins
  ;(interior margins from multiple row span)
  yplot_size = ysize*pos[2] + margins[5]*(pos[2]-1)
  
  ;because rows count in the opposite direction of coordinate system,
  ;the position needs to be shifted down when a panel spans multiple
  ;rows.  This way adding span appears to increase panel size in the
  ;direction of increasing rows.
  pos[0] = pos[0] - (pos[2] - 1)
  
  ;calculate:
  ;1.  Y size/location of view
  ;2. Y dimensions of viewplane_rect
  ;3. Y Start & Stop of the plot in normalized coordinates
  
  ypos = 0.
  ylen = 1.
  
  if pos[0] eq 1 && pos[0] + pos[2] - 1 eq dims[0] then begin ;case 1: 1 row layout
  
    ystart = margins[2] + currentvarsize
    
    outmargins[2] = margins[2]/2D
    outmargins[3] = margins[3]/2D
    
  endif else if pos[0] eq 1 then begin ;case 2: bottom row in layout
  
    ystart = margins[3] + currentvarsize
    
    outmargins[2] = margins[5]/2D
    outmargins[3] = margins[3]/2D
    
  endif else if pos[0] + pos[2] - 1 eq dims[0] then begin ;case 3: top row in layout
  
    ystart = 1. - (margins[2] + yplot_size)
    
    outmargins[2] = margins[2]/2D
    outmargins[3] = margins[5]/2D
    
  endif else begin  ;case 4, somewhere in the middle of layout
  
    ystart = margins[3] + (ysize+margins[5])*(pos[0]-1) + cumulvarsize
    
    outmargins[2] = margins[5]/2D
    outmargins[3] = margins[5]/2D
    
    
  endelse
  
  ;handle explicit positioning by user
  if pcoord[1] ne -1 then begin
    ystart = pcoord[1]
  endif
  
  if pcoord[3] ne -1 then begin
    yplot_size = pcoord[3]
    if yplot_size eq 0 then begin
    
      self.statusBar->update,'0 height panel cannot be created'
      self.historyWin->update,'0 height panel cannot be created'
      return
    endif
  endif
  
  ystop = ystart + yplot_size
  viewYpos = -(ystart / yplot_size)
  viewYlen = 1./ yplot_size
  
  outmargins[4] = currentvarsize
  
  view = OBJ_NEW('IDLgrView')
  
  view->setProperty,units=3,viewplane_rect=[viewXpos,viewYpos,viewXlen,viewYlen], $
    location=[xpos,ypos],dimensions=[xlen,ylen],zclip=[1.,-1.], $
    eye=5.,transparent=1
    
  annotation = OBJ_NEW('IDLgrView')
  
  annotation->setProperty,units=3,viewplane_rect=[viewXpos,viewYpos,viewXlen,viewYlen], $
    location=[xpos,ypos],dimensions=[xlen,ylen],zclip=[1.,-1.], $
    eye=5.,transparent=1,hide=0
    
  if markernum gt 0 then begin
    markers = objarr(markernum)
    
    for i = 0,markernum - 1 do begin
    
      marker = obj_new('IDLgrView')
      
      marker->setProperty,units=3,viewplane_rect=[viewXpos,viewYpos,viewXlen,viewYlen], $
        location=[xpos,ypos],dimensions=[xlen,ylen],zclip=[1.,-1.], $
        eye=5.,transparent=1,hide=0
        
      markers[i] = marker
    endfor
  endif
  
  xplotPos = [xstart,xstop]
  yplotPos = [ystart,ystop]
  
  fail = 0
  
end

;updates the draw object to reflect the page settings
function thm_ui_draw_object::updatePage,pageSettings

  compile_opt idl2
  
  vertical_offset_top = 20 ; pts
  vertical_offset_bottom = 5 ; pts
  
  pageSettings->getproperty,backgroundcolor=bgcol,canvasSize=canvasSize
  
  title = pageSettings->getTitleString()
  
  footer = pageSettings->getFooterString()
  
  self.currentPageSize = canvasSize
  
  ; dim = self->getDim()
  
  ; dim /= self->getZoom()
  
  if ~obj_valid(self.pageview) then begin
    pageview = obj_new('IDLgrView')
    self.pageview = pageview
    pageview->setProperty,units=3,viewplane_rect=[0.,0.,1.,1.],location=[0.,0.],dimensions=[1.,1.],zclip=[1.,-1.],eye=5,name="pageview",/transparent
    self.scene->add,pageview
  endif
  
  ;add pageview to the group of static views
  ;used for instancing
  self.staticViews->add,self.pageView
  
  ;set the background color
  self.scene->setProperty,color=self->convertColor(bgcol)
  
  ;remove old page info
  self.pageview->remove,/all
  
  testmodel = OBJ_NEW('IDLgrModel')
  
  self.pageview->add,testmodel
  
  if obj_valid(title) then begin
  
    grTitle = self->getTextModel(title,[.5,1.-self->pt2norm(vertical_offset_top,1),0.],-1,0)
    
    self.pageview->add,grTitle
    
    obj_destroy,title
    
  endif
  
  if obj_valid(footer) then begin
  
    grFooter = self->getTextModel(footer,[.5,self->pt2norm(vertical_offset_bottom,1),0.],1,0)
    
    self.pageview->add,grFooter
    
    obj_destroy,footer
    
  end
  
  return,1
  
end

;Calculates the zrange of a spectral plot based upon the set of spectral traces in the panel
;and the z-axis settings
pro thm_ui_draw_object::getZRange,dataptrs,zaxisSettings,range=range,scaling=scaling,fail=fail,fixed=fixed

  compile_opt idl2
  
  fail = 1
  
  zAxissettings->getProperty,fixed=fixed,maxRange=maxRange,minRange=minRange,scaling=scaling
  
  if fixed then begin
  
    if maxRange le 0 && scaling ne 0 then begin
      self.statusbar->update,'Error: Negative range with logarithmic Z axis'
      ;ok = error_message('Negative range with logarithmic axis',/traceback)
      range= double([1e-10,1])
      return
    endif else if minRange le 0 && scaling ne 0 then begin
      self.statusbar->update,'Error: Negative range with logarithmic Z axis'
      ; ok = error_message('Negative range with logarithmic axis',/traceback)
      range= double([1e-10,1])
      return
    endif
    
    if scaling eq 1 then begin
      range = alog10([minRange,maxRange])
    endif else if scaling eq 2 then begin
      range = alog([minRange,maxRange])
    endif else begin
      range = [minRange,maxRange]
    endelse
    
  endif else begin
  
    minRange = !VALUES.D_NAN
    maxRange = !VALUES.D_NAN
    
    for i = 0, n_elements(dataptrs)-1 do begin
    
      if ptr_valid(dataptrs[i]) then begin
      
        if scaling eq 1 || scaling eq 2 then begin
         
          idx = where(*dataptrs[i] gt 0,c1)
          idx = where(*dataptrs[i] eq 0,c2)
          
          ;This is fix deals with out of range log values
          if c1 eq 0 && c2 eq 0 then begin
            ;do nothing
          endif else if c1 eq 0 then begin
            if ~finite(minRange) && ~finite(maxRange) then begin
              minRange = -!VALUES.D_INFINITY
              maxRange = -!VALUES.D_INFINITY
            endif
          endif else if scaling eq 1 then begin
            minRange = min([min(alog10(*dataptrs[i]),/nan),minRange],/nan)
            maxRange = max([max(alog10(*dataptrs[i]),/nan),maxRange],/nan)
          endif else begin
            minRange = min([min(alog(*dataptrs[i]),/nan),minRange],/nan)
            maxRange = max([max(alog(*dataptrs[i]),/nan),maxRange],/nan)
          endelse
        endif else begin
          minRange = min([min(*dataptrs[i],/nan),minRange],/nan)
          maxRange = max([max(*dataptrs[i],/nan),maxRange],/nan)
        endelse
      endif
      
    endfor
    
    range=[minRange,maxRange]
    
  endelse
  
  range = double(range)
  
  fail = 0
  
  return
  
  
end

;Calculates the range of a linear sequence. Based upon axis settings and the set of sequences(line traces) in the panel
pro thm_ui_draw_object::getRange,dataptrs,axisSettings,mirror=mirror,scaling=scaling,range=range,fail=fail

  compile_opt idl2
  
  fail = 1
  
  axisSettings->getProperty,rangeOption=rangeOption, $
    rangeMargin=rangeMargin,$
    boundScaling=boundScaling, $
    boundfloating=boundfloating,$
    minFloatRange=minFloatRange,$
    maxFloatRange=maxFloatRange,$
    minBoundRange=minBoundRange, $
    maxBoundRange=maxBoundRange, $
    minFixedRange=minFixedRange, $
    maxFixedRange=maxFixedRange, $
    floatingSpan=floatingSpan, $
    floatingCenter=floatingCenter,$
    scaling=scaling,$
    isTimeAxis=isTimeAxis,$
    annotateStyle=annotateStyle,$
    annotateExponent=annotateExponent
    
  if rangeOption eq 2 then begin
    range = [minFixedRange,maxFixedRange]
    
    if maxFixedRange le 0 && scaling ne 0 then begin
      self.statusbar->update,'Error: Negative range with logarithmic X/Y axis'
      ;  ok = error_message('Negative range with logarithmic axis',/traceback)
      range = double([1e-10,1])
      return
    endif else if minFixedRange le 0 && scaling ne 0 then begin
      self.statusbar->update,'Error: Negative range with logarithmic X/Y axis'
      ;ok = error_message('Negative range with logarithmic axis',/traceback)
      range = double([1e-10,1])
      return
    endif
    
    if scaling eq 1 then begin
      range = double(alog10(range))
    endif else if scaling eq 2 then begin
      range = double(alog(range))
    endif else begin
      range = double(range)
    endelse
    
  endif else if rangeOption eq 1 then begin
    ;The calculation for the floating range is the join of the floating range for each trace
    ;This is faster to calculate and prevents variations in sample rate between traces for creating an
    ;unreasonable result
  
    centers = dblarr(n_elements(dataptrs))
    
    for i = 0,n_elements(dataptrs)-1 do begin
    
      if ~ptr_valid(dataPtrs[i]) then continue
      
      if keyword_set(mirror) && ptr_valid(mirror[i]) then begin
        centers[i] = 0
      endif else if scaling eq 1 then begin
        if floatingCenter eq 0 || floatingCenter eq 2 then begin
          centers[i] = mean(alog10(*dataPTrs[i]),/nan)
        endif else if floatingCenter eq 1 || floatingCenter eq 3 then begin
          centers[i] = median(alog10(*dataPTrs[i]))
        endif else begin
          self.statusBar->update,'Error: Unrecognized floating center option'
          ;ok = error_message('Unrecognized floating center option',/traceback)
          range = double([0,1])
          return
        endelse
      endif else if scaling eq 2 then begin
        if floatingCenter eq 0 || floatingCenter eq 2 then begin
          centers[i] = mean(alog(*dataPTrs[i]),/nan)
        endif else if floatingCenter eq 1 || floatingCenter eq 3 then begin
          centers[i] = median(alog(*dataPTrs[i]))
        endif else begin
          self.statusBar->update,'Error: Unrecognized floating center option'
          ;ok = error_message('Unrecognized floating center option',/traceback)
          range = double([0,1])
          return
        endelse
      endif else begin
        if floatingCenter eq 0 || floatingCenter eq 2 then begin
          centers[i] = mean(*dataPTrs[i],/nan)
        endif else if floatingCenter eq 1 || floatingCenter eq 3 then begin
          centers[i] = median(*dataPTrs[i])
        endif else begin
          self.statusBar->update,'Error: Unrecognized floating center option'
          ;ok = error_message('Unrecognized floating center option',/traceback)
          range = double([0,1])
          return
        endelse
      endelse
      
    endfor
    
    center = mean(centers,/nan)
    
    range = [center-floatingSpan,center+floatingSpan]
    
    if keyword_set(boundfloating) then begin
      range[0] = max([range[0],minFloatRange],/nan)
      range[1] = min([range[1],maxFloatRange],/nan)
    endif
    
    range = double(range)
    
  endif else if rangeOption eq 0 then begin
  
    minRange = !values.d_nan
    maxRange = !values.d_nan
    
    for i = 0,n_elements(dataptrs) -1 do begin
    
      if ~ptr_valid(dataPtrs[i]) then continue
      
      if scaling eq 1 then begin
        minRangeTmp = min(alog10(*dataPtrs[i]),/nan)
        maxRangeTmp = max(alog10(*dataPtrs[i]),/nan)
      endif else if scaling eq 2 then begin
        minRangeTmp = min(alog(*dataPtrs[i]),/nan)
        maxRangeTmp = max(alog(*dataPtrs[i]),/nan)
      endif else begin
        minRangeTmp = min(*dataPtrs[i],/nan)
        maxRangeTmp = max(*dataPtrs[i],/nan)
      endelse
      
      if keyword_set(mirror) && ptr_valid(mirror[i]) then begin
        rangeAbs = max([abs(maxRangeTmp),abs(minRangeTmp)],/nan)
        maxRangeTmp = rangeAbs
        minRangeTmp = -rangeAbs
      endif
      
      minRange = min([minRange,minRangeTmp],/nan)
      maxRange = max([maxRange,maxRangeTmp],/nan)
      
    endfor
    
    margin = rangeMargin*(maxRange-minRange)
    
    if margin eq 0 && rangeMargin ne 0 then begin
    
      if maxRange eq 0 then begin
        margin = 1
      endif else begin
        margin = rangeMargin * maxRange
      endelse
    
    endif
    
    maxRange+= margin
    minRange-= margin
    
    if keyword_set(boundScaling) then begin
    
      if scaling eq 0 then begin
        minRange = max([minRange,minBoundRange],/nan)
        maxRange = min([maxRange,maxBoundRange],/nan)
      endif else if scaling eq 1 then begin
        minRange = max([minRange,alog10(minBoundRange)],/nan)
        maxRange = min([maxRange,alog10(maxBoundRange)],/nan)
      endif else if scaling eq 2 then begin
        minRange = max([minRange,alog(minBoundRange)],/nan)
        maxRange = min([maxRange,alog(maxBoundRange)],/nan)
      endif
      
      if minRange gt maxRange then begin
        self.statusBar->update,'No data found within range when using bound autoscaling range option.'
      ;  return
      endif
      
    endif
    
    range = double([minRange,maxRange])
    
  endif else begin
    self.statusBar->update,'Error: Illegal range option on axis'
    ;error_message,'Illegal range option on axis',/traceback
    range = double([0,1])
  endelse
  
  ;draw object shouldn't set properties
  ;  if scaling eq 0 then begin
  ;    axisSettings->setProperty,minFixedRange=range[0],maxFixedRange=range[1]
  ;  endif else if scaling eq 1 then begin
  ;    axisSettings->setProperty,minFixedRange=10.^range[0],maxFixedRange=10.^range[1]
  ;  endif else if scaling eq 2 then begin
  ;    axisSettings->setProperty,minFixedRange=exp(range[0]),maxFixedRange=exp(range[1])
  ;  endif
  
  if ~finite(range[0]) || ~finite(range[1]) then begin
    self.statusbar->update,'There is no real and finite data in the current range.'
    return
  endif
  
  if range[1]-range[0] lt 0 then begin
    self.statusbar->update,'Error: range interval is less than 0'
    self.historyWin->update,'Error: range interval is less than 0'
    return
  endif
  
  data_struct = {scaling:scaling,timeAxis:isTimeAxis,formatid:annotateStyle,exponent:annotateExponent}
  
  if range[1] eq range[0] then begin
 
    self.statusbar->update,'Warning: range interval is 0: ' +'Range: [' + formatannotation(0,0,range[0],data=data_struct)  + ',' + formatannotation(0,0,range[1],data=data_struct)  + ']
    self.historyWin->update,'Warning: range interval is 0: ' +'Range: [' + formatannotation(0,0,range[0],data=data_struct)  + ',' + formatannotation(0,0,range[1],data=data_struct)  + ']
  endif else begin
    
    self.historyWin->update,'Range: [' + formatannotation(0,0,range[0],data=data_struct)  + ',' + formatannotation(0,0,range[1],data=data_struct)  + '] will be used to plot.'
      
  endelse
  
  fail = 0
  
end

;returns a text object by digesting the settings on a thm_ui_text object
;text: a thm_ui_text object
;loc: the normalized location of the text, relative to the view [xpos,ypos,zpos]
;dim: the dimensions of the view in pixels
;offsetDirFlag: 0=centered,1=abovelocation,-1=belowlocation
;justify, -1 = left, 1=right, 0 = middle
function thm_ui_draw_object::getTextObject,text,loc,offsetDirFlag,orientation,justify=justify

  compile_opt idl2
  
  tFont = text->getGrFont()
  
  text->getProperty,color=color,value=value,show=show,size=size
  
  if orientation eq 0 then begin
    if offsetDirFlag eq 0 then begin
      yAlign = .5
    endif else if offsetDirFlag eq 1 then begin
      yAlign = 0.
    endif else begin
      yAlign = 1.
    endelse
    
    if ~keyword_set(justify) then begin
      xAlign = .5
    endif else if justify eq 1 then begin
      xAlign = 1.
    endif else if justify eq -1 then begin
      xAlign = 0.
    endif
    
    baseline = [1,0,0]
    updir = [0,1,0]
    
  endif else begin
  
    if offsetDirFlag eq 0 then begin
      xAlign=.5
    endif else if offsetDirFlag eq 1 then begin
      xAlign = 0.
    endif else begin
      xAlign = 1.
    endelse
    
    if ~keyword_set(justify) then begin
      yAlign = .5
    endif else if justify eq 1 then begin
      yAlign = 1.
    endif else if justify eq -1 then begin
      yAlign = 0.
    endif
    
    baseline=[0,1,0]
    updir = [-1,0,0]
    
  endelse
  
  if size eq 0 then begin
    show = 0
    size = 1D
  endif
  
  tFont->setProperty,size=size*self->getZoom()
  
  grText = obj_new('IDLgrText',value,$
    font=tFont,$
    color=self->convertColor(color),$
    hide=~show,$
    location=loc,$
    alignment=xAlign,$
    vertical_alignment=yalign,$
    baseline=baseline,$
    recompute_dimensions=0,$
    updir=updir)
    
    
  return,grText
  
end

;returns a text model by digesting the settings on a thm_ui_text object
;text: a thm_ui_text object
;loc: the normalized location of the text, relative to the view [xpos,ypos,zpos]
;dim: the dimensions of the view in pixels
;offsetDirFlag: 0=centered,1=abovelocation,-1=belowlocation
;justify, -1 = left, 1=right, 0 = middle
function thm_ui_draw_object::getTextModel,text,loc,offsetDirFlag,orientation,justify=justify

  compile_opt idl2
  
  if keyword_set(justify) then begin
    grText = self->getTextObject(text,loc,offsetDirFlag,orientation,justify=justify)
  endif else begin
    grText = self->getTextObject(text,loc,offsetDirFlag,orientation)
  endelse
  
  model = obj_new('IDLgrModel')
  
  model->add,grText
  
  return,model
  
end


;constructs a zaxis model for display, from a thm_ui_z_axis
pro thm_ui_draw_object::makeZAxisModel,zrange,zAxis,xplotpos,yplotpos,framecolor,framethick,model=model,palette=palette

  compile_opt idl2
  
  edgeTickMargin = .01
  
  zstack = .1
  thick = 10
  
  zAxis->getProperty, $
    tickNum=tickNum, $
    minorTickNum=minorTickNum,$
    annotationStyle=annotationStyle, $
    annotateTextObject=annotateTextObject, $
    annotationOrientation=annotationOrientation,$
    annotateExponent=annotateExponent,$
    labelTextObject=labelTextObject,$
    labelOrientation=labelOrientation,$
    labelMargin=labelMargin,$
    scaling=scaling,$
    placement=placement,$
    margin=margin
    
  ; minorTickNum = 3
    
  model = obj_new('IDLgrModel')
  
  palette = obj_new('IDLgrPalette')
  
  getctpath,colortablepath
  
  palette->loadCt,zAxis->getColorTableNumber(),file=colortablepath
  
  ;create location
  
  if placement eq 0 then begin ; top
  
    marginNorm1 = self->pt2Norm(margin,1)
    marginNorm2 = self->pt2Norm(margin+thick,1)
    
    pt1 = [0.,1. + marginNorm1/(yplotpos[1]-yplotpos[0]),zstack]
    pt2 = [1.,1. + marginNorm1/(yplotpos[1]-yplotpos[0]),zstack]
    pt3 = [1.,1. + marginNorm2/(yplotpos[1]-yplotpos[0]),zstack]
    pt4 = [0.,1. + marginNorm2/(yplotpos[1]-yplotpos[0]),zstack]
    
    location = pt1
    xsize = pt3[0]-pt1[0]
    ysize = pt3[1]-pt1[1]
    
  endif else if placement eq 1 then begin ; bottom
  
    marginNorm1 = self->pt2Norm(margin,1)
    marginNorm2 = self->pt2Norm(margin+thick,1)
    
    pt1 = [0.,0. - marginNorm1/(yplotpos[1]-yplotpos[0]),zstack]
    pt2 = [1.,0. - marginNorm1/(yplotpos[1]-yplotpos[0]),zstack]
    pt3 = [1.,0. - marginNorm2/(yplotpos[1]-yplotpos[0]),zstack]
    pt4 = [0.,0. - marginNorm2/(yplotpos[1]-yplotpos[0]),zstack]
    
    location = pt4
    xsize = pt4[0]-pt2[0]
    ysize = pt4[1]-pt2[1]
    
  endif else if placement eq 2 then begin ; left
  
    marginNorm1 = self->pt2Norm(margin,0)
    marginNorm2 = self->pt2Norm(margin+thick,0)
    
    pt1 = [0. - marginNorm1/(xplotpos[1]-xplotpos[0]),0.,zstack]
    pt2 = [0. - marginNorm1/(xplotpos[1]-xplotpos[0]),1.,zstack]
    pt3 = [0. - marginNorm2/(xplotpos[1]-xplotpos[0]),1.,zstack]
    pt4 = [0. - marginNorm2/(xplotpos[1]-xplotpos[0]),0.,zstack]
    
    location = pt4
    xsize = pt4[0]-pt2[0]
    ysize = pt4[1]-pt2[1]
    
  endif else if placement eq 3 then begin ;right
  
    marginNorm1 = self->pt2Norm(margin,0)
    marginNorm2 = self->pt2Norm(margin+thick,0)
    
    pt1 = [1. + marginNorm1/(xplotpos[1]-xplotpos[0]),0.,zstack]
    pt2 = [1. + marginNorm1/(xplotpos[1]-xplotpos[0]),1.,zstack]
    pt3 = [1. + marginNorm2/(xplotpos[1]-xplotpos[0]),1.,zstack]
    pt4 = [1. + marginNorm2/(xplotpos[1]-xplotpos[0]),0.,zstack]
    
    location = pt1
    xsize = pt3[0]-pt1[0]
    ysize = pt3[1]-pt1[1]
    
  endif else begin
  
    return
    
  endelse
  
  ;now create image for key
  
  poly = obj_new('IDLgrPolygon')
  
  if ~keyword_set(self.postscript) then begin
  
    ; if 0 then begin
  
    image = obj_new('IDLgrImage',indgen(1,256),palette=palette)
    
    poly->setProperty,data=[[pt1],[pt2],[pt3],[pt4]],texture_map = image,texture_coord=[[0,0],[0,1],[1,1],[1,0]],/texture_interp,color=self->convertColor([255,255,255])
    
  endif else begin
  
    poly = obj_new('IDLgrImage',indgen(1,256),palette=palette,location=location,dimensions=[xsize,ysize],depth_test_disable=2)
    
  ;This takes advantage of a little bit of linear algebra to split
  ;the four vertices into 257 different segments per side(total 514 vertices)
    
  ;    divisor = transpose(dindgen(257,3) mod 257) / 256
  ;
  ;    dif21 = rebin(pt2 - pt1,3,257)
  ;    dif34 = rebin(pt3 - pt4,3,257)
  ;
  ;    adder1 = rebin(pt1,3,257)
  ;    adder4 = rebin(pt4,3,257)
  ;
  ;    verts1 = dif21*divisor + adder1
  ;    verts2 = dif34*divisor + adder4
  ;
  ;    vertsall = [[verts1],[verts2]]
  ;
  ;    ;now calculate the vertex permutation
  ;
  ;    ;number of vertices per polygon
  ;    v_num = dblarr(256)+4
  ;
  ;    ;the permutation for each vertex of each polygon
  ;    ;first vertex of each polygon
  ;    v1 = lindgen(256)
  ;    ;second vertex of each polygon
  ;    v2 = lindgen(256)+1
  ;    ;third vertex of each polygon
  ;    v3 = lindgen(256)+257
  ;    ;fourth vertex of each polygon
  ;    v4 = lindgen(256)+257+1
  ;
  ;    v_perm = transpose([[v_num],[v1],[v2],[v4],[v3]])
  ;
  ;    ;generate the colors for the polygons
  ;
  ;    v_colors = lindgen(514) mod 256
  ;
  ;    poly->setProperty,$
  ;                data=vertsall,$
  ;                vert_colors=v_colors,$
  ;                shading=0,$
  ;                style=2,$
  ;                polygon=v_perm,$
  ;                palette=palette
    
  endelse
  
  ;now create axis
  
  if tickNum lt 0 then begin
  
    self.statusBar->update,'Illegal negative z axis major tick number, using 0 ticks'
    self.historyWin->update,'Illegal negative z axis major tick number, using 0 ticks'
    ticknum = 0
    
  endif
  
 ; tickValues = dindgen(tickNum + 2)/(tickNum+1)
  
  ;tickValues[tickNum+1] = 1D
 
  ;this is a bit of a kluge to deal with the
  ;case of a logarithmic z-axis with only 0 values
  ;To fix this we treat it as 0,0 range with log values
  if finite(zrange[0],/infinity,sign=-1) && $
     finite(zrange[1],/infinity,sign=-1) && $
     (scaling eq 1 || scaling eq 2) then begin
     
     scaling = 0
     zrange = [0D,0D]    
 
  endif
 
  if ~finite(zrange[0]) || ~finite(zrange[1]) then begin
    zrange = [0D,1D]
  endif
  
  if tickNum gt 0 then begin
  
    ;if both firstTick and majorTick are autoscaling
    ;then we use human readable tick algorithm
    self->goodTicks,0,zrange,scaling,tickNum, $
                  tickValues=tickValues,tickInterval=tickInterval,$
                  /nozero
                  
    if n_elements(tickValues) lt 2 && $
       tickNum ge 2 && $
       scaling eq 1 then begin
    
      self->logFixTicks,$
        zrange,$
        tickValues=tickValues,$
        tickInterval=tickInterval
        
      ticksFixed = 1
       
    endif
     

    ;if values aren't too close to the edge of the range, we 
    ;add leading and trailing ticks
    if n_elements(tickValues) gt 0 then begin       
     
      ;make sure ticks run to the edge of available range
      while tickValues[0] gt edgeTickMargin do begin
        tickValues = [tickValues[0]-tickInterval,tickValues]
      endwhile
        
      while tickValues[n_elements(tickValues)-1] lt (1-edgeTickMargin) do begin
        tickValues = [tickValues,tickValues[n_elements(tickValues)-1]+tickInterval]
      endwhile
      
      tickValuesForMinors = tickValues
      
      ;remove out of range major tick values
      idx = where(tickValues le 1. and tickValues ge 0.,c)
      if c gt 0 then begin
       
       edgeTickMargin = .01
       
       tickValues = tickValues[idx]
        
      endif else begin
        tickValues = tickValues[0]
        noMajors=1
      endelse
        
    endif else begin
      noMajors = 1
    endelse
  endif else begin
   
    tickValues = [0D,1D]
    tickValuesForMinors = tickValues
    tickInterval = 1
     
  endelse
  
  ;if we're using less than powers of 10 on a log axis
  ;Then force scientific notation
  if keyword_set(ticksFixed) && annotateExponent eq 0 then begin
    annotateExponent = 2
  endif
          
  data = {timeAxis:0,formatid:annotationStyle,scaling:scaling,range:zrange,exponent:annotateExponent}
  
  if placement eq 0 then begin
  
    if annotationOrientation eq 0 then begin
      annobaseline = [1,0,0]
      annoupdir = [0,1,0]
      annoalignment = [.5,0.0]
    endif else begin
      annobaseline = [0,1,0]
      annoupdir = [-1,0,0]
      annoalignment = [0.0,.5]
    endelse
    
    loc = [0D, [1. + marginNorm2/(yplotpos[1]-yplotpos[0])],zstack+.1]
    
    ticklen = self->pt2norm(thick,1)/(yplotpos[1]-yplotpos[0])
    
    tickDir = 0
    textpos = 1
    
    dir = 0
    
  endif else if placement eq 1 then begin
  
    if annotationOrientation eq 0 then begin
      annobaseline = [1,0,0]
      annoupdir = [0,1,0]
      annoalignment = [.5,1.0]
    endif else begin
      annobaseline = [0,1,0]
      annoupdir = [-1,0,0]
      annoalignment = [1.0,.5]
    endelse
    
    loc = [0D, [0. - marginNorm2/(yplotpos[1]-yplotpos[0])],zstack+.1]
    
    ticklen = self->pt2norm(thick,1)/(yplotpos[1]-yplotpos[0])
    
    tickDir = 1
    textpos = 0
    
    dir = 0
    
  endif else if placement eq 2 then begin
  
    if annotationOrientation eq 0 then begin
      annobaseline = [1,0,0]
      annoupdir = [0,1,0]
      annoalignment = [1.0,.5]
    endif else begin
      annobaseline = [0,1,0]
      annoupdir = [-1,0,0]
      annoalignment = [.5,0.0]
    endelse
    
    
    loc = [[0. - marginNorm2/(xplotpos[1]-xplotpos[0])],0D,zstack+.1]
    
    ticklen = self->pt2norm(thick,1)/(xplotpos[1]-xplotpos[0])
    
    tickDir = 1
    textpos = 0
    
    dir = 1
    
  endif else if placement eq 3 then begin
  
    if annotationOrientation eq 0 then begin
      annobaseline = [1,0,0]
      annoupdir = [0,1,0]
      annoalignment = [0.0,.5]
    endif else begin
      annobaseline = [0,1,0]
      annoupdir = [-1,0,0]
      annoalignment = [.5,1.0]
    endelse
    
    loc = [[1. + marginNorm2/(xplotpos[1]-xplotpos[0])],0D,zstack+.1]
    
    ticklen = self->pt2norm(thick,1)/(xplotpos[1]-xplotpos[0])
    
    tickDir = 0
    textpos = 1
    
    dir = 1
    
  endif
  
  if tickNum gt 0 && ~keyword_set(nomajors) then begin
  
    borderpos = [[pt4],[pt1],[pt2],[pt3]]
  
    ;calculate irregular minor tick spacing
    if scaling ne 0 && minorTickNum ne 0 then begin
    
      realTickNum = n_elements(tickValuesForMinors)
    
      ;make sure interval is regular. (This may mean that some values will be out of range, they will be clipped later)
;      if realTickNum gt 2 && tickValuesForMinors[1] - tickValuesForMinors[0] lt tickInterval then begin
;      
;        tickValuesForMinors[0] = tickValuesForMinors[1] - tickInterval
;      
;      endif
;      
;      if realTickNum gt 2 && tickValuesForMinors[realTickNum-1] - tickValuesForMinors[realTickNum-2] lt tickInterval then begin
;      
;        tickValuesForMinors[realTickNum-1] = tickValuesForMinors[realTickNum-2] + tickInterval 
;      
;      endif
    
      n = minorTickNum+1
      t = alog10((dindgen(n+1)/(n))*9 + 1)
      minorTickValues = t*tickInterval
      minorTickValues = minorTickValues[0:n]
      
      minorTickInd1 = lindgen((realTickNum+1)*(minorTickNum+1)) / long(minorTickNum+1)
      minorTickInd2 = lindgen((realTickNum+1)*(minorTickNum+1)) mod long(minorTickNum+1)
      
      minorTickValues = tickValuesForMinors[minorTickInd1]+minorTickValues[minorTickInd2]
      
      idx = where(minorTickValues lt 1. and minorTickValues gt 0.,c)
    ;  idx = where(minorTickValues lt max(tickValues) and minorTickValues gt 0.,c)
      
      if c gt 0 then begin
        minorTickValues = minorTickValues[idx]
      endif else begin
        minorTickValues = tickValuesForMinors[0]
      endelse
      
      axis_minor = obj_new('IDLgrAxis',dir, $
        range=[0D,1D], $
        location=loc, $
        minor=0,$
        tickLen = ticklen/2,$
        tickdir=tickDir,$
        tickValues=minorTickValues,$
        subTickLen=0,$
        thick=framethick,$
        tickfrmtdata=data,$
        tickformat='formatannotation',$
        color=self->convertColor(frameColor),$
        /notext,$
        /exact)
        
      model->add,axis_minor
      
      ;if we are manually setting minor ticks, then suppress them on the major axis
      minorTickNum = 0
      
    endif
    
    ;determine is any ticks are going to be auto-shifted into exponential
    ;if yes, make them all exponential
    if data.exponent eq 0 && (scaling eq 0 || scaling eq 1) then begin
      for i = 0,n_elements(tickValues)-1 do begin
        val = (tickValues[i] + data.range[0]/(data.range[1]-data.range[0]))*(data.range[1]-data.range[0])
        
        if scaling eq 1 then begin
          val = 10^val
        endif
        
        thm_ui_usingexponent,val,data,type=type
        
        if type ne 0 && scaling eq 0 then begin
          data.exponent = 2
          break
        endif else if type ne 0 && scaling eq 1 then begin
          data.exponent = 3
          break
        endif
      endfor
    endif

    if ~keyword_set(nomajors) then begin

      axis1 = obj_new('IDLgrAxis',dir, $
          range=[0D,1D], $
          location=loc, $
          minor=minorTickNum,$
          tickLen = ticklen,$
          tickdir=tickDir,$
          textpos=textpos,$
          tickValues=tickValues,$
          thick=framethick,$
          tickfrmtdata=data,$
          tickformat='formatannotation',$
          color=self->convertColor(frameColor),$
          textBaseLine = annobaseline,$
          textupdir = annoupdir, $
          textalignments = annoalignment, $
          /use_text_color,$
          /exact)
          
    endif else begin
    
       axis1 = obj_new('IDLgrAxis',dir, $
          range=[0D,1D], $
          location=loc, $
          minor=0,$
          tickLen = ticklen,$
          tickdir=tickDir,$
          textpos=textpos,$
          major=0,$
          thick=framethick,$
          tickfrmtdata=data,$
          tickformat='formatannotation',$
          color=self->convertColor(frameColor),$
          textBaseLine = annobaseline,$
          textupdir = annoupdir, $
          textalignments = annoalignment, $
          /use_text_color,$
          /exact)
    
    endelse
  
      
    axis1->getProperty,tickText=tickObj
    
    annofont = annotateTextObject->getGrFont()
    
    annotateTextObject->getProperty,color=annocolor,size=size
    
    annoFont->setProperty,size=self->getZoom()*size
    
    tickObj->setProperty,font=annofont,color=self->convertColor(annocolor)
    
    model->add,axis1
  
  endif else begin
  
    borderpos = [[pt4],[pt1],[pt2],[pt3],[pt4]]
  
  endelse
  
  ; now create label
  if obj_valid(labelTextObject) then begin
  
    if placement eq 0 then begin
    
      pos = [.5,1.+self->pt2norm(margin+thick+labelmargin,1)/(yplotpos[1]-yplotpos[0]),zstack+.1]
      
      offset = 1
      justify = 0
      
    endif else if placement eq 1 then begin
    
      pos = [.5,0.-self->pt2norm(margin+thick+labelmargin,1)/(yplotpos[1]-yplotpos[0]),zstack+.1]
      
      offset = -1
      justify = 0
      
    endif else if placement eq 2 then begin
    
      pos = [0.-self->pt2norm(margin+thick+labelmargin,0)/(xplotpos[1]-xplotpos[0]),.5,zstack+.1]
      
      if labelOrientation eq 0 then begin
        offset = 0
        justify = 1
      endif else begin
        offset = 0
        justify = -1
      endelse
      
    endif else if placement eq 3 then begin
    
      pos = [1.+self->pt2norm(margin+thick+labelmargin,0)/(xplotpos[1]-xplotpos[0]),.5,zstack+.1]
      
      if labelOrientation eq 0 then begin
        offset = 0
        justify = -1
      endif else begin
        offset = 0
        justify = 1
      endelse
      
    endif
    
    labelObj = self->getTextObject(labelTextObject,pos,offset,labelOrientation,justify=justify)
    
    model->add,labelObj
    
  endif
  
  border = obj_new('IDLgrPolyLine',borderpos,color=self->convertColor(frameColor),thick=framethick)
  
  model->add,border
  
  model->add,poly
  
  
end

;Function adapted from Graphics Gems:
;Heckbert, Paul S., Nice Numbers for Graph Labels, Graphics Gems, p. 61-63, code: p. 657-659
;It identifies the closest "nice" number
;Nice being a number j * 10 ^ n where j = 1,2,5 and n = any integer
function thm_ui_draw_object::niceNum,n,floor=floor,ceil=ceil,factors=factors,bias=bias

  compile_opt idl2

  ;factors = dindgen(100)+1
 ; factors = [1,2,5,10]

  ;this input used to determine the values that
  ;the leading digit(s) should snap to 
  if ~keyword_set(factors) then begin
    factors = [1,2,3,4,5,6,7,8,9,10]
  endif
  
  ;If the algorithm should consider more than 1 digit
  ;This value should grow(bias = 1 => 2 digits considered)
  if ~keyword_set(bias) then begin
    bias = 0
  endif

  exponent = floor(alog10(n))
  
  f = n/(10D^(exponent-bias))
  
 ; print,'f'+string(f)
  
  ;this version performs a floored operation
  
  if keyword_set(floor) then begin
    for i = 0,n_elements(factors)-1 do begin
  
      if i eq n_elements(factors)-1 || $
         f lt factors[i+1] then begin
        nf = factors[i]
        break
      endif 
      
    endfor
  endif else if keyword_set(ceil) then begin
    for i = 0,n_elements(factors)-1 do begin
  
      if  i eq n_elements(factors)-1 || $
          f le factors[i] then begin
          
        nf = factors[i]
        break
      endif 
    
    endfor
  
  endif else begin
  
    for i = 0,n_elements(factors)-1 do begin
    
      if  i eq n_elements(factors)-1 || $
          f lt (factors[i] + factors[i+1])/2D then begin
            
        nf = factors[i]
        break
      endif
       
    endfor
    
  endelse
;  
;  if keyword_set(floor) then begin
;    if f lt 2. then begin
;      nf = 1D
;    endif else if f lt 5. then begin
;      nf = 2D
;    endif else if f lt 10. then begin
;      nf = 5D
;    endif else begin
;      nf = 10D
;    endelse
;  endif else if keyword_set(ceil) then begin
;    if f le 1. then begin
;      nf = 1D
;    endif else if f le 2. then begin
;      nf = 2D
;    endif else if f le 5. then begin
;      nf = 5D
;    endif else begin
;      nf = 10D
;    endelse
;  endif else begin
;    ;this version performs a rounded operation
;    if f lt 1.5D then begin
;      nf = 1d
;    endif else if f lt 3D then begin
;      nf = 2d
;    endif else if f lt 7D then begin
;      nf = 5D
;    endif else begin
;      nf = 10D
;    endelse
;  endelse
 
  return, nf*(10D^(exponent-bias))

end

;Function adapted from Graphics Gems:
;Heckbert, Paul S., Nice Numbers for Graph Labels, Graphics Gems, p. 61-63, code: p. 657-659
;It identifies the closest "nice" number
;Nice being a number j * 10 ^ n where j = 1,2,5 and n = any integer
function thm_ui_draw_object::niceNumTime,n,_extra=ex

  compile_opt idl2

  if n le 60D then begin
    return,self->niceNum(n,_extra=ex)
  endif else if n le 60D*60D then begin
    return,self->niceNum(n/60D,_extra=ex)*60D
  endif else if n le 60D*60D*24D then begin
    return,self->niceNum(n/(60D*60D),_extra=ex)*60D*60D
  endif else begin
    return,self->niceNum(n/(60D*60D*24D),_extra=ex)*60D*60D*24D
  endelse
  
end

;This routine actually does the bulk of the work
;to select good ticks. 
pro thm_ui_draw_object::goodTicks,$
                        isTime,$ ;Boolean, is the axis a time axis
                        range, $ ;2 element double precision, axis range(in log space for log axes)
                        scaling, $ ; 0: linear,1: log10,2 logN
                        numTicks, $ ; The recommended number of ticks( 2 or more)
                        tickValues=tickValues, $ ; Returns the tick values here
                        tickInterval=tickInterval,$ ; Returns the spacing here
                        nozero=nozero ; set this keyword to guarantee there are never 0 ticks

  compile_opt idl2

  ;These values are output only, prevents previous iterations from interacting.
  if n_elements(tickValues) gt 0 then begin
    tmp = temporary(tickValues)
  endif
  
  if n_elements(tickInterval) gt 0 then begin
    tmp = temporary(tickInterval)
  endif

  ;if the range is too small then return with evenly spaced ticks
  ;this case generally means that all ticks will have the same
  ;value and should not ever happen on normal plots
  if (range[1] - range[0]) / (numTicks+1) eq 0 then begin
    tickValues = dindgen(numTicks+2)/(numTicks+1)
    tickInterval = 1D/(numTicks+1) 

    return
    
  endif 

  if istime then begin
  
    realTickIntervalFloor = self->nicenumtime((range[1]-range[0])/(numTicks+1),/floor)
    realTickIntervalCeil = self->nicenumtime((range[1]-range[0])/(numTicks+1),/ceil)
    
  endif else begin

    ;Use two different techniques to indentify nice ticks.
    ;The best result is used
  
    if scaling eq 0 || scaling eq 2 then begin
  
      ;Identify a nice interval close to the tick interval(from below)
      realTickIntervalFloor = self->nicenum((range[1]-range[0])/(numTicks+1),/floor,factors=[1D,2D,5D,10D])
      
      ;Identify a nice interval close to the tick interval(from above)
      realTickIntervalCeil = self->nicenum((range[1]-range[0])/(numTicks+1),/ceil,factors=[1D,2D,5D,10D])
      
    endif else begin
     
      ;the max function here disallows fractional exponents
      realTickIntervalFloor = max([self->nicenum((range[1]-range[0])/(numTicks+1),/floor),1D])
      
      realTickIntervalCeil = max([self->nicenum((range[1]-range[0])/(numTicks+1),/ceil),1D])
    
    endelse
    
  endelse
    
  ;Identify the value at which the ticks will start.
  tickStartFloor = ceil(range[0]/realTickIntervalFloor)*realTickIntervalFloor - range[0]
  ;Identify the value at which the ticks will stop.
  tickStopFloor =  floor(range[1]/realTickIntervalFloor)*realTickIntervalFloor - range[1]
  
  ;Identify the actual number of ticks that will be drawn.(numTicks is treated as an approximate value.
  realTickNumFloor = round((range[1]-range[0]+tickStopFloor-tickStartFloor)/realTickIntervalFloor + 1)
  
  ;Identify a nice interval close to the tick interval.
 
  
  ;Identify the value at which the ticks will start.
  tickStartCeil = ceil(range[0]/realTickIntervalCeil)*realTickIntervalCeil - range[0]
  ;Identify the value at which the ticks will stop.
  tickStopCeil =  floor(range[1]/realTickIntervalCeil)*realTickIntervalCeil - range[1]
  
  ;Identify the actual number of ticks that will be drawn.(numTicks is treated as an approximate value.
  realTickNumCeil = round((range[1]-range[0]+tickStopCeil-tickStartCeil)/realTickIntervalCeil + 1)
  
  if ~keyword_set(nozero) then begin
   if abs(numTicks-realTickNumCeil) lt abs(numTicks-realTickNumFloor) || $
     (abs(numTicks-realTickNumCeil) eq abs(numTicks-realTickNumFloor) && $
      realTickNumCeil gt realTickNumFloor) then begin
      realTickInterval = realTickIntervalCeil
      tickStart = tickStartCeil
      realTickNum = realTickNumCeil
    endif else begin
      realTickInterval = realTickIntervalFloor
      tickStart = tickStartFloor
      realTickNum = realTickNumFloor
    endelse
  endif else begin
   if (abs(numTicks-realTickNumCeil) lt abs(numTicks-realTickNumFloor) || $
     (abs(numTicks-realTickNumCeil) eq abs(numTicks-realTickNumFloor) && $ 
     realTickNumCeil gt realTickNumFloor)) && realTickNumCeil gt 0 then begin  
      realTickInterval = realTickIntervalCeil
      tickStart = tickStartCeil
      realTickNum = realTickNumCeil
    endif else begin
      realTickInterval = realTickIntervalFloor
      tickStart = tickStartFloor
      realTickNum = realTickNumFloor
    endelse
  
  endelse
      
  ;if we don't generate any ticks, lets get out of here
  if ~finite(realTickNum) || realTickNum le 0 then return
    
  
  ;Determine the tick values in range space.
  realTickValues = dindgen(realTickNum)*realTickInterval+tickStart
  
  ;Normalize the values for placement on the axis.
  tickValues = realTickValues/(range[1]-range[0])
  tickInterval = realTickInterval/(range[1]-range[0])

  return 

end

;It is sometimes the case that logarithmic axes end up with 1 or 0 ticks
;when more were requested.  This routine attempts to fix this problem
pro thm_ui_draw_object::logFixTicks,$
      range_in, $ ;2 element double precision, axis range(in log space)
      tickValues=tickValues, $ ; Returns the tick values here
      tickInterval=tickInterval ; Returns the spacing here
     
  numTicks = 2
  
  ;destroy any inputs via the outputs
  if n_elements(tickValues) gt 0 then begin
    tmp = temporary(tickValues)
  endif
  
  if n_elements(tickInterval) gt 0 then begin
    tmp = temporary(tickInterval)
  endif
  
  ;if the range is too small then return with evenly spaced ticks
  ;this case generally means that all ticks will have the same
  ;value and should not ever happen on normal plots
  if (range_in[1] - range_in[0]) / (numTicks+1) eq 0 then begin
    tickValues = dindgen(numTicks+2)/(numTicks+1)
    tickInterval = 1D/(numTicks+1) 

    return
    
  endif 
     
  range = 10^range_in
      
   ;the max function here disallows fractional exponents
  realTickIntervalFloor = self->nicenum((range[1]-range[0])/(numTicks+1),/floor,factors=[1,2,5,10])
      
  realTickIntervalCeil = self->nicenum((range[1]-range[0])/(numTicks+1),/ceil,factors=[1,2,5,10])
 
  ;Identify the value at which the ticks will start.
  tickStartFloor = ceil(range[0]/realTickIntervalFloor)*realTickIntervalFloor - range[0]
  ;Identify the value at which the ticks will stop.
  tickStopFloor =  floor(range[1]/realTickIntervalFloor)*realTickIntervalFloor - range[1]
  
  ;Identify the actual number of ticks that will be drawn.(numTicks is treated as an approximate value.
  realTickNumFloor = round((range[1]-range[0]+tickStopFloor-tickStartFloor)/realTickIntervalFloor + 1)
  
  ;Identify a nice interval close to the tick interval.
 
  ;Identify the value at which the ticks will start.
  tickStartCeil = ceil(range[0]/realTickIntervalCeil)*realTickIntervalCeil - range[0]
  ;Identify the value at which the ticks will stop.
  tickStopCeil =  floor(range[1]/realTickIntervalCeil)*realTickIntervalCeil - range[1]
  
  ;Identify the actual number of ticks that will be drawn.(numTicks is treated as an approximate value.
  realTickNumCeil = round((range[1]-range[0]+tickStopCeil-tickStartCeil)/realTickIntervalCeil + 1)
  
  if (abs(numTicks-realTickNumCeil) lt abs(numTicks-realTickNumFloor) || $
     (abs(numTicks-realTickNumCeil) eq abs(numTicks-realTickNumFloor) && $ 
    realTickNumCeil gt realTickNumFloor)) && realTickNumCeil gt 0 then begin  
    realTickInterval = realTickIntervalCeil
    tickStart = tickStartCeil
    realTickNum = realTickNumCeil
  endif else begin
    realTickInterval = realTickIntervalFloor
    tickStart = tickStartFloor
    realTickNum = realTickNumFloor
  endelse
  
  ;if we don't generate any ticks, lets get out of here
  if ~finite(realTickNum) || realTickNum le 0 then return
    
    
  ;Determine the tick values in range space.
  realTickValues = dindgen(2)*realTickInterval+tickStart
  
  ;relog
  realTickValues = alog10(realTickValues+range[0])
  
  ;Normalize the values for placement on the axis.
  tickValues = (realTickValues-range_in[0])/(range_in[1]-range_in[0])
  tickInterval = tickValues[1]-tickValues[0]

  return 
      
end
      
      
      
;Determines where to place the tick marks for
;an axis, and deals with the various input validation issues
;and positioning options.
;If they are placed automatically,
;They should be at human readable values, if possible.
;In this case human readable means that if the axis
;is non time, the ticks will be at values of
;1*10^n or 2*10^n or 5*10^n where n is some number
;appropriate to the scale of the axis
;If the axis is a time axis the ticks will be at
;1,2,5 * 10^n or 60*1,2,5*10^n or 60*60*1,2,5*10^n or
;24*60*60*1,2,5*10^n  With selection/n dependent on scale
;
pro thm_ui_draw_object::placeMajorTicks, $
                        isTime,$  ;Boolean, is the axis a time axis
                        range, $  ;2 element double precision, axis range(in log space for log axes)
                        scaling, $ ; 0: linear,1: log10,2 logN
                        majorTickAuto, $ ; Boolean, automatically position major ticks? (If this is 0, ticks will not be placed at human readable values)
                        firstTickAuto, $ ; Boolean, automatically position tick start?  (If this is 0, ticks will not be placed at human readable values)
                        numMajorTicks, $ ; number of major ticks(only used if autopositioning is on)
                        firstTickAt, $ ; the location of the first tick(only used if autopositioning is off)
                        majorTickSpace, $ ; the space between major ticks(only used if autopositioning is off)
                        majorTickValues=majorTickValues,$ ; the locations of major ticks are returned as an array from this argument
                        majorTickInterval=majorTickInterval,$ ; the final space between ticks
                        fail=fail  ;Boolean, 1 indicates a failure
          
    
  compile_opt idl2
          
  fail = 1
  
  ;this margin determines whether or not another
  ;tick should be added at the beginning or end to create a full range
  edgeTickMargin = .01
          
  ;adding extra ticks for start and stop ticks
  
  ;if both firstTick and majorTick are autoscaling
  ;then we use human readable tick algorithm
  if firstTickAuto && majorTickAuto then begin
  
    if numMajorTicks gt 0 then begin
      self->goodTicks,isTime,$ ;Boolean, is the axis a time axis
                    range, $ ;2 element double precision, axis range(in log space for log axes)
                    scaling, $ ; 0: linear,1: log10,2 logN
                    numMajorTicks, $ ; The recommended number of ticks( 2 or more)
                    tickValues=majorTickValues, $ ; Returns the tick values here
                    tickInterval=majorTickInterval ; Returns the spacing here
     
      ;if keyword_set(majorTickValues) then begin
     if keyword_set(majorTickValues) && keyword_set(majorTickInterval) then begin       
      
        if majorTickValues[0] gt edgeTickMargin then begin
          majorTickValues = [0,majorTickValues]
        endif
        
        if majorTickValues[n_elements(majorTickValues)-1] lt (1-edgeTickMargin) then begin
          majorTickValues = [majorTickValues,1]
        endif
      endif else begin
        majorTickValues = [0,1]
        majorTickInterval = 1
      endelse
    endif else begin
      majorTickValues = [0,1]
      majorTickInterval = 1
    endelse
      
    fail = 0
    return
  
  endif
  
  numMajorTicks += 2
  
  if firstTickAuto then begin
    tickStart = 0.
  endif else begin
  
   if firstTickAt lt 0 then begin
     self.statusBar->update,'First tick at is negative, Using 0 first value'
     self.historywin->update,'First tick at is negative, Using 0 first value'
     tickStart = 0
   endif else if firstTickAt ge 1 then begin
     self.statusBar->update,'First tick at is larger than range, Using 0 first value'
     self.historywin->update,'First tick at is larget than range, Using 0 first value'
     tickStart = 0
   endif else begin
     tickstart = firstTickAt
   endelse
    
  endelse
  
  if majorTickAuto then begin
  
    if numMajorTicks lt 2 then begin
    
      self.statusBar->update,'Num Major Ticks set to less than 0.  Using 0 ticks'
      self.historyWin->update,'Num Major Ticks set to less than 0.  Using 0 ticks'
      
      numMajorTicks = 2
      
    endif
    
    if range[1] eq range[0] then begin
      tickStart = 0D
    endif else begin
      tickStart = tickStart
    endelse
    
    majorTickInterval = (1-tickStart)/(numMajorTicks-1)
    
    ;calculate the location of each tick using the provided info.
    majorTickValues = dindgen(numMajorTicks)*majorTickInterval+tickStart
    fail = 0
    return
    
  endif else begin
  
    if range[1] eq range[0] then begin
      self.statusBar->update,'Max and Min range are the same cannot determine major tick spacing.'
      self.historyWin->update,'Max and Min range are the same cannot determine major tick spacing.'
      
      fail = 0
      majorTickValues = dindgen(3)/2
      majorTickInterval = .5
      return
  
    endif else begin
    
      if majorTickSpace le 0 then begin
        self.statusBar->update,'Major tick spacing set to less than 0.  Using current range'
        self.historyWin->update,'Major tick spacing set to less than 0.  Using current range'
        
        fail = 0
        majorTickValues = dindgen(3)/2
        majorTickInterval = .5
        return
      endif else if (1-tickStart) lt majorTickSpace then begin
        self.statusBar->update,'Major tick spacing too large.  Using defaults'
        self.historyWin->update,'Major tick spacing too large.  Using defaults'
        
        fail = 0
        majorTickValues = dindgen(3)/2
        majorTickInterval = .5
        return
      
      endif else begin
       
        majorTickInterval = majorTickSpace
        numMajorTicks = floor((1-tickStart) / majorTickInterval)+1
      
        fail = 0
        ;calculate the location of each tick using the provided info.
        majorTickValues = dindgen(numMajorTicks)*majorTickInterval+tickStart
        return
        
      endelse
    endelse
  endelse 
  
end
                        
;Constructs an X or Y axis model from a thm_ui_axis_settings object
;dir = direction of the axis(0=x,1=y,2=z)
;xrange,yrange: data range of the panel
;axisSettings: axis settings object
;windowDim: device size of the window dimension perpendicular to the axis
;plotDim1: normalized length of the plot on the dimension perpendicual to the axis
;plotDim2: normalized length of the plot on the dimension parallel to the axis
;color: The color of the plot frame
;thick: The thickness of the plot frame/ticks
;model: the axis model is returned in this keyword
;majorTickValues:the major tickvalues are returned in this keyword
;isTimeAxis: flag indicating whether the axis is time annotated or not returned in this keyword
;labelPos: this keyword returns the most distant label position from the axis
pro thm_ui_draw_object::makeXYAxisModel,dir,xrange,yrange,scaling,axisSettings,plotDim1,plotdim2,color,thick,bgcolor,gridmodel=gridmodel,model=model,majors=majorTickValues,isTimeAxis=isTimeAxis,labelPos=labelPos,fail=fail

  compile_opt idl2
  
  fail = 1
  
  noMajor = 0
  noMinor = 0
  
  axisSettings->getProperty,$
    annotateOrientation=annotateOrientation,$
    placeAnnotation=placeAnnotation,$
    annotateAxis=annotateAxis,$
    isTimeAxis=isTimeAxis,$
    annotateMajorTicks=annotateMajorTicks,$
    annotateFirstTick=annotateFirstTick,$
    annotateLastTick=annotateLastTick,$
    annotateStyle=annotateStyle,$
    annotateTextObject=annotateTextObject,$
    annotateExponent=annotateExponent,$
    majorTickAuto=majorTickAuto,$
    minorTickAuto=minorTickAuto,$
    firstTickAuto=firstTickAuto,$
    majorLength=majorLength,$
    minorLength=minorLength,$
    bottomPlacement=bottomPlacement,$
    topPlacement=topPlacement,$
    tickStyle=tickStyle,$
    numMajorTicks=numMajorTicks,$
    numMinorTicks=numMinorTicks,$
    lineAtZero=lineAtZero,$
    majorGrid=majorGrid,$
    minorGrid=minorGrid,$
    margin=margin,$
    showLabels=showLabels,$
    labels=labels,$
    orientation=orientation,$
    stackLabels=stackLabels   
    
  xrange = double(xrange)
  yrange = double(yrange)
  plotdim1 = double(plotdim1)
  
  ; xconv = [-xrange[0]/(xrange[1]-xrange[0]),1./(xrange[1]-xrange[0])]
  ; yconv = [-yrange[0]/(yrange[1]-yrange[0]),1./(yrange[1]-yrange[0])]
  
  maxAnnoNum = 200
  
  axiszstack = .25D
  gridzstack = .07D
  linezstack = .08D
  AnnotationDepth = .05D
  
  tickSpacing = axisSettings->getTickSpacing()
  
  location1 = [0.,0.,axiszstack]
  
  if dir eq 0 then begin
    range = xrange
    location2 = [0.,1.,axiszstack]
    
    tickconversion = 1./(plotdim1[1]-plotdim1[0])
    
  endif else begin
    range=yrange
    location2 = [1.,0.,axiszstack]
    
    tickconversion = 1./(plotdim1[1]-plotdim1[0])
  endelse
  
  maxMajorNum = 200
  maxMinorNum = 400
  
  ;transform ticklength from pixels to data coordinates
  majTicklen = self->pt2norm(majorLength,~dir)*tickConversion
  minTicklen = self->pt2norm(minorLength,~dir)*tickConversion
  
  model = obj_new('IDLgrModel')
  gridmodel = obj_new('IDLgrModel')
  
  ;just a dummy parameter for axis objects
  tickNum = -1
  
;  if firstTickAuto then begin
;    tickStart = 0.
;  endif else begin
;  
;    if tickSpacing[0] lt 0 then begin
;      self.statusBar->update,'First tick at is negative, Using 0 first value'
;      self.historywin->update,'First tick at is negative, Using 0 first value'
;      tickSpacing[0] = 0
;    endif
;    
;    tickstart = tickSpacing[0]
;  endelse
;  
;  if majorTickAuto then begin
;  
;    if numMajorTicks lt 0 then begin
;    
;      self.statusBar->update,'Num Major Ticks set to less than 0.  Using 0 ticks'
;      self.historyWin->update,'Num Major Ticks set to less than 0.  Using 0 ticks'
;      
;      numMajorTicks = 0
;      
;    endif else if range[1] eq range[0] then begin
;      tickStart = 0D
;    endif else begin
;      tickStart = (tickStart)/(range[1]-range[0])
;    endelse
;    
;    tickSpacingMajor = (1-tickStart)/(numMajorTicks+1)
;  endif else begin
;  
;    if range[1] eq range[0] then begin
;      self.statusBar->update,'Max and Min range are the same cannot determine major tick spacing.'
;      self.historyWin->update,'Max and Min range are the same cannot determine major tick spacing.'
;      
;      numMajorTicks = 3
;      tickStart = 0
;      tickSpacingMajor = 1D/3
;  
;    endif else begin
;    
;      if tickSpacing[1] le 0 then begin
;        self.statusBar->update,'Major tick spacing set to less than 0.  Using current range'
;        self.historyWin->update,'Major tick spacing set to less than 0.  Using current range'
;        
;        tickSpacing[1] = range[1]-range[0]
;      endif
;    
;      
;      numMajorTicks = floor((range[1] - (range[0]+tickStart)) / tickSpacing[1])
;      tickStart = tickStart / (range[1]-range[0])
;      tickSpacingMajor = tickSpacing[1] / (range[1]-range[0])
;    endelse
;  endelse

  if ~finite(range[1]) || ~finite(range[0]) then begin
    range = [0D,1D]
  endif

  self->placeMajorTicks,$
    isTimeAxis,$
    range,$
    scaling,$
    majorTickAuto,$
    firstTickAuto,$
    numMajorTicks,$
    tickSpacing[0]/(range[1]-range[0]),$
    tickSpacing[1]/(range[1]-range[0]),$
    majorTickValues=majorTickValues,$
    majorTickInterval=tickSpacingMajor,$
    fail=placefail
    
;  if ~keyword_set(tickSpacingMajor) then stop
    
  if placefail then begin
    nomajor = 1
    nominor = 1
  endif
  
  numMajorTicks = n_elements(majorTickValues)
    
  if ~minorTickAuto then begin
  
    if range[1] eq range[0] then begin
    
      self.statusBar->update,'Max and Min range are the same cannot determine minor tick spacing.'
      self.historyWin->update,'Max and Min range are the same cannot determine minor tick spacing.'
      
      tickSpacingMinor = tickSpacingMajor/(numMinorTicks+1)
    endif else begin
    
      tickSpacingMinor = tickSpacing[2]/(range[1]-range[0])
      numMinorTicks = floor(tickSpacingMajor/tickSpacingMinor)
      
      if tickSpacingMajor lt tickSpacingMinor then begin
        self.statusBar->update,'Minor Tick Spacing larger than the spacing between Major Ticks.  No Major Ticks will be drawn'
        self.historyWin->update,'Minor Tick Spacing larger than the spacing between Major Ticks.  No Major Ticks will be drawn'
        noMinor = 1
      endif
    endelse
    
  endif else begin
    tickSpacingMinor = tickSpacingMajor/(numMinorTicks+1)
  endelse   
  
 ; numMajorTicks+=1
  numMinorTicks+=1
  
;  if numMajorTicks le 0 then begin
;    ok = error_message('ERROR:  The current settings will result in the creation of less than or equal to 0 major ticks. Draw Operation Failed',/center)
;    return
;  endif

  
  if numMinorTicks le 0 then begin
    ok = error_message('ERROR:  The current settings will result in the creation of less than or equal to 0 minor ticks. Draw Operation Failed',/center)
    return
  endif
  
  ;calculate the location of each tick using the provided info.
  ;majorTickValues = dindgen(numMajorTicks)*tickSpacingMajor+tickStart
  
  ;if auto tick spacing is being used and scaling is logarithmic
  ;use unevenly spaced minor ticks to show the user that the axis has log scale
  if minorTickAuto && scaling ne 0 then begin
    n = numMinorTicks
    t = alog10((dindgen(n+1)/(n))*9 + 1)
    minorTickValues = t*tickSpacingMajor
    minorTickValues = minorTickValues[0:n]
  endif else begin
    minorTickValues = dindgen(numMinorTicks)*tickSpacingMinor
  endelse
  
  majorTickValuesForMinors = majorTickValues
  
  if numMajorTicks gt 2 && majorTickValuesForMinors[1] - majorTickValuesForMinors[0] lt tickSpacingMajor then begin
  
    majorTickValuesForMinors[0] = majorTickValuesForMinors[1] - tickSpacingMajor 
  
  endif
  
  if numMajorTicks gt 2 && majorTickValuesForMinors[numMajorTicks-1] - majorTickValuesForMinors[numMajorTicks-2] lt tickSpacingMajor then begin
  
    majorTickValuesForMinors[numMajorTicks-1] = majorTickValuesForMinors[numMajorTicks-2] + tickSpacingMajor 
  
  endif
  
  minorTickInd1 = indgen(numMajorTicks*numMinorTicks) / long(numMinorTicks)
  minorTickInd2 = indgen(numMajorTicks*numMinorTicks) mod long(numMinorTicks)
  
  minorTickValues = majorTickValuesForMinors[minorTickInd1]+minorTickValues[minorTickInd2]
  
  idx = where(minorTickValues lt 1. and minorTickValues gt 0.,c)
  if c gt 0 then begin
    minorTickValues = minorTickValues[idx]
  endif else begin
    minorTickValues = majorTickValues[0]
    noMinor=1
  endelse
  
;  idx = where(majorTickValues lt 1. and majorTickValues gt 0.,c)
;  if c gt 0 then begin
;   
;   edgeTickMargin = .01
;   
;   majorTickValues = majorTickValues[idx]
;    
;   if majorTickValues[0] gt edgeTickMargin then begin
;    majorTickValues = [0,majorTickValues]
;   endif
;        
;   if majorTickValues[n_elements(majorTickValues)-1] lt (1-edgeTickMargin) then begin
;    majorTickValues = [majorTickValues,1]
;   endif
;    
;  endif else begin
;    majorTickValues = majorTickValues[0]
;    noMajor=1
;  endelse
  
  if n_elements(majorTickValues) gt maxMajorNum ||  n_elements(minorTickValues) gt maxMinorNum then begin
    ok = error_message('ERROR:  The current settings will result in the creation of ' + strcompress(string(n_elements(majorTickValues)),/remove_all) + ' major ticks and ' + strcompress(string(n_elements(minorTickValues)),/remove_all) + ' minor ticks.  Draw operation failed.',/center)
    return
  endif
  
  if (range[0] + tickSpacing[0] gt range[1] && ~firstTickAuto) && range[1] ne range[0] then begin
    self.statusBar->update,'"First Tick At" larger than available range. No ticks will be drawn'
    self.historyWin->update,'"First Tick At" larger than available range. No ticks will be drawn'
    noMinor = 1
    noMajor = 1
  endif
  
  ;majorTicks
  if topPlacement then topTickLen = majTickLen else topTickLen = 0.
  
  if bottomPlacement then bottomTickLen = majTickLen else bottomTickLen = 0.
  
  if ~noMajor then begin
  
    ;outside ticks
    if tickStyle eq 1 || tickStyle eq 2 then begin
    
      ;bottom axis
      axis1 = OBJ_NEW('IDLgrAxis',dir,$
        range=[0D,1D],$
        location=location1,$
        color=self->convertColor(color),$
        thick=thick,$
        /exact,$
        /notext,$
        tickLen = bottomTickLen, $
        tickDir=1,$
        minor=0,$
        major=tickNum,$
        tickValues=majorTickValues)
        
      ;top axis
      axis2 = OBJ_NEW('IDLgrAxis',dir,$
        range=[0D,1D],$
        location=location2,$
        color=self->convertColor(color),$
        thick=thick,$
        /exact,$
        /notext,$
        tickLen = topTickLen, $
        tickDir=0,$
        minor=0,$
        major=ticknum,$
        tickValues=majorTickValues)
        
      noText1 = 1
      noText2 = 1
      
      model->add,axis1
      
      model->add,axis2
      
    endif
    
    ;inside ticks
    if tickStyle eq 0 || tickStyle eq 2 then begin
    
      ;bottom axis
      axis1 = OBJ_NEW('IDLgrAxis',dir,$
        range=[0D,1D],$
        location=location1,$
        color=self->convertColor(color),$
        thick=thick,$
        /exact,$
        /notext,$
        tickLen = bottomTickLen, $
        tickDir=0,$
        minor=0, $
        major=ticknum,$
        tickValues=majorTickValues)
        
      ;top axis
      axis2 = OBJ_NEW('IDLgrAxis',dir,$
        range=[0D,1D],$
        location=location2,$
        color=self->convertColor(color),$
        thick=thick,$
        /exact,$
        /notext,$
        tickLen = topTickLen, $
        tickDir=1,$
        minor=0,$
        major=ticknum,$
        tickValues=majorTickValues)
        
      model->add,axis1
      
      model->add,axis2
      
    endif
  endif else begin ;no major ticks
  
    ;bottom axis
    axis1 = OBJ_NEW('IDLgrAxis',dir,$
      range=[0D,1D],$
      location=location1,$
      color=self->convertColor(color),$
      thick=thick,$
      /exact,$
      /notext,$
      minor=0,$
      major=0)
      
    ;top axis
    axis2 = OBJ_NEW('IDLgrAxis',dir,$
      range=[0D,1D],$
      location=location2,$
      color=self->convertColor(color),$
      thick=thick,$
      /exact,$
      /notext,$
      minor=0,$
      major=0)
      
    model->add,axis1
    
    model->add,axis2
    
  endelse
  
  ;perform annotation on Axis
  if annotateAxis then begin
  
    noAnno = 0
  
    if range[1] eq range[0] && ~annotateMajorTicks then begin
      self.statusBar->update,'Max and Min range are the same: Annotating major ticks.'
      self.historyWin->update,'Max and Min range are the same: Annotating major ticks.'
      annotateMajorTicks=1
    endif
  
    if annotateMajorTicks then begin
      if ~noMajor then begin
        axis1->getProperty,tickValues=annotateValues
      endif else begin
        noAnno = 1
        annotateValues = [0,1]
        self.historyWin->update,'Cannot annotate major ticks, because no major ticks being drawn'
      endelse
    endif else begin
    
      if tickSpacing[3] lt 0 then begin
        self.statusBar->update,'"First Annotation At" negative, using 0 value'
        self.historyWin->update,'"First Annotation At" negative, using 0 value'
        tickSpacing[3] = 0
      endif
      
      numAnnotate = floor((range[1] - (range[0]+tickSpacing[3])) / tickSpacing[4])+1
      
      if numAnnotate gt 0  && (range[0] + tickSpacing[3]) lt range[1] then begin
        annotateValues = (dindgen(numAnnotate)*tickSpacing[4]+tickSpacing[3])/(range[1]-range[0])
      endif else begin
        self.statusBar->update,'No annotations in selected range'
        self.historyWin->update,'No annotations in selected range'
        annotateValues = [0,1]
        noAnno = 1
      endelse
    endelse
    
    if ~annotateLastTick && ~noAnno then begin
      if n_elements(annotateValues) eq 1 then begin
        noAnno = 1
      endif else begin
        annotateValues = annotateValues[0:n_elements(annotateValues)-2]
      endelse
    endif
  
    if ~annotateFirstTick && ~noAnno then begin
      if n_elements(annotateValues) eq 1 then begin
        noAnno = 1
      endif else begin
        annotateValues = annotateValues[1:*]
      endelse
    endif
    
    if ~noAnno then begin
    
      ;Check if there are too many annotations.
      if n_elements(annotateValues) gt maxAnnoNum then begin
        ok = error_message('ERROR:  The current settings will result in the creation of ' + strcompress(string(n_elements(annotateValues)),/remove_all) + ' annotations.  Draw operation failed.',/center)
        return
      endif
      
      data = {timeAxis:isTimeAxis,formatid:AnnotateStyle,scaling:scaling,range:range,exponent:annotateExponent}
      
      if placeAnnotation eq 1 then begin
        annoPos = 1
      endif else begin
        annoPos = 0
      endelse
      
      if tickStyle eq 1 || tickStyle eq 2 then begin
        AnnoColor = bgcolor ; the color to draw the axis so that it won't be visible
        ;we draw real but invisible ticks so that the annotations will be shifted out of the way of collision with the ticks
        if placeAnnotation eq 1 then begin
          AnnoDir = 0
          AnnoLoc = location2
          AnnoLen = topTickLen
        endif else begin
          AnnoDir = 1
          AnnoLoc = location1
          AnnoLen = bottomTickLen
        endelse
      endif else if tickStyle eq 0 then begin
        AnnoColor = color
        AnnoLen = 0 ;when ticks are only on the inside, ticklength can be 0, because there is no need to shift the annotations down.w
        if placeAnnotation eq 1 then begin
          AnnoDir = 1
          AnnoLoc = location2
        endif else begin
          AnnoDir = 0
          AnnoLoc = location1
        endelse
      endif else begin
        self.statusBar->update,'Error: Illegal tickstyle'
        ;ok = error_message('Illegal tickstyle',/traceback)
        return
      endelse
      
      if annotateOrientation eq 1 then begin
        annoBaseLine = [0,1,0]
        annoUpDir = [-1,0,0]
        if dir eq 1 then begin
          if annoPos eq 0 then begin
            annoAlign = [.5,0.]
          endif else begin
            annoAlign = [.5,1.]
          endelse
        endif else begin
          if AnnoPos eq 0 then begin
            annoAlign = [1.,.5]
          endif else begin
            annoAlign = [0.,.5]
          endelse
        endelse
      endif else begin
        annoBaseLine = [1,0,0]
        annoUpDir = [0,1,0]
      ;annoAlign = [1.,-.5]
      endelse
      
      labFont = annotateTextObject->getGrFont()
      
      annoLoc[2] = AnnotationDepth
      
      annotateTextObject->getProperty,color=labcolor,size=size
      
      if size le 0 then begin
        size = 1D
        fontshow = 0
      endif else begin
        fontshow = 1
      endelse
      
      labFont->setProperty,size=self->getZoom()*size
      
      ; labFont->setProperty,size=self->getZoom()*size
      
      ;determine is any ticks are going to be auto-shifted into exponential
      ;if yes, make them all exponential
;     
;      
      if ~isTimeAxis && data.exponent eq 0 && (scaling eq 0 || scaling eq 1) then begin
        for i = 0,n_elements(annotateValues)-1 do begin
          val = (annotateValues[i] + data.range[0]/(data.range[1]-data.range[0]))*(data.range[1]-data.range[0])
          
          if scaling eq 1 then begin
            val = 10^val
          endif
          
          thm_ui_usingexponent,val,data,type=type
          
          if type ne 0 && scaling eq 0 then begin
            data.exponent = 2
            break
          endif else if type ne 0 && scaling eq 1 then begin
            data.exponent = 3
            break
          endif
        endfor
      endif

      annaxis = OBJ_NEW('IDLgrAxis',dir,$
        range=[0D,1D],$
        location=AnnoLoc,$
        alpha_channel = 1.0,$
        thick=thick,$
        color=AnnoColor,$
        /exact,$
        tickLen = AnnoLen, $
        tickDir=AnnoDir,$
        textPos=AnnoPos,$
        minor=0,$
        major=-1,$
        tickValues=annotateValues,$
        /use_text_color, $
        tickfrmtdata=data,$
        tickformat='formatannotation',$
        textBaseline=annoBaseLine,$
        textupdir=annoUpDir,$
        textAlignments=annoAlign)
        
        
      annaxis->getProperty,tickText=tickObj
      
      tickObj->setProperty,font=labFont,color=self->convertColor(labcolor),alpha_channel=1.0,hide=~fontshow,/enable_formatting
      
      model->add,annaxis
      
    endif
  endif
  
  ;minorTicks
  
  if topPlacement then topTickLen = minTickLen else topTickLen = 0.
  
  if bottomPlacement then bottomTickLen = minTickLen else bottomTickLen = 0.
  
  if ~noMinor then begin
  
    ;outside ticks
    if tickStyle eq 1 || tickStyle eq 2 then begin
    
      ;bottom axis
      axis1 = OBJ_NEW('IDLgrAxis',dir,$
        range=[0D,1D],$
        location=location1,$
        color=self->convertColor(color),$
        thick=thick,$
        /exact,$
        /notext,$
        tickLen = bottomTickLen, $
        tickDir=1,$
        minor=0,$
        major=ticknum,$
        tickValues=minorTickValues)
        
      ;top axis
      axis2 = OBJ_NEW('IDLgrAxis',dir,$
        range=[0D,1D],$
        location=location2,$
        color=self->convertColor(color),$
        thick=thick,$
        /exact,$
        /notext,$
        tickLen = topTickLen, $
        tickDir=0,$
        minor=0,$
        major=ticknum,$
        tickValues=minorTickValues)
        
      noText1 = 1
      noText2 = 1
      
      model->add,axis1
      
      model->add,axis2
      
    endif
    
    ;inside ticks
    if tickStyle eq 0 || tickStyle eq 2 then begin
    
      ;bottom axis
      axis1 = OBJ_NEW('IDLgrAxis',dir,$
        range=[0D,1D],$
        location=location1,$
        color=self->convertColor(color),$
        thick=thick,$
        /exact,$
        /notext,$
        tickLen = bottomTickLen, $
        tickDir=0,$
        minor=0., $
        major=ticknum,$
        tickValues=minorTickValues,$
        log=log)
        
      ;top axis
      axis2 = OBJ_NEW('IDLgrAxis',dir,$
        range=[0D,1D],$
        location=location2,$
        color=self->convertColor(color),$
        thick=thick,$
        /exact,$
        /notext,$
        tickLen = topTickLen, $
        tickDir=1,$
        minor=0,$
        major=ticknum,$
        tickValues=minorTickValues)
        
      model->add,axis1
      
      model->add,axis2
      
    endif
  endif
  
  ;draw lineatzero
  if lineatzero && $
    0. gt range[0] && $
    0. lt range[1] then begin
    
    if dir eq 0 then begin
      locationx = replicate((0-range[0])/(range[1]-range[0]),2)
      locationy = [0D,1D]
    endif else begin
      locationx = [0D,1D]
      locationy = replicate((0-range[0])/(range[1]-range[0]),2)
    endelse
    
    locationz = [linezstack,linezstack]
    
    line = obj_new('IDLgrPolyline',locationx,locationy,locationz,$
      color = self->convertColor([0,0,0]))
      
    model->add,line
    
  endif
  
  majorGridShow = 0 ; needed to generate minor grid
  
  ;add major grid lines
  if obj_valid(majorGrid) && obj_isa(majorgrid,'thm_ui_line_style') && ~noMajor then begin
    majorGrid->getProperty,id=gridstyle,$
      color=gridcolor,$
      show=majorgridshow,$
      thickness=gridthick,$
      opacity=opacity
      
      
    if majorgridshow && range[1] ne range[0] then begin
      ;get tick values, (may have been automatically set)
      tickValues = majorTickValues
      normGridThick = self->pt2norm(gridthick,dir)/(2D*(plotDim2[1]-plotDim2[0]))
      
      if n_elements(tickValues) gt 1 then begin
        for i = 1,n_elements(tickvalues)-1 do begin
          if dir eq 0 then begin
            gridx = [tickvalues[i],tickvalues[i]]
            gridy = [0D,1D]
            
            ;it looks bad if you draw grid lines on top of frame
            if tickvalues[i] eq xrange[0] || tickvalues[i] eq xrange[1] then begin
              lineshow=0
            endif else begin
              lineshow=1
            endelse
            
          endif else begin
            gridx = [0D,1D]
            gridy = [tickvalues[i],tickvalues[i]]
            
            ;it looks bad if you draw grid lines on top of frame
            if tickvalues[i] eq yrange[0] || tickvalues[i] eq yrange[1] then begin
              lineshow=0
            endif else begin
              lineshow=1
            endelse
          endelse
          
          gridz = [gridzstack,gridzstack]
          
          if lineshow then begin
          
            gridline = obj_new('IDLgrPolyline',$
              gridx,gridy,gridz,alpha=opacity,$
              linestyle=gridstyle,thick=gridthick,$
              color=self->convertColor(gridColor))
              
            gridmodel->add,gridline
          ;model->add,gridline
          endif
        endfor
      endif
    endif
  endif
  
  ;minor grid
  if obj_valid(minorGrid) && obj_isa(minorgrid,'thm_ui_line_style') && ~noMinor then begin
    minorGrid->getProperty,id=gridstyle,$
      color=gridcolor,$
      show=gridshow,$
      thickness=gridthick,$
      opacity=opacity
      
    if gridshow && range[1] ne range[0] then begin
      ;get tick values, (may have been automatically set)
      ;axis1->getProperty,tickValues=tickValues
    
      tickValues = minorTickValues
      
      for i = 0,n_elements(tickvalues)-1 do begin
      
        ;don't draw minor grids on major grids, if majorGrids are drawn
        if n_elements(majorTickValues) gt 0 then begin
          val = min(abs(majorTickValues - tickvalues[i]))
          
          if val lt .01 then continue
          
        endif
        
        if dir eq 0 then begin
        
          gridx = [tickvalues[i],tickvalues[i]]
          gridy = [0D,1D]
          
          ;it looks bad if you draw grid lines on top of frame
          if tickvalues[i] eq xrange[0] || tickvalues[i] eq xrange[1] then begin
            lineshow=0
          endif else begin
            lineshow=1
          endelse
          
        endif else begin
          gridx = [0D,1D]
          gridy = [tickvalues[i],tickvalues[i]]
          
          ;it looks bad if you draw grid lines on top of frame
          if tickvalues[i] eq yrange[0] || tickvalues[i] eq yrange[1] then begin
            lineshow=0
          endif else begin
            lineshow=1
          endelse
          
        endelse
        
        gridz = [gridzstack,gridzstack]
        
        if lineshow then begin
          gridline = obj_new('IDLgrPolyline',$
            gridx,gridy,gridz,alpha=opacity,$
            linestyle=gridstyle,thick=gridthick,$
            xcoord_conv=xconv,ycoord_conv=yconv,$
            color=self->convertColor(gridColor))
            
          gridmodel->add,gridline
        ;model->add,gridline
        endif
      endfor
    endif
  endif
  
  ; axis label

  axisSettings->getProperty,blacklabels=blacklabels
  
  self->addAxisLabels,model,labels,margin,placeAnnotation,dir,orientation,stacklabels,showlabels,self->pt2norm(1.,~dir)/(plotDim1[1]-plotDim1[0]),self->pt2norm(1.,dir)/(plotDim2[1]-plotDim2[0]),blacklabels,labelPos=labelPos
  
  fail = 0
  
end

;This routine actually draws the display, it should be called after
;any update to this function via another call.  It should also be
;called any time that there is damage to the window from opening a
;panel or after an expose_event
pro thm_ui_draw_object::draw,_extra=ex

  compile_opt idl2
  
  if ~obj_isa(self.destination,'IDLgrWindow') then begin
    self.destination->draw,self.scene,_extra=ex
  endif else begin
    self.destination->draw,self.scene,/draw_instance,_extra=ex
  endelse
  
end

;This routine creates an instance of the static components
;of any display, it also leaves only the dynamic components
;unhidden after it is complete, so the draw object is ready 
;for instance based drawing
pro thm_ui_draw_object::createInstance

  compile_opt idl2
  
  ;instancing should only be done on window objects,
  ;but since this method is called at the end of updates,
  ;we need to be certain we don't try to instance a printer or postscript
  if obj_isa(self.destination,'IDLgrWindow') then begin
 ; if 0 then begin
  
    ;hide only the dynamic components of the image
    self->setModelHide,self.staticViews,0
    self->setModelHide,self.dynamicViews,1
    self->setLegendHide
    
    self.destination->draw,self.scene,create_instance=1
    
    ;hide only the static components of the image
    self->setModelHide,self.dynamicViews,0
    self->setModelHide,self.staticViews,1
    self->setLegendHide,/dynamic
 
    self->setCursor,self.cursorLoc
    
   ; self->draw
  
  endif
  
end

;this method will remove the instance hide settings from static components
;This is generally meant to be used by image export routines when the
;the legend is off,  if the legend is on, this method may leave the
;dynamic components of the legend hidden
pro thm_ui_draw_object::removeInstance

  compile_opt idl2
  
  self->setModelHide,self.staticViews,0
  ;hide the components of the legend if they just got turned on
  ;by the previous call
  self->setLegendHide

end

;This routine sets the hide values for all the models inside the views that it receives as an argument
;It just exists to remove some duplication that occurs when generating an instanced display
;This is a recursive function
pro thm_ui_draw_object::setModelHide,input,hideValue

  compile_opt idl2
 
  if obj_isa(input,'IDLgrModel') then begin
    input->setProperty,hide=hideValue
  endif else if obj_isa(input,'IDL_Container') then begin
    list = input->get(/all,count=c)
  
    if c gt 0 then begin
      for i = 0,c-1 do begin
      
        self->setModelHide,list[i],hideValue
      
      endfor
    endif
  endif 
    
end

;Because the static components of the legend are still somewhat
;dynamic(Because they can be manipulated between updates), 
;This routine is needed to manipulate them separately
;From the lists in self.staticViews & self.dynamicViews
pro thm_ui_draw_object::setLegendHide,dynamic=dynamic,hide=hide

 if n_elements(hide) gt 0 then begin
   if keyword_set(hide) then begin
     hideValueDynamic = 1
     hideValueStatic = 1
   endif else begin
     hideValueDynamic = 0
     hideValueStatic = 0
   endelse 
 endif else if self.legendOn eq 2 then begin
   if keyword_set(dynamic) then begin
     hideValueDynamic = 0
     hideValueStatic = 1
   endif else begin
     hideValueDynamic = 1
     hideValueStatic = 0
   endelse
 endif else if self.legendOn eq 1 then begin
   if keyword_set(dynamic) then begin
     hideValueStatic = 0
     hideValueDynamic = 0
   endif else begin
     hideValueDynamic = 1
     hideValueStatic = 1
   endelse
 endif else begin
   hideValueDynamic = 1
   hideValueStatic = 1
 endelse

 if ptr_valid(self.panelInfo) then begin
  
   for i = 0,n_elements(*self.panelInfo)-1 do begin
    
     panel = (*self.panelInfo)[i]
      
     if ~obj_valid(panel.legendModel) then continue
    
  ;   print,i,hidevaluestatic,hidevaluedynamic
    
     panel.legendModel->setProperty,hide=hideValueStatic
     panel.legendAnnoModel->setProperty,hide=hideValueDynamic
    
     self->setVarHide,panel,hideValueDynamic
      
   endfor
    
  endif

end

pro thm_ui_draw_object::setVarHide,panel,hideValue

  if ptr_valid(panel.varInfo) then begin
  
     for j = 0,n_elements(*panel.varInfo)-1 do begin
        var = (*panel.varInfo)[j]
        
        if obj_valid(var.textObj) then begin
          var.textObj->setProperty,hide=hideValue
        endif
     endfor
    
   endif

end

;start drawing a vertical bar on one or all panels

pro thm_ui_draw_object::vBarOn,all=all

  compile_opt idl2
  
  if keyword_set(all) then begin
    self.vBarOn = 2
  endif else begin
    self.vBarOn = 1
  endelse
  
end

;stop drawing a vertical bar
pro thm_ui_draw_object::vBarOff

  compile_opt idl2
  
  self.vBarOn = 0
  
  ;hide vbar
  
  if ptr_valid(self.panelInfo) then begin
  
    for i = 0,n_elements(*self.panelInfo)-1 do begin
    
    
      vBar = ((*self.panelInfo)[i]).vBar
      old = vBar->get(/all)
      if obj_valid(old) then obj_destroy,old
      vBar->remove,/all
      
    endfor
    
  endif
  
  self->draw
  
end

;start drawing the horizontal bar
pro thm_ui_draw_object::hBarOn,all=all

  compile_opt idl2
  
  if keyword_set(all) then begin
    self.hBarOn = 2
  endif else begin
    self.hBarOn = 1
  endelse
  
end

;stop drawing the horizontal bar
pro thm_ui_draw_object::hBarOff

  compile_opt idl2
  
  self.hBarOn = 0
  
  ;hide vbar
  
  if ptr_valid(self.panelInfo) then begin
  
    for i = 0,n_elements(*self.panelInfo)-1 do begin
    
    
      hBar = ((*self.panelInfo)[i]).hBar
      old = hBar->get(/all)
      if obj_valid(old) then obj_destroy,old
      hBar->remove,/all
      
    endfor
    
  endif
  
  self->draw
  
end


;Start drawing a legend on one or more panels
pro thm_ui_draw_object::legendOn,all=all

  compile_opt idl2
  
  if keyword_set(all) then begin
    self.legendOn = 2
  endif else begin
    self.legendOn = 1
  endelse
  
  self->createInstance
  
  self->draw
  
end

;Stop drawing the legend
pro thm_ui_draw_object::legendOff

  compile_opt idl2
  
  self.legendOn = 0
  
  self->createInstance
  
  self->draw
  
end

;starts drawing a new marker at the current location
;'default' is a marker object from which the marker defaults will be copied
;'all' indicates that markers should be drawn on all panels
;note that markerOn can fail if the current location isn't within the
;boundaries of a panel.  The 'fail' keyword indicates this
pro thm_ui_draw_object::markerOn,default=default,all=all,fail=fail

  compile_opt idl2
  
  fail = 1
  
  self.markerStart = self.cursorLoc
  
  if ~ptr_valid(self.panelInfo) then return
  
  panels = *self.panelInfo
  
  inBounds = 0
  
  ;only activate if the start location is in bounds
  for i = 0,n_elements(panels)-1 do begin
  
    if self->inBounds(panels[i]) then begin
      inbounds = 1
      panel = i
    endif
    
  endfor
  
  if ~inBounds then return
  
  if ~keyword_set(all) then begin
    self.markerOn = 1
    
    if ~keyword_set(default) || ~obj_valid(default) then begin
      marker = [obj_new('thm_ui_marker')]
    endif else begin
      marker = [default->copy()]
    endelse
    
    panelIdx = [panel]
  endif else begin
  
    self.markerOn = 2
    marker = objarr(n_elements(*self.panelInfo))
    
    panelIdx = indgen(n_elements(*self.panelInfo))
    
    for i = 0,n_elements(marker)-1 do begin
      if ~keyword_set(default) || ~obj_valid(default) then begin
        marker[i] = obj_new('thm_ui_marker')
      endif else begin
        marker[i] = default->copy()
      endelse
    endfor
    
  endelse
  
  if ptr_valid(self.currentMarkers) then begin
    obj_destroy,*self.currentMarkers
    ptr_free,self.currentMarkers
    ptr_free,self.markerIdx
  endif
  
  self.currentMarkers = ptr_new(marker)
  self.markerIdx = ptr_new(panelIdx)
  
  fail = 0
  
end

;stops drawing the marker
pro thm_ui_draw_object::markerOff

  compile_opt idl2
  
  self.markerOn = 0
  
  if ptr_valid(self.panelInfo) then begin
  
    panels = *self.panelInfo
    
    for i = 0,n_elements(panels)-1 do begin
    
      self->drawMarker,0,1,panels[i],obj_new(),/remove
      
    endfor
    
  endif
  
;hide marker
  
end

;Starts drawing the rubber band at the current location
pro thm_ui_draw_object::rubberBandOn

  compile_opt idl2
  
  self.rubberOn = 1
  self.rubberStart = self.cursorLoc
  
  if ~self->rubberBand(self.rubberStart,[0.,0.]) then begin
    self.statusBar->update,'Error: Problem drawing rubber band'
  ;t=error_message('Problem drawing rubber band',/traceback)
  endif
  
  self->draw
  
end

;Stops drawing the rubber band
pro thm_ui_draw_object::rubberBandOff

  compile_opt idl2
  
  self.rubberOn = 0
  
  if ~self->rubberBand(self.rubberStart,self.cursorLoc - self.rubberStart,/hide) then begin
    self.statusBar->update,'Error: Problem drawing rubber band'
  ; t=error_message('Problem drawing rubber band',/traceback)
  endif
  
  self->draw
  
end

;for making a rubber band.  This thing actually manipulates the
;draw tree to move the rubber band around according to the current cursor position
;location is the location in normalized coordinates
;dimensions is the dimensions in normalized coordinates
function thm_ui_draw_object::rubberBand,location,dimensions,hide=hide

  compile_opt idl2
  
  zstack=.7
  
  if ~obj_valid(self.rubberview) then begin
  
    rubberview = obj_new('IDLgrView')
    self.rubberview = rubberview
    rubberview->setProperty,units=3,viewplane_rect=[0.,0.,1.,1.],location=[0.,0.],dimensions=[1.,1.],zclip=[1.,-1.],eye=5,name="rubberview",/transparent
    self.scene->add,rubberview
    
  endif
  
  old = self.rubberview->get(/all)
  if obj_valid(old) then obj_destroy,old
  self.rubberview->remove,/all
  
  model = obj_new('IDLgrModel')
  
  self.rubberview->add,model
  
  xlocs = [location[0],location[0]+dimensions[0],location[0]+dimensions[0],location[0],location[0]] > 0. < 1.
  ylocs = [location[1],location[1],location[1]+dimensions[1],location[1]+dimensions[1],location[1]] > 0. < 1.
  zlocs = replicate(zstack,5)
  
  rubberband = obj_new('IDLgrPolyline',xlocs,ylocs,zlocs,hide=hide)
  
  model->add,rubberband
  
  return,1
  
end

;Set a couple of properties related to drawing
;Mainly used to perform output to other devices
pro thm_ui_draw_object::setProperty,destination=destination,lineres=lineres,specres=specres

  compile_opt idl2
  
  if keyword_set(destination) then begin
    if ~obj_valid(destination) then begin
      self.statusBar->update,'Error: Invalid destination object passed to thm_ui_draw_object'
      ;t=error_message('Invalid destination object passed to thm_ui_draw_object',/traceback)
      return
    endif else begin
      ;self->removeInstance
      self.destination = destination
      ;self->createInstance
    endelse
  endif
  
  if keyword_set(lineres) then begin
    self.lineres=lineres
  endif
  
  if keyword_set(specres) then begin
    self.specres = specres
  endif
  
end

;Query fun draw object settings
pro thm_ui_draw_object::getProperty, $
    destination=destination, $
    markerOn=markerOn, $
    legendOn=legendOn, $
    rubberOn=rubberOn, $
    vBarOn=vBarOn, $
    hBarOn=hBarOn, $
    pageSize=pageSize,$
    lineres=lineres,$
    specres=specres
    
  compile_opt idl2
  
  destination = self.destination
  markerOn = self.markerOn
  legendOn = self.legendOn
  rubberOn = self.rubberOn
  vBarOn = self.vBarOn
  hBarOn = self.hBarOn
  pageSize = self.currentpageSize
  lineres = self.lineres
  specres = self.specres
  
end

;PRIVATE FUNCTIONS, ONLY FOR INTERNAL USAGE

function thm_ui_draw_object::hueRotation,color

  compile_opt idl2
  
  color_convert,color[0],color[1],color[2],hue,light,sat,/rgb_hls
  hue = (hue + 120) mod 360
  light = 1 - light
  color_convert,hue,light,sat,red,grn,blu,/hls_rgb
  return,[red,grn,blu]
  
end

;this routine converts between True color and indexed color
;depending up which color mode is set by the window
;if keyword: backwards is set, the routine will return a 1x3 array
function thm_ui_draw_object::convertColor,color,backwards=backwards

  compile_opt idl2
  
  self.destination->getProperty,palette=pal,color_model=col
  
  if col eq 1 && n_elements(color) eq 1 then return,color
  
  if col eq 0 && n_elements(color) eq 3 then begin
    if (keyword_set(backwards) && size(color,/n_dim) eq 2) || $
      (~keyword_set(backwards) && size(color,/n_dim) eq 1) then begin
      return,color
    endif
    
    if (keyword_set(backwards) && size(color,/n_dim) eq 1) || $
      (~keyword_set(backwards) && size(color,/n_dim) eq 2) then begin
      return,transpose(color)
    endif
  endif
  
  if ~obj_valid(pal) then begin
    tvlct,r,g,b,/get
    
    pal = obj_new('IDLgrPalette',r,g,b)
    
    self.destination->setProperty,palette=pal
  endif
  
  if n_elements(color) eq 1 then begin
    if keyword_set(backwards) then begin
      return,transpose(pal->getRGB(color))
    endif else begin
      return,pal->getRGB(color)
    endelse
  endif
  
  if n_elements(color) eq 3 then begin
    return,pal->nearestcolor(color[0],color[1],color[2])
  endif
  
  self.statusBar->update,'Error: Invalid color argument passed to thm_ui_draw_object'
  ;t=error_message('Invalid color argument',/traceback)
  return,-1
  
end

;value is the value that need conversion, xy indicates whether this is an x or y element
function thm_ui_draw_object::pt2norm,value,xy

  compile_opt idl2
  
  pt2mm = 127D/360D
  mm2cm = .1D
  
  v = value*pt2mm*mm2cm
  
  dim = self->getDim()
  
  self.destination->getProperty,resolution=r
  
  dim /= self->getZoom()
  
  return,v/(r[xy]*dim[xy])
  
end

;converts back from the normalized value into points,
;While the normalized value is dependent on screen dimensions
;zoom, & resolution.  The value in points should be an
;absolute quantity
function thm_ui_draw_object::norm2pt,value,xy

  compile_opt idl2
  
  pt2mm = 127D/360D
  mm2cm = .1D
  
  dim = self->getDim()
  
  self.destination->getProperty,resolution=r
  
  dim /= self->getZoom()
  
  v = value*r[xy]*dim[xy]
  
  return,v/(pt2mm*mm2cm)
  
end

;calculates the panel size for the purpose of spectral plot generation
function thm_ui_draw_object::getSpecSize,plotdim1,plotdim2

  compile_opt idl2
  
  res_factor = self.specres ;this factor increases the resolution of the spectral plot above the visible resolution
  ;this way zooming out will provide extra detail. (If aliasing becomes a problem it might
  ;be worth increasing the value to 4
  
  dim = self->getDim()
  
  dim /= self->getZoom()
  
  if max(dim,sub) gt self.pointmax then begin
    if sub eq 0 then begin
      dim[1] = ceil(self.pointmax*dim[1]/dim[0])
      dim[0] = self.pointmax
    endif else begin
      dim[0] = ceil(self.pointmax*dim[0]/dim[1])
      dim[1] = self.pointmax
    endelse
  endif
  
  xpt = dim[0] * res_factor
  ypt = dim[1] * res_factor
  
  ;size of the panel in points
  xpanel_sz_pt = ceil(xpt * (plotdim1[1]-plotdim1[0]))
  ypanel_sz_pt = ceil(ypt * (plotdim2[1]-plotdim2[0]))
  
  return,[xpanel_sz_pt,ypanel_sz_pt]
  
end

function thm_ui_draw_object::getLineSize,plotdim1

  compile_opt idl2
  
  res_factor = self.lineres ;this factor increases the resolution of the line plot above the visible resolution
  ;this way zooming out will provide extra detail. (If aliasing becomes a problem it might
  ;be worth increasing the value to 4
  
  ;calculate number of pixels in view to generate quick reference for legend
  dim = self->getdim()
  
  dim /= self->getZoom()
  
  if dim[0] gt self.pointmax then begin
    xpx = self.pointmax
  endif
  
  ;this factor will convert normalized window coords into normalized panel coords
  xpanelConv = plotdim1[1] - plotdim1[0]
  
  ;number of pixels across the panel * res_factor = number of points in lookup
  xpx = (dim[0] * res_factor)*xPanelConv
  
  return,xpx
  
end

function thm_ui_draw_object::getZoom

  compile_opt idl2
  
  if obj_isa(self.destination,'IDLgrWindow') then begin
    self.destination->getProperty,current_zoom=cz
  endif else begin
    cz = 1D
  endelse
  
  return, cz
  
end

;Set the zoom factor on the current destination object
;Only works if the destination is an IDLgrWindow
pro thm_ui_draw_object::setZoom,zoom

  compile_opt idl2
  
  if obj_isa(self.destination,'IDLgrWindow') then begin
    ;I suspect that there is a bug(feature?) in IDL's recompute_dimensions routine
    ;This is causing the routine to incorrectly recalculate dimensions of some of the
    ;IDLgrText based upon the current hide value of the model in which it resides.
    ;This may also be based upon whether the text has been drawn yet.    
    self->setLegendHide,hide=0
    self->draw
 
    self->setLegendHide,hide=1
    self.destination->setCurrentZoom,zoom
    self->createInstance
    
  ;  if self.legendOn eq 1 then begin
  ;  self->setLegendHide,hide=0
 ;   self->draw
   ; endif
    
    self->setCursor,self.cursorLoc
    
  endif
  
end

;This routine should blank the current contents of the object
;in the event of an error
pro thm_ui_draw_object::nukeDraw

  compile_opt idl2
  
  drawTree = self.scene->get(/all)
  self.scene->remove,/all
  
  obj_destroy,drawTree
  
  ptr_free,self.panelInfo
  
  heap_gc
  
end

;gets dimensions in pixels,
;abstracts some unit fuss
function thm_ui_draw_object::getDim

  compile_opt idl2
  
  self.destination->getProperty,units=un
  self.destination->setProperty,units=0
  self.destination->getProperty,dimensions=dim
  self.destination->setProperty,units=un
  
  return,float(dim)
  
end

;create a new object, the destination object is the
;window created by widget_draw(graphics_level=2)
function thm_ui_draw_object::Init,destination,statusbar,historyWin

  compile_opt idl2
  
  if ~obj_valid(destination) then begin
    t=error_message('Invalid destination object passed to thm_ui_draw_object',/traceback)
    return,0
  endif
  
  if ~obj_valid(Statusbar) then begin
    t = error_message('Expected valid status bar',/traceback)
    return,0
  endif
  
  if ~obj_valid(historyWin) then begin
    t = error_message('Expected valid history Window',/traceback)
    return,0
  endif
  
  self.destination = destination
  self.statusbar = statusbar
  self.historyWin = historyWin
  
  self.scene = OBJ_NEW('IDLgrScene',color=[255,255,255])
  
  rubberview = obj_new('IDLgrView')
  self.rubberview = rubberview
  rubberview->setProperty,units=3,viewplane_rect=[0.,0.,1.,1.],location=[0.,0.],dimensions=[1.,1.],zclip=[1.,-1.],eye=5,name="rubberview",/transparent
  self.scene->add,rubberview
  
  self.staticViews = obj_new('IDL_Container')
  self.dynamicViews = obj_new('IDL_Container')
   
  self.lineres = 4.0D
  self.specres = 1.0D
  self.pointmax = 4096
  
  return,1
  
end



pro thm_ui_draw_object__define

  struct = { THM_UI_DRAW_OBJECT, $
    destination:OBJ_NEW(), $ ; The output object
    scene:OBJ_NEW(), $ ; The current scene(private)
    pageview:OBJ_NEW(), $  ; The view for the page includes the title
    panelViews:obj_new(),$ ; view group of panel views
    rubberview:OBJ_NEW(), $  ; The rubber band view
    rubberstart:dblarr(2), $ ; The starting location of a rubberband box
    markerstart:dblarr(2), $  ; The starting location of a marker draw
    cursorloc:dblarr(2), $  ; The current location of the cursor
    markerOn:0, $ ; boolean value indicating whether to draw a marker 0: no marker, 1: single panel marker, 2: all panel markers
    rubberOn:0, $ ; boolean value indicating whether to draw a rubber band
    vBarOn:0, $  ;value indicating whether to draw a vertical bar, 0 = no bar, 1 = single panel bar, 2 = all panel bar
    hBarOn:0, $  ;value indicating whether to draw a horizontal bar, 0 = no bar, 1 = single panel bar, 2 = all panel bar
    legendOn:0, $  ;value indicating whether to draw a legend, 0 = no legend, 1 = single panel legend, 2 = all panel legend
    nRows:1, $ ;number of rows in current display
    nCols:1, $ ;number of columns in current display
    panelInfo:ptr_new(), $ ;an array of structures that describe information in panels that is relevant to drawing annotations
    currentPageSize:[8.5,11], $ ; the current size of the page in inches
    currentMarkers:ptr_new(), $ ;a list of the currently/most recently drawn markers
    markerIdx:ptr_new(), $ ; a list of the marker panel indexes
    statusBar:obj_new(), $  ; reference to the status bar from the main window, for error output
    historyWin:obj_new(), $ ; reference to the history window
    postscript:0B, $  ; indicates whether or not a vector postscript is being generated
    lineres:2D, $ ; a multiplier for line plots to determine how much it is up or down sampled prior to display
    specres:1D, $ ; a multiplier for spectral plots to determine how they are up or down sampled prior to display
    pointmax:4096, $ ; the maximum resolution at which a screen render should be made
    staticViews:obj_new(), $ ; This view group contains all visual components that change only during updates
    dynamicViews:obj_new()  $ ; This view group contains all visual components that change between updates and during updates
    }
;margins:dblarr(5) } ; The [left,right,top,bottom,internal] margins in normalized coordinates
    
end


